package de.joerg.diary.backend.tools;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import javax.crypto.Cipher;
import javax.crypto.SecretKey;
import javax.crypto.spec.SecretKeySpec;
import com.sun.org.apache.xml.internal.security.utils.Base64;
import de.joerg.diary.shared.IRessources;

/*
 *  SimpleDiary Version 2.0.0
    Copyright (C) 2010  Andreas Joerg (E-Mail: simpleDiary@freenet.de)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Klasse um einen String mit AES in Abhaengikeit eines Keys zu crypten bzw. zu
 * encrypten.
 * 
 * @author Andreas Joerg
 */
public class Crypting 
{
	/**
	 * Gibt den uebergebenen String (uncrypted) verschluesselt zurueck
	 * 
	 * @param uncrypted
	 * @param key
	 * @return crypted
	 * @throws Exception
	 */
	public static String crypt(String uncrypted, String key) throws Exception
	{
		if(uncrypted == null)
		{
			//Der zu cryptende String darf nicht null sein
			throw new NullPointerException(IRessources.res.getString("0156"));
		}
		if(key == null)
		{
			//Key darf nicht null sein
			throw new NullPointerException(IRessources.res.getString("0157"));
		}
		
		Cipher cipher = Cipher.getInstance("AES");         
		cipher.init(Cipher.ENCRYPT_MODE, getSecretKey(key));    
		return new String(Base64.encode(cipher.doFinal(uncrypted.getBytes("UTF8"))));
	}
	
	/**
	 * Gibt den uebergebenen String (crypted) entschluesselt zurueck
	 * 
	 * @param crypted
	 * @param key
	 * @return uncrypted
	 * @throws Exception
	 */
	public static String decrypt(String crypted, String key) throws Exception
	{
		if(crypted == null)
		{
			//Der zu entcryptende String darf darf nicht null sein
			throw new NullPointerException(IRessources.res.getString("0158"));
		}
		if(key == null)
		{
			//Key darf nicht null sein
			throw new NullPointerException(IRessources.res.getString("0159"));
		}
		
		Cipher cipher = Cipher.getInstance("AES");     
		cipher.init(Cipher.DECRYPT_MODE, getSecretKey(key));     
		return new String(cipher.doFinal(Base64.decode(crypted.getBytes("UTF8"))));
	}
	
	/**
	 * Gibt einen AES SecretKey zurueck der aus dem uebergebenem String (key) gebildet wird.
	 * 
	 * @param key
	 * @return new SecretKeySpec(passwordData, "AES");
	 * @throws NoSuchAlgorithmException
	 */
	private static SecretKey getSecretKey(String key) throws NoSuchAlgorithmException
	{
		MessageDigest digester = MessageDigest.getInstance("MD5");
		char[] password = key.toCharArray();
		
		for (int i = 0; i < password.length; i++) 
		{
			digester.update((byte) password[i]);
		}
		
		byte[] passwordData = digester.digest();

		return new SecretKeySpec(passwordData, "AES");
	}
	
	/**
	 * Gibt den uebergebenen String entschluesselt zurueck.
	 * ACHTUNG: ALTE VERSION, UNSICHERE VERSCHLUESSELUNG!!!!
	 * ##########DEPRECATED#############
	 * 
	 * @param crypted
	 * @param key
	 * @return uncrypted
	 * @throws Exception
	 */
	@Deprecated
	public static String decryptOld(String crypted, String key) throws Exception
	{
		if(crypted == null)
		{
			//Der zu entcryptende String darf darf nicht null sein
			throw new NullPointerException(IRessources.res.getString("0158"));
		}
		if(key == null)
		{
			//Key darf nicht null sein
			throw new NullPointerException(IRessources.res.getString("0159"));
		}
		
		int cryptValue = 0;
		String uncrypted = IRessources.EMPTY;
		
		for (int x = 0;x<key.length();x++)
		{
			cryptValue += (int)key.charAt(x);
		}
		
		while(cryptValue > 255)
		{
			cryptValue /= 2;
		}
		
		for(int x = 0; x<crypted.length();x++)
		{
			if((int)crypted.charAt(x)-cryptValue < 0)
			{
				uncrypted += (char)((int)crypted.charAt(x) - cryptValue + 255);
			}
			else
			{
				uncrypted += (char)((int)crypted.charAt(x) - cryptValue);
			}
		}
		
		uncrypted = uncrypted.replaceAll("XCode", "X");
		uncrypted = uncrypted.replaceAll("XNew", "\n");
		
		return uncrypted;
	}
}
