package de.joerg.diary.backend.tools;

import java.util.ArrayList;
import de.joerg.diary.shared.IRessources;
import de.joerg.diary.shared.models.DiaryEvent;
import de.joerg.diary.shared.models.SDPicture;
import de.joerg.diary.shared.models.User;

/*
 *  SimpleDiary Version 2.0.0
    Copyright (C) 2010  Andreas Joerg (E-Mail: simpleDiary@freenet.de)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Klasse um UTF-8 Validierungen durchzufuehren und um Sonderzeichen gegen spezielle Codes zu ersetzen
 * damit Strings mit Sonderzeichen ebenfalls UTF-8 konform sind.
 * 
 * @author Andreas Joerg
 */
public class UTF8Helper 
{
	/**
	 * Prueft den uebergebenen String auf UTF-8 Konformheit.
	 * Ist ein Zeichen nicht erlaubt, wird es zurckgegeben ansonsten "OK".
	 * 
	 * @param str
	 * @return "OK"/invalidChar
	 */
	public static String isStringUTF8Valid(String str)
	{	
		for(int x=0;x<str.length();x++)
		{
			int intValue = str.charAt(x);
			
			if(!((intValue>=32&&intValue<=63)||(intValue>=65&&intValue<=126)||
					intValue==176||(intValue>=178&&intValue<=180)||(intValue>=9&&intValue<=10)))
			{
				return String.valueOf(str.charAt(x));
			}
		}
		
		//return "OK"
		return IRessources.res.getString("0010");
	}
	
	/**
	 * Methode um alle Sonderzeichen in den Parametern eines uebergebenen User Objects zu
	 * ersetzen gegen UTF-8 Konforme Codes.
	 * 
	 * @param user
	 */
	public static void replaceSpecialSignsInUserProps(User user)
	{
		//Speicherpfad
		if(user.getSavePath()!=null&&!user.getSavePath().isEmpty())
		{
			user.setSavePath(replaceSpecialSignsInString(user.getSavePath()));
		}
		//UserName
		if(user.getUserName()!=null&&!user.getUserName().isEmpty())
		{
			user.setUserName(replaceSpecialSignsInString(user.getUserName()));
		}
		//Passwort
		if(user.getUserPW()!=null&&!user.getUserPW().isEmpty())
		{
			user.setUserPW(replaceSpecialSignsInString(user.getUserPW()));
		}
	}
	
	/**
	 * Methode um alle Sonderzeichen in den Parametern eines uebergebenen DiaryEvent Objects zu
	 * ersetzen gehen UTF-8 Konforme Codes.
	 * 
	 * @param diaryEvt
	 */
	public static void replaceSpecialSignsInDiaryEventProps(DiaryEvent diaryEvt)
	{
		//Beschreibung
		if(diaryEvt.getDescription()!=null&&!diaryEvt.getDescription().isEmpty())
		{
			diaryEvt.setDescription(replaceSpecialSignsInString(diaryEvt.getDescription()));
		}
		//Ort
		if(diaryEvt.getPlace()!=null&&!diaryEvt.getPlace().isEmpty())
		{
			diaryEvt.setPlace(replaceSpecialSignsInString(diaryEvt.getPlace()));
		}
		//Zeit bis
		if(diaryEvt.getTimeEnd()!=null&&!diaryEvt.getTimeEnd().isEmpty())
		{
			diaryEvt.setTimeEnd(replaceSpecialSignsInString(diaryEvt.getTimeEnd()));
		}
		//Zeit von
		if(diaryEvt.getTimeStart()!=null&&!diaryEvt.getTimeStart().isEmpty())
		{
			diaryEvt.setTimeStart(replaceSpecialSignsInString(diaryEvt.getTimeStart()));
		}
		//User
		if(diaryEvt.getUser()!=null)
		{
			replaceSpecialSignsInUserProps(diaryEvt.getUser());
		}
		
		//Bildbeschreibungen
		ArrayList<SDPicture> imgs = diaryEvt.getPictures();
		if(imgs != null)
		{
			for(int x=0;x<imgs.size();x++)
			{
				if(imgs.get(x).getPictureDesc()!=null)
				{
					imgs.get(x).setPictureDesc(replaceSpecialSignsInString(imgs.get(x).getPictureDesc()));
				}
			}
		}
		
		//Stichpunkte
		String[] keyWords = diaryEvt.getKeywords();
		if(keyWords!=null)
		{
			for(int x=0;x<keyWords.length;x++)
			{
				if(keyWords[x]!=null)
				{
					keyWords[x] = replaceSpecialSignsInString(keyWords[x]);
				}
			}
		}
		
		//Rekursiver Aufruf der Methode fuer jedes SubEvent
		if(diaryEvt.getSubEvents()!=null&&diaryEvt.getSubEvents().size()!=0)
		{
			for(int x=0;x<diaryEvt.getSubEvents().size();x++)
			{
				replaceSpecialSignsInDiaryEventProps(diaryEvt.getSubEvent(x));
			}
		}
	}
	
	/**
	 * Methode um alle Sonderzeichen in einem uebergebenem String durch UTF-8 konforme Codes 
	 * zu ersetzen. 
	 * Ersetzt werden folgende Zeichen:
	 * 
	 * ae, Ae, oe, Oe, ue, Ue, ss(=scharfes s), [at], Euro-Zeichen
	 * 
	 * @param str
	 * @return str
	 */
	public static String replaceSpecialSignsInString(String str)
	{
		str=str.replaceAll("X","XCode");
		
		//ae
		str=str.replaceAll("\u00E4", "Xu00E4");
		//Ae
		str=str.replaceAll("\u00C4", "Xu00C4");
		
		//oe
		str=str.replaceAll("\u00F6", "Xu00F6");
		//Oe
		str=str.replaceAll("\u00D6", "Xu00D6");
		
		//ue
		str=str.replaceAll("\u00FC", "Xu00FC");
		//Ue
		str=str.replaceAll("\u00DC", "Xu00DC");
		
		//scharfes s
		str=str.replaceAll("\u00DF", "Xu00DF");
		
		//@
		str=str.replaceAll("@", "XAT");
		
		//Euro Zeichen
		str=str.replaceAll("", "XEuro");
		
		//Paragraph Zeichen
		str=str.replaceAll("", "XPara");
		
		//Tab
		str=str.replaceAll("\t", "XTab");
		
		return str;
	}
	
	/**
	 * Stellt die urspruenglichen Werte der Parameter eines DiaryEvent Objects wieder her.
	 * D.h. die UTF-8 konformen Codes werden wieder durch entsprechende Sonderzeichen ersetzt.
	 * 
	 * @param diaryEvt
	 */
	public static void encodeSpecialSignsInDiaryEventProps(DiaryEvent diaryEvt)
	{
		//Beschreibung
		if(diaryEvt.getDescription()!=null&&!diaryEvt.getDescription().isEmpty())
		{
			diaryEvt.setDescription(encodeSpecialSignsInString(diaryEvt.getDescription()));
		}
		//Ort
		if(diaryEvt.getPlace()!=null&&!diaryEvt.getPlace().isEmpty())
		{
			diaryEvt.setPlace(encodeSpecialSignsInString(diaryEvt.getPlace()));
		}
		//Zeit bis
		if(diaryEvt.getTimeEnd()!=null&&!diaryEvt.getTimeEnd().isEmpty())
		{
			diaryEvt.setTimeEnd(encodeSpecialSignsInString(diaryEvt.getTimeEnd()));
		}
		//Zeit von
		if(diaryEvt.getTimeStart()!=null&&!diaryEvt.getTimeStart().isEmpty())
		{
			diaryEvt.setTimeStart(encodeSpecialSignsInString(diaryEvt.getTimeStart()));
		}
		//User
		if(diaryEvt.getUser()!=null)
		{
			encodeSpecialSignsInUserProps(diaryEvt.getUser());
		}
		
		//Rekursiver Aufruf der Methode fuer jedes SubEvent
		if(diaryEvt.getSubEvents()!=null&&diaryEvt.getSubEvents().size()!=0)
		{
			for(int x=0;x<diaryEvt.getSubEvents().size();x++)
			{
				encodeSpecialSignsInDiaryEventProps(diaryEvt.getSubEvent(x));
			}
		}
	}
	
	/**
	 * Stellt die urspruenglichen Werte der Parameter eines Users Objects wieder her.
	 * D.h. die UTF-8 konformen Codes werden wieder durch entsprechende Sonderzeichen ersetzt.
	 * 
	 * @param user
	 */
	public static void encodeSpecialSignsInUserProps(User user)
	{
		//Speicherpfad
		if(user.getSavePath()!=null&&!user.getSavePath().isEmpty())
		{
			user.setSavePath(encodeSpecialSignsInString(user.getSavePath()));
		}
		//UserName
		if(user.getUserName()!=null&&!user.getUserName().isEmpty())
		{
			user.setUserName(encodeSpecialSignsInString(user.getUserName()));
		}
		//Passwort
		if(user.getUserPW()!=null&&!user.getUserPW().isEmpty())
		{
			user.setUserPW(encodeSpecialSignsInString(user.getUserPW()));
		}
	}
	
	/**
	 * Ersetzt die UTF-8 konformen Codes im uebergebenem String wieder durch Sonderzeichen.
	 * 
	 * @param str
	 * @return str
	 */
	public static String encodeSpecialSignsInString(String str)
	{
		str=str.replaceAll("XCode","X");
		
		//ae
		str=str.replaceAll("Xu00E4", "\u00E4");
		//Ae
		str=str.replaceAll("Xu00C4", "\u00C4");
		
		//oe
		str=str.replaceAll("Xu00F6", "\u00F6");
		//Oe
		str=str.replaceAll("Xu00D6", "\u00D6");
		
		//ue
		str=str.replaceAll("Xu00FC", "\u00FC");
		//Ue
		str=str.replaceAll("Xu00DC", "\u00DC");
		
		//scharfes s
		str=str.replaceAll("Xu00DF", "\u00DF");
		
		//@
		str=str.replaceAll("XAT", "@");
		
		//Euro Zeichen
		str=str.replaceAll("XEuro", "");
		
		//Paragraph Zeichen
		str=str.replaceAll("XPara", "");
		
		//Tab
		str=str.replaceAll("XTab", "\t");
		
		return str;
	}
}
