package de.joerg.diary.backend.userManagement;

import java.beans.XMLDecoder;
import java.beans.XMLEncoder;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import de.joerg.diary.backend.exceptions.FileForThisUserInvalidException;
import de.joerg.diary.backend.exceptions.FileInvalidException;
import de.joerg.diary.backend.fileManagement.DiaryEventFileManager;
import de.joerg.diary.backend.tools.Crypting;
import de.joerg.diary.backend.tools.SDServices;
import de.joerg.diary.backend.tools.UTF8Helper;
import de.joerg.diary.shared.DiaryLogger;
import de.joerg.diary.shared.IRessources;
import de.joerg.diary.shared.models.DiaryEvent;
import de.joerg.diary.shared.models.User;

/*
 *  SimpleDiary Version 2.0.0
    Copyright (C) 2010  Andreas Joerg (E-Mail: simpleDiary@freenet.de)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *  Klasse zur Realisierung der Zugriffe auf die UserAccounts und Organisation dieser.
 * 
 * @author Andreas Joerg
 */
public class UserAccountsManager 
{
	private DiaryEventFileManager diaryEvtFileMgr = new DiaryEventFileManager();
	private static String FILE_USERDATA = System.getProperty("user.dir")+"\\"+"defuaad.def";
	
	/**
	 * Methode zur Aenderung des Passwortes eines Users.
	 * 
	 * @param user
	 * @param newPW
	 * @param savePath
	 * @throws Exception 
	 */
	public void changeUserPassword(User user, String newPW, String savePath) throws Exception
	{		
		//Laden der User aus der Userdatei
		ArrayList<User> users = getUsers();
		User tempUser = null;
		
		//altesUserPW merken
		String oldPW = user.getUserPW();
		
		if(users==null)
		{
			//Userdatei nicht vorhanden oder nicht lesbar.
			DiaryLogger errMsg = new DiaryLogger(IRessources.res.getString("0160")
					+getClass().getName()+" <changeUserSavePath>");
			
			errMsg.start();
			
			throw new Exception(IRessources.res.getString("0161"));
		}
		//Speicherung der Referenz des gesuchten Users in tempUser
		for (User existingUser : users) 
		{
			if(existingUser.equals(user))
			{
				tempUser = existingUser;
				break;
			}
		}
		if(tempUser==null)
		{
			//User nicht vorhanden
			DiaryLogger logger = new DiaryLogger(IRessources.res.getString("0162")+
					getClass().getName()+" <changeUserPassword>");
			
			logger.start();
			
			throw new Exception(IRessources.res.getString("0163"));
		}
		
		//Entfernen des Users aus der Userlist, neusetzen des PW und wieder hinzufuegen des Users
		users.remove(tempUser);
		tempUser.setUserPW(newPW);
		users.add(tempUser);

		//UserAccount Datei neu schreiben
		writeUserFile(users);
		
		try
		{
			//Speicherung der DiaryEvents des Users in files
			ArrayList<String> files = diaryEvtFileMgr.getListOfEventFiles(user, savePath);
			
			for (String file : files) 
			{
				//Wenn FileName gueltig ist
				if(diaryEvtFileMgr.checkFileName(file))
				{
					if(file.startsWith(String.valueOf(user.getUserID())+"_"))
					{
						try
						{
							//wird benoetigt da das PW am Ende dieser schleife immer auf das neue gesetzt wird.
							//Fuer jeden weiteren alten Eintrag muss im user Objekt allerdings noch das alte
							//Passwort stehen um die Datei richtig entschluesseln zu koennen.
							//S. Mantis 126
							user.setUserPW(oldPW);
							
							DiaryEvent diaryEvent = diaryEvtFileMgr.loadDiaryEventFromFile(savePath, file, user);
							
							if(diaryEvent.getUser().equals(user))
							{
								//neusetzen des PW in DiaryEvent
								diaryEvent.getUser().setUserPW(newPW);
							}
							
							//Wenn SubEvents vorhanden sind -> Ebenfalls neu setzen deren Passwoerter
							if(diaryEvent.getSubEvents()!=null&&diaryEvent.getSubEvents().size()!=0)
							{
								for(int x=0;x<diaryEvent.getSubEvents().size();x++)
								{
									if(diaryEvent.getSubEvent(x).getUser().equals(user))
									{
										//neusetzen des PW in DiaryEvent
										diaryEvent.getSubEvent(x).getUser().setUserPW(newPW);
									}
								}
							}
							
							//Neuspeichern des DiaryEvents
							user.setUserPW(newPW);
							diaryEvtFileMgr.writeDiaryEventToFile(diaryEvent, file, user, savePath);
						}
						catch(FileForThisUserInvalidException ex)
						{
							diaryEvtFileMgr.markFileAsInvalid(savePath, file);
							throw ex;
						}
						catch(FileInvalidException ex)
						{
							diaryEvtFileMgr.markFileAsInvalid(savePath, file);
							throw ex;
						}
					}
				}
			}
		}
		catch(Exception ex)
		{
			//Speicherpfad ungueltig -> Nur loggen, da PW trotzdem geaendert werden kann.
			DiaryLogger log = new DiaryLogger(ex);
			log.start();
		}
		
		
	}
	
	/**
	 * Methode zur Aenderung des Speicherpfades eines Users
	 * 
	 * @param user
	 * @param savePath
	 * @throws Exception
	 */
	public void changeUserSavePath(User user,String savePath) throws Exception
	{
		//Laden der User aus der Userdatei
		ArrayList<User> users = getUsers();
		User tempUser = null;
		
		if(users==null)
		{
			//Userdatei nicht vorhanden oder nicht lesbar.
			DiaryLogger errMsg = new DiaryLogger(IRessources.res.getString("0160")
					+getClass().getName()+" <changeUserSavePath>");
			
			errMsg.start();
			
			throw new Exception(IRessources.res.getString("0161"));
		}
		//Speicherung der Referenz des gesuchten Users in tempUser
		for (User existingUser : users) 
		{
			if(existingUser.equals(user))
			{
				tempUser = existingUser;
			}
		}
		if(tempUser==null)
		{
			//User nicht vorhanden
			DiaryLogger errMsg = new DiaryLogger(IRessources.res.getString("0162")+
					getClass().getName()+" <changeUserSavePath>");
			
			errMsg.start();
			
			throw new Exception(IRessources.res.getString("0163"));
		}
		
		//Entfernen des Users aus der Userlist, neusetzen des PW und wieder hinzufuegen des Users
		users.remove(tempUser);
		tempUser.setSavePath(savePath);
		users.add(tempUser);
		
		//UserAccount Datei neu schreiben
		writeUserFile(users);
		
		//neuen Speicherpfad in UserReferenz schreiben
		user.setSavePath(savePath);
	}
	
	/**
	 * Gibt einen User aus der UserAccounts Datei zurueck.
	 * Bei Rueckgabe null -> User nicht vorhanden.
	 * 
	 * @param user
	 * @return existingUser/null
	 * @throws Exception 
	 */
	public User getUser(User user) throws Exception
	{
		ArrayList<User> users = getUsers();
		
		if(users!=null)
		{
			for (User existingUser : users) 
			{
				if(existingUser.equals(user))
				{
					return existingUser;
				}
			}
		}
		else
		{
			//Noch kein User angelegt
			DiaryLogger errMsg = new DiaryLogger(IRessources.res.getString("0164")
					+getClass().getName()+" <getUser>");
			
			errMsg.start();
			
			throw new Exception (IRessources.res.getString("0165"));
		}
		
		return null;
	}
	
	@SuppressWarnings("unchecked")
	/**
	 * Gibt die User aus der UserAccount Datei zurueck.
	 * 
	 * @return (ArrayList<User>) xml.readObject()
	 */
	private ArrayList<User> getUsers() throws Exception
	{
		try 
		{
			boolean hasUnsecureCrypting = false;
			ArrayList<User> users = new ArrayList<User>();
			
			//-------Liest den Inhalt der Users Datei in den String content ein--------------
			File userAccount = new File(FILE_USERDATA);
			FileReader reader = new FileReader(userAccount);
			String content = null;
			char[] buffer = new char[(int)userAccount.length()];
			
			try 
			{
				reader.read(buffer);
				reader.close();
			} 
			catch (IOException ex) 
			{
				//Fehler bei Zugriff auf UserAccountsDatei
				DiaryLogger errMsg = new DiaryLogger(IRessources.res.getString("0166"));
				errMsg.start();
				
				DiaryLogger errEx = new DiaryLogger(ex);
				errEx.start();
				
				throw new Exception(IRessources.res.getString("0166"),ex);
			}
		
			content = new String(buffer);
			//--------------------------------------------------------------------------------
			
			try
			{
				//Entschluesselt content
				content = Crypting.decrypt(content, "a8g45%)YS%%5412457651hdz454262 %XY");
			}
			//Wenn bei der Entschluesselung ein Fehler aufgetreten ist
			//->moeglicher Grund: Datei wurde noch mit altem Algorithmus verschluesselt
			//	-> Aufruf der Methode Crypting.decryptOld()
			catch(Exception ex)
			{
				//Datei konnte nicht decrypted werden. Try Crypto V1.0.0.1
				DiaryLogger errMsg = new DiaryLogger(IRessources.res.getString("0138")+
						FILE_USERDATA+IRessources.res.getString("0139"));
				
				errMsg.start();
				
				content = Crypting.decryptOld(content, "a8g45%)YS%%5412457651hdz454262 %XY");
				hasUnsecureCrypting = true;
			}
			
			//Einlesen der Userliste mittels XML aus dem String content
			XMLDecoder xml = new XMLDecoder(new ByteArrayInputStream(content.getBytes()));
			users = (ArrayList<User>) xml.readObject();
			xml.close();
			
			//Wenn die Datei noch mit altem Algorithmus verschluesselt ist
			//-> Aufruf der Methode writeUserFile(), bewirkt, dass die Datei neu geschrieben wird
			//	 MIT AES Crypting.
			if(hasUnsecureCrypting)
			{
				writeUserFile(users);
			}
			
			//Stellt in den Properties aller User in der Liste die Sonderzeichen wieder her, die vor der
			//Speicherung gecrypted wurden.
			for(int x=0;x<users.size();x++)
			{
				UTF8Helper.encodeSpecialSignsInUserProps(users.get(x));
			}
			
			return users;
		} 
		catch(NullPointerException ex)
		{
			DiaryLogger errLog = new DiaryLogger(ex);
			errLog.start();
			
			//Userdatei ungueltig
			throw new NullPointerException(IRessources.res.getString("0188"));
		}
		catch (FileNotFoundException ex) 
		{
			return null;
		}
	}
	
	/**
	 * Schreiben des UserAccount Files mit uebergebener UserListe auf das FileSystem.
	 * 
	 * @param users
	 * @throws Exception 
	 */
	private void writeUserFile(ArrayList<User> users) throws Exception
	{
		for(int x=0;x<users.size();x++)
		{
			UTF8Helper.replaceSpecialSignsInUserProps(users.get(x));
			
			String responeOfIsStringUTF8Valid = UTF8Helper.isStringUTF8Valid(users.get(x).getSavePath());
			if(!responeOfIsStringUTF8Valid.equals(IRessources.res.getString("0010")))
			{
				//Ungueltiges Zeichen in Speicherpfad
				DiaryLogger errMsg = new DiaryLogger(SDServices.getPreparedMessage(
						IRessources.res.getString("0133")+responeOfIsStringUTF8Valid,
						getClass().getName(), "writeUserFile"));
				
				errMsg.start();
				
				throw new Exception(IRessources.res.getString("0133")+responeOfIsStringUTF8Valid+
						IRessources.res.getString("0127"));
			}
			
			responeOfIsStringUTF8Valid = UTF8Helper.isStringUTF8Valid(users.get(x).getUserName());
			if(!responeOfIsStringUTF8Valid.equals(IRessources.res.getString("0010")))
			{
				//Ungueltiges Zeichen in Username
				DiaryLogger errMsg = new DiaryLogger(SDServices.getPreparedMessage(
						IRessources.res.getString("0132")+responeOfIsStringUTF8Valid,
						getClass().getName(), "writeUserFile"));
				
				errMsg.start();
				
				throw new Exception(IRessources.res.getString("0132")+responeOfIsStringUTF8Valid+
						IRessources.res.getString("0127"));
			}
			
			responeOfIsStringUTF8Valid = UTF8Helper.isStringUTF8Valid(users.get(x).getUserPW());
			if(!responeOfIsStringUTF8Valid.equals(IRessources.res.getString("0010")))
			{
				//Ungueltiges Zeichen in PW
				DiaryLogger errMsg = new DiaryLogger(SDServices.getPreparedMessage(
						IRessources.res.getString("0131")+responeOfIsStringUTF8Valid,
						getClass().getName(), "writeUserFile"));
				
				errMsg.start();
				
				throw new Exception(IRessources.res.getString("0131")+responeOfIsStringUTF8Valid+
						IRessources.res.getString("0127"));
			}
		}
		
		//XML Speicherung: Integer (users.size()), ArrayList<User> (users)
		ByteArrayOutputStream xmlToByteArray = new ByteArrayOutputStream(); 
		XMLEncoder xml = new XMLEncoder(xmlToByteArray);
		String content = null;
		
		xml.writeObject(users);
		xml.close();
		users = null;
		
		content = xmlToByteArray.toString();
		
		content = Crypting.crypt(content, "a8g45%)YS%%5412457651hdz454262 %XY");
		
		File userAccount = new File(FILE_USERDATA);
		FileWriter fileWriter;
		
		try 
		{
			fileWriter = new FileWriter(userAccount);
			fileWriter.write(content);
			fileWriter.close();	
		} 
		catch (IOException ex) 
		{
			//Fehler bei IO UserAcc File
			DiaryLogger errMsg = new DiaryLogger(IRessources.res.getString("0166"));
			errMsg.start();
			
			DiaryLogger errEx = new DiaryLogger(ex);
			errEx.start();
			
			throw new Exception(IRessources.res.getString("0166"));
		}
	}
	
	/**
	 * Fuegt einen User zur UserAccount Liste hinzu.
	 * 
	 * @param user
	 * @throws Exception
	 */
	public void addUser(User user) throws Exception
	{	
		ArrayList<User> users = null;
		try
		{
			users = getUsers();
		}
		catch(Exception ex)
		{
			if(ex.getCause() != null && ex.getCause() instanceof FileNotFoundException)
			{
				//Nichts machen, kann passieren
			}
			else
			{
				throw ex;
			}
		}
		 
		if(users == null)
		{
			users = new ArrayList<User>();
			user.setUserID(1000);
			users.add(user);
		}
		else
		{
			int freeID = 999;
			boolean found = false;
			
			for (User usr : users) 
			{
				if(usr.getUserName().equals(user.getUserName()))
				{
					//Username bereits vorhanden
					DiaryLogger errMsg = new DiaryLogger(IRessources.res.getString("0167"));
					
					errMsg.start();
					
					throw new Exception(IRessources.res.getString("0167"));
				}
			}
			
			//solange keine freie ID gefunden wurde
			while(!found)
			{
				found = true;
				//Erhoehen der Variable freeIndex um 1
				freeID += 1;
				
				//Ueberprueft jedes Partner Object in der Liste
				for(int x = 0; x<users.size();x++)
				{
					//Wenn die ID des aktuellen Partner Objects der Variable freeIndex entspricht
					//(= ID bereits vorhanden) -> setze found auf false
					if(users.get(x).getUserID()==freeID)
					{
						found = false;
					}
				}
			}
			
			user.setUserID(freeID);
			users.add(user);
		}
		
		writeUserFile(users);
	}
	
	/**
	 * Loeschen eines Users aus der UserAccounts Datei.
	 * Loescht ebenfalls alle DiaryEvent Files des Users aus dem uebergebenem Speicherpfad.
	 * 
	 * @param user
	 * @param savePath
	 * @throws Exception 
	 */
	public void delUser(User user, String savePath) throws Exception
	{
		if(user == null)
		{
			//User darf nicht null sein
			DiaryLogger errMsg = new DiaryLogger(IRessources.res.getString("0168")+
					getClass().getName()+" <delUser>");
			
			errMsg.start();
			
			throw new Exception(IRessources.res.getString("0169"));
		}
		
		ArrayList<User> users = getUsers();

		if(users==null)
		{
			//User File nicht vorhanden oder fehlerhaft
			DiaryLogger errMsg = new DiaryLogger(IRessources.res.getString("0160")
					+ getClass().getName() + " <delUser>");
			
			errMsg.start();
			
			throw new Exception(IRessources.res.getString("0161"));
		}
		
		if(!users.remove(user))
		{
			//User kann nicht del werden da nicht vorhanden
			DiaryLogger errMsg = new DiaryLogger(IRessources.res.getString("0170") + user.getUserName() + 
					IRessources.res.getString("0171"));
			
			errMsg.start();
			
			throw new Exception(IRessources.res.getString("0170") + user.getUserName() + 
					IRessources.res.getString("0171"));
		}
		
		ArrayList<String> files = diaryEvtFileMgr.getListOfEventFiles(user, savePath);
		
		for (String file : files) 
		{
			new File(savePath+"\\"+file).delete();
		}
		
		writeUserFile(users);
	}
}