package de.joerg.diary.frontend.general;

import java.awt.BorderLayout;
import java.awt.Dimension;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import de.joerg.diary.frontend.sdBasicComponents.BackGroundPane;
import de.joerg.diary.frontend.sdBasicComponents.View;
import de.joerg.diary.shared.IRessources;
import de.joerg.diary.shared.dataSet.DataSet;
import de.joerg.diary.shared.dataSet.DataSetRowAlreadyExistsException;

/*
 *  SimpleDiary Version 2.0.0
    Copyright (C) 2010  Andreas Joerg (E-Mail: simpleDiary@freenet.de)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Beschreibt einen View fuer den Dialog "Stichwoerter zum Eintrag"
 * 
 * @author Andreas Joerg
 */
public class EntryKeywordsV extends View
{
	private static final long serialVersionUID = 6095550497467156100L;
	private JButton btnAdd = null;
	private JButton btnDel = null;
	private JButton btnOK = null;
	private JScrollPane tblScrollPane = null;
	private JTextField keyword = null;
	private JTable tblKeywords = null;
	private EntryKeywordsVC controller = null;
	private DataSet dataSet = null;
	
	/**
	 * Constructor
	 * 
	 * @param controller -> ParentController
	 * @throws DataSetRowAlreadyExistsException
	 */
	public EntryKeywordsV(EntryKeywordsVC controller) throws DataSetRowAlreadyExistsException
	{
		setController(controller);
		
		initComponents();
		
		initDataSet();
	}

	//-------------------------GETTER/SETTER------------------------
	public void setController(EntryKeywordsVC controller)
	{
		this.controller = controller;
	}
	public EntryKeywordsVC getController() 
	{
		return controller;
	}

	public void setDataSet(DataSet dataSet) 
	{
		this.dataSet = dataSet;
	}
	public DataSet getDataSet() 
	{
		return dataSet;
	}
	//---------------------------------------------------------------

	/**
	 * Init der Components
	 */
	private void initComponents()
	{
		setContentPane(new BackGroundPane(IRessources.BG_IMAGE));
		setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		setTitle(IRessources.res.getString("0199"));
		setLocation(100, 100);
		setPreferredSize(new Dimension(500,294));
		setLayout(new BorderLayout());
		addWindowListener(controller);
		
		tblKeywords = new JTable();
		tblKeywords.setOpaque(false);
		tblKeywords.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		tblKeywords.getSelectionModel().addListSelectionListener(controller);
		tblKeywords.setModel(new javax.swing.table.DefaultTableModel(
				new Object[][] 
				             { 
								{ "1", null },
								{ "2", null },
								{ "3", null }
							},
							new String[] 
							           {//Nr, Stichwort 
											IRessources.res.getString("0200"),
											IRessources.res.getString("0201")
										}));
		
		tblScrollPane = new JScrollPane(tblKeywords);
		tblScrollPane.getViewport().setOpaque(false);
		tblScrollPane.setOpaque(false);

		btnDel = new JButton(IRessources.res.getString("0036"));
		btnDel.setEnabled(false);
		btnDel.setActionCommand("0041");
		btnDel.addActionListener(controller);

		btnAdd = new JButton(IRessources.res.getString("0202"));
		btnAdd.setEnabled(false);
		btnAdd.setActionCommand("0042");
		btnAdd.addActionListener(controller);

		btnOK = new JButton(IRessources.res.getString("0010"));
		btnOK.setActionCommand("0043");
		btnOK.addActionListener(controller);
		
		keyword = new JTextField();
		keyword.setEditable(false);

		//---------------------LAYOUT BY MATISSE-------------------------------------------------------------
		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(
				getContentPane());
		getContentPane().setLayout(layout);
		layout
				.setHorizontalGroup(layout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								layout
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												layout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.LEADING)
														.addComponent(
																btnAdd,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																103,
																javax.swing.GroupLayout.PREFERRED_SIZE)
														.addComponent(
																btnDel,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																103,
																javax.swing.GroupLayout.PREFERRED_SIZE)
														.addComponent(
																keyword,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																460,
																Short.MAX_VALUE)
														.addComponent(
																tblScrollPane,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																460,
																Short.MAX_VALUE)
														.addComponent(
																btnOK,
																javax.swing.GroupLayout.Alignment.TRAILING,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																103,
																javax.swing.GroupLayout.PREFERRED_SIZE))
										.addContainerGap()));
		layout
				.setVerticalGroup(layout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								layout
										.createSequentialGroup()
										.addContainerGap()
										.addComponent(
												tblScrollPane,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												70, Short.MAX_VALUE)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
										.addComponent(btnDel)
										.addGap(25, 25, 25)
										.addComponent(
												keyword,
												javax.swing.GroupLayout.PREFERRED_SIZE,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												javax.swing.GroupLayout.PREFERRED_SIZE)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
										.addComponent(btnAdd)
										.addGap(18, 18, 18).addComponent(btnOK)
										.addContainerGap()));
		//----------------------------------------------------------------------------------------------------
		
		pack();
	}
	
	/**
	 * Initialisiert das DataSet
	 * 
	 * @throws DataSetRowAlreadyExistsException
	 */
	private void initDataSet() throws DataSetRowAlreadyExistsException
	{
		dataSet = new DataSet();
		
		dataSet.addRow("keyword", null);
		
		//Table dataSet init
		DataSet dsTable = new DataSet();
		dsTable.addRow("keyword1", null);
		dsTable.addRow("keyword2", null);
		dsTable.addRow("keyword3", null);
		
		dataSet.addRow("dsTable", dsTable);
	}
	
	/**
	 * Setzt die Werte aus dem DataSet in die entsprechenden Components
	 */
	public void updateView()
	{
		keyword.setText(dataSet.getString("keyword"));
		
		DataSet dsTable = dataSet.getDataSet("dsTable");
		tblKeywords.getModel().setValueAt(dsTable.getString("keyword1"), 0, 1);
		tblKeywords.getModel().setValueAt(dsTable.getString("keyword2"), 1, 1);
		tblKeywords.getModel().setValueAt(dsTable.getString("keyword3"), 2, 1);
	}
	
	/**
	 * Setzt die Werte aus den Controls ins DataSet
	 */
	public void updateDataSet()
	{
		dataSet.setValueOfRow("keyword", keyword.getText());
	}
	
	/**
	 * Stellt die Zustaende der Controls entsprechende dem uebergebenem Zustand.
	 * 
	 * @param state
	 */
	public void updateComponents(String state)
	{
		//Eintrag wird geaendert
		if(state.equals("change"))
		{
			keyword.setEditable(true);
			btnAdd.setEnabled(true);
		}
		//alle Stichwoerter sind gefuellt worden.
		else if(state.equals("noChange"))
		{
			keyword.setEditable(false);
			btnAdd.setEnabled(false);
		}
		//Auswahl in Table ist erfolgt
		else if(state.equals("selected"))
		{
			btnDel.setEnabled(true);
		}
		//In Table wurde leerer Eintrag selected
		else if(state.equals("selectedNull"))
		{
			btnDel.setEnabled(false);
		}
	}
}
