package de.joerg.diary.frontend.general;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import de.joerg.diary.frontend.sdBasicComponents.ViewController;
import de.joerg.diary.shared.IRessources;
import de.joerg.diary.shared.dataSet.DataSet;
import de.joerg.diary.shared.dataSet.DataSetRowAlreadyExistsException;

/*
 *  SimpleDiary Version 2.0.0
    Copyright (C) 2010  Andreas Joerg (E-Mail: simpleDiary@freenet.de)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Beschreibt einen Controller fuer den Dialog "Stichwoerter zum Eintrag"
 * 
 * @author Andreas Joerg
 */
public class EntryKeywordsVC extends ViewController implements ActionListener, WindowListener, 
															ListSelectionListener 
{
	private EntryKeywordsV view = null;
	private MainWindowVC controller = null;
	private int sub = -1;
	private int selectedRow = -1;
	
	/**
	 * Constructor
	 * 
	 * @param controller	-> Parent Controller
	 * @param sub			-> Nummer des Untereintrag, wenn kein Untereintrag -> -1
	 * @throws DataSetRowAlreadyExistsException
	 */
	public EntryKeywordsVC(MainWindowVC controller, int sub) throws DataSetRowAlreadyExistsException
	{
		setController(controller);
		
		//Untereintrag setzen, wenn kein Untereintrag sub = -1
		setSub(sub);
		
		setView(new EntryKeywordsV(this));
		
		//Eintrag wird geaendert oder neu erstellt.
		if(controller.isInChange()||controller.isNewEntryBeingCreated())
		{
			setChangeAble();
		}
		
		//Stichwoerte (falls vorhanden) in View dataSet setzen
		updateTableData(getKeywordsFromModel());
		
		//Values aus DataSet im View in Componenten uebertragen
		view.updateView();
		
		view.setVisible(true);
	}

	//-------------------------GETTER/SETTER-----------------------------------------------------------------
	public void setView(EntryKeywordsV view) 
	{
		this.view = view;
	}
	public EntryKeywordsV getView() 
	{
		return view;
	}

	public void setController(MainWindowVC controller) 
	{
		this.controller = controller;
	}
	public MainWindowVC getController() 
	{
		return controller;
	}

	public void setSub(int sub) 
	{
		this.sub = sub;
	}
	public int getSub() 
	{
		return sub;
	}
	//-----------------------------------------------------------------------------------------------------
	
	/**
	 * Setzt Controls zum Adden neuen Eintrag aktiv, wenn nicht schon alle 3 gefuellt.
	 */
	public void setChangeAble()
	{
		//Controls zum Adden neues Stichwort duerfen nur aktiv gesetzt werden, wenn nicht schon in allen drei
		//Stichwortfeldern etwas steht. Pruefung aufs letzte langt, da bei Loeschen die Werte immer nach vorne
		//gezogen werden.
		if(getKeywordsFromModel()[2]==null)
		{
			view.updateComponents("change");
			
			//Wenn ein Eintrag selektiert ist -> Button "Loeschen" ebenfalls aktiv setzen
			if(selectedRow!=-1&&getKeywordsFromModel()[selectedRow]!=null)
			{
				view.updateComponents("selected");
			}
		}
	}
	
	/**
	 * @return Stichwortliste des Eintrags je nachdem ob Haupt- oder Untereintrag
	 */
	private String[] getKeywordsFromModel()
	{
		//Parent
		if(sub<0)
		{
			return controller.getModel().getKeywords();
		}
		//Untereintrag
		else
		{
			return controller.getModel().getSubEvent(sub).getKeywords();
		}
	}
	
	/**
	 * Setzt die Werte aus der uebergebenen Stichwoerterliste ins ViewDataSet.
	 * 
	 * @param keywords
	 */
	private void updateTableData(String[] keywords)
	{
		//Table init
		DataSet dsTable = view.getDataSet().getDataSet("dsTable");
		
		//setzen in View DataSet
		dsTable.setValueOfRow("keyword1", keywords[0]);
		dsTable.setValueOfRow("keyword2", keywords[1]);
		dsTable.setValueOfRow("keyword3", keywords[2]);
	}
	
	/**
	 * Prueft das uebergebene Stichwort auf Gueltigkeit.
	 * 
	 * @param keyword
	 * @return true/false
	 */
	private boolean validateKeyword()
	{
		//Stichwort aus Textfeld in DataSet setzen
		view.updateDataSet();
		
		//Holen Stichwort aus View
		String keyword = view.getDataSet().getString("keyword");
		
		//Stichwort darf nicht leer sein
		if(keyword==null||keyword.isEmpty())
		{
			handleException(IRessources.res.getString("0203"), getClass().getName(), "validateKeyword");
		}
		//Stichwort darf nur ein einzelnes Wort sein.
		else if(keyword.contains(" "))
		{	
			handleException(IRessources.res.getString("0204"), getClass().getName(), "validateKeyword");
		}
		//Stichwort darf nicht mehr als 25 Stellen haben.
		else if(keyword.length() > 25)
		{
			handleException(IRessources.res.getString("0205"), getClass().getName(), "validateKeyword");
		}
		//alles OK
		else
		{
			return true;
		}
		
		return false;
	}
	
	/**
	 * Loescht das ausgewaehlte Stichwort aus dem Model
	 */
	private void doDeleteKeyword()
	{
		try
		{
			//Setzen copyModel im MainWindow um ggf wieder rueckgaengig machen zu koennen und um abspeichern
			//zu koennen. siehe auch Mantis 31
			if(controller.getCopyModel() == null)
			{
				controller.setCopyModel(controller.getModel().getCopy());
			}
			
			String[] keywords = getKeywordsFromModel();
			
			//Stichwort loeschen, Liste wird dabei ggf. von hinten nach vorne aufgefuellt.
			if(selectedRow==2)
			{
				keywords[2] = null;
			}
			else if(selectedRow==1)
			{
				if(keywords[2]!=null)
				{
					keywords[1] = keywords[2];
					keywords[2] = null;
				}
				else
				{
					keywords[1] = null;
				}
			}
			else if(selectedRow == 0)
			{
				if(keywords[1]!=null)
				{
					keywords[0] = keywords[1];
					
					if(keywords[2]!=null)
					{
						keywords[1] = keywords[2];
						keywords[2] = null;
					}
					else
					{
						keywords[1] = null;
					}
				}
				else
				{
					keywords[0] = null;
				}
			}
			
			//Inhalt Stichwortfeld auf leer setzen + Stichwoerter ins View DataSet setzen + View aktualisieren
			updateTableData(keywords);
			view.getDataSet().setValueOfRow("keyword", null);
			view.updateView();
			
			//Wenn geloeschter Eintrag letzter war, darf wieder ein Stichwort hinzugefuegt werden
			setChangeAble();
			
			//Wenn im aktuell markierten Tabelleneintrag nichts mehr steht
			if(keywords[selectedRow]==null)
			{
				//Button "Loeschen" disablen
				view.updateComponents("selectedNull");
			}
		}
		catch(Exception ex)
		{
			handleException(ex.getMessage(), getClass().getName(), "doDeleteKeyword", ex);
		}
	}

	/**
	 * Fuegt dem Model das aktuell eingegebene Stichwort aus dem View hinzu.
	 */
	private void doAddKeyword()
	{
		try
		{
			//Wenn Stichwort aus View ungueltig ist -> STOP
			if(!validateKeyword())
			{
				return;
			}
			
			//Holen Stichwoerter
			String[] keywords = getKeywordsFromModel();
			int free = 2;
			
			//Ermitteln des ersten leeren Stichwortfeldes
			if(keywords[1]==null)
			{
				free = 1;
			}
			if(keywords[0]==null)
			{
				free = 0;
			}
			
			//Setzen copyModel im MainWindow um ggf wieder rueckgaengig machen zu koennen und um abspeichern
			//zu koennen. siehe auch Mantis 104
			if(controller.getCopyModel() == null)
			{
				controller.setCopyModel(controller.getModel().getCopy());
			}
			
			//Setzen des Stichworts aus dem ViewDataSet in die freie Stelle in der Stichwortliste.
			keywords[free] = view.getDataSet().getString("keyword");
			
			//Parent keywords setzen
			if(sub<0)
			{
				controller.getModel().setKeywords(keywords);
			}
			//Untereintrag keywords setzen
			else
			{
				controller.getModel().getSubEvent(sub).setKeywords(keywords);
			}
			
			//Stichwoerter ins View DataSet setzen + View aktualisieren
			updateTableData(keywords);
			view.updateView();
			
			//Wenn hinzugefuegter Eintrag bereits selektiert war in Table -> Button "Loeschen" aktiv setzen
			if(selectedRow==free)
			{
				view.updateComponents("selected");
			}
			
			//Alle Felder wurden befuellt -> Es kann kein Stichwort mehr gespeichert werden.
			if(free==2)
			{
				view.updateComponents("noChange");
			}
		}
		catch(Exception ex)
		{
			handleException(ex.getMessage(), getClass().getName(), "doAddKeyword", ex);
		}
	}
	
	@Override
	public void actionPerformed(ActionEvent evt) 
	{
		//Stichwort loeschen
		if(evt.getActionCommand().equals("0041"))
		{
			doDeleteKeyword();
		}
		//Stichwort hinzufuegen
		else if(evt.getActionCommand().equals("0042"))
		{
			doAddKeyword();
		}
		//OK -> Close
		else if(evt.getActionCommand().equals("0043"))
		{
			view.dispose();
		}
	}

	@Override
	public void windowActivated(WindowEvent arg0) 
	{
	}

	@Override
	public void windowClosed(WindowEvent evt) 
	{
		//Referenz auf this muss im ParentController auf null gesetzt werden.
		controller.handleKeywordDialogClose();
	}

	@Override
	public void windowClosing(WindowEvent arg0) 
	{
	}

	@Override
	public void windowDeactivated(WindowEvent arg0)
	{
	}

	@Override
	public void windowDeiconified(WindowEvent arg0)
	{
	}

	@Override
	public void windowIconified(WindowEvent arg0) 
	{
	}

	@Override
	public void windowOpened(WindowEvent arg0)
	{
	}

	@Override
	public void valueChanged(ListSelectionEvent evt) 
	{
		try
		{
			String rowNumber = evt.getSource().toString();
			
			//Noetig da event immer zwei mal ausgeloest wird. evt.getSource().toString() kommt in den Formaten
			//javax.swing.DefaultListSelectionModel 22522451 ~{0}
			//javax.swing.DefaultListSelectionModel 22522451 ={0}
			//Die Zahl in den Klammern ist die selectede Row.
			if(rowNumber.contains("~{"))
			{
				//Holen der selecteden Row
				rowNumber = rowNumber.substring(0, rowNumber.length()-1);
				selectedRow = Integer.valueOf(rowNumber.substring(rowNumber.indexOf("~{")+2));
				
				//Wenn Parent
				if(sub<0)
				{
					//Setzen des Stichworts ins View DataSet und Update View kann auch gemacht werden wenn leeres
					//Feld in Table gewaehlt wird, da dann einfach null ins dataSet geschriebn wird.
					view.getDataSet().setValueOfRow("keyword", controller.getModel().getKeywords()
							[selectedRow]);
					view.updateView();
					
					//Stichwort ist im Model vorhanden
					if(controller.getModel().getKeywords()[selectedRow]!=null)
					{
						//Wenn der Eintrag aktuell bearbeitet wird -> Button "Loeschen" in View aktiv setzen
						if(controller.isInChange())
						{
							view.updateComponents("selected");
						}
					}
					//Stichwort ist null -> Button "Loeschen" muss disabled werden, da nichts geloescht werden
					//kann
					else
					{
						view.updateComponents("selectedNull");
					}
				}
				//Untereintrag
				else
				{
					//Setzen des Stichworts ins View DataSet und Update View kann auch gemacht werden wenn leeres
					//Feld in Table gewaehlt wird, da dann einfach null ins dataSet geschriebn wird.
					view.getDataSet().setValueOfRow("keyword", controller.getModel().getSubEvent(sub)
							.getKeywords()[selectedRow]);
					view.updateView();
					
					//Stichwort ist im Model vorhanden
					if(controller.getModel().getSubEvent(sub).getKeywords()[selectedRow]!=null)
					{
						//Wenn der Eintrag aktuell bearbeitet wird -> Button "Loeschen" in View aktiv setzen
						if(controller.isInChange())
						{
							view.updateComponents("selected");
						}
					}
					//Stichwort ist null -> Button "Loeschen" muss disabled werden, da nichts geloescht werden
					//kann
					else
					{
						view.updateComponents("selectedNull");
					}
				}
			}
		}
		catch(Exception ex)
		{
			handleException(ex.getMessage(), getClass().getName(), "valueChanged", ex);
		}
	}
}
