package de.joerg.diary.frontend.general;

import java.awt.Point;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.border.EtchedBorder;
import de.joerg.diary.frontend.sdBasicComponents.View;
import de.joerg.diary.shared.IRessources;
import de.joerg.diary.shared.dataSet.DataSet;

/*
 *  SimpleDiary Version 2.0.0
    Copyright (C) 2010  Andreas Joerg (E-Mail: simpleDiary@freenet.de)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * View fuer einen Fehlerdialog
 * 
 * @author Andreas Joerg
 */
public class ExceptionDialogV extends View
{
	private static final long serialVersionUID = -8636493380132759594L;
	private ExceptionDialogVC viewController = null;
	private JCheckBox chkDetails = null;
	private JTextField tfClassPath = null;
	private JTextArea textStackTrace = null;
	private JTextArea text = null;
	private JButton btnOK = null;
	
	/**
	 * Erweiterter Konstruktor
	 * 
	 * @param viewController
	 */
	public ExceptionDialogV (ExceptionDialogVC viewController)
	{
		setViewController(viewController);
	}

	//---GETTER/SETTER---
	public void setViewController(ExceptionDialogVC viewController) 
	{
		this.viewController = viewController;
	}
	public ExceptionDialogVC getViewController() 
	{
		return viewController;
	}
	//------------------
	
	/**
	 * Methode zur Initialisierung des Views
	 */
	public void init() 
	{
		this.setSize(500, 200);
		this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		this.setLayout(null);
		this.setTitle(IRessources.res.getString("0008"));
		this.setLocation(new Point(300,300));
		this.setResizable(false);
		
		text = new JTextArea();
		text.setEditable(false);
		
		JScrollPane scrollPane = new JScrollPane(text);
		scrollPane.setBounds(10, 10, 391, 110);
		
		chkDetails = new JCheckBox(IRessources.res.getString("0009"));
		chkDetails.setBounds(420, 10, 100, 20);
		chkDetails.addItemListener(viewController);
		
		btnOK = new JButton(IRessources.res.getString("0010"));
		btnOK.setBounds(300, 130, 180, 30);
		btnOK.setActionCommand("0014");
		btnOK.addActionListener(viewController);
		
		this.add(btnOK);
		this.add(chkDetails);
		this.add(scrollPane);
		this.setVisible(true);
	}
	
	/**
	 * Methode um den View um die Details zu erweitern.
	 */
	public void showDetails()
	{
		JPanel pnlDetails = new JPanel();
		pnlDetails.setLayout(null);
		pnlDetails.setBounds(10, 185, 470, 250);
		pnlDetails.setBorder(new EtchedBorder());
		this.setSize(500, 485);
		
		JLabel lblSource = new JLabel(IRessources.res.getString("0011"));
		lblSource.setBounds(10, 13, 50, 20);
		
		tfClassPath = new JTextField();
		tfClassPath.setBounds(60, 10, 380, 26);
		tfClassPath.setEditable(false);
		
		textStackTrace = new JTextArea();
		textStackTrace.setEditable(false);
		
		JScrollPane detailScrollPane = new JScrollPane(textStackTrace);
		detailScrollPane.setBounds(10, 40, 430, 200);
		
		pnlDetails.add(detailScrollPane);
		pnlDetails.add(lblSource);
		pnlDetails.add(tfClassPath);
		
		this.add(pnlDetails);
		chkDetails.setEnabled(false);
	}
	
	/**
	 * Setzt das DataSet des Views
	 * 
	 * @param dataSet
	 */
	public void setDataSet(DataSet dataSet)
	{
		text.setText((String) dataSet.getRowValueByName("text"));
		
		//tfClassPath und textStackTrace sind nur null, falls die Details (noch) nicht angezeigt
		//werden sollen.
		if(tfClassPath!=null)
		{
			tfClassPath.setText((String) dataSet.getRowValueByName("classPath"));
		}
		if(textStackTrace!=null)
		{
			textStackTrace.setText((String) dataSet.getRowValueByName("stackTrace"));
		}
	}
}
