package de.joerg.diary.frontend.general;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.ByteArrayOutputStream;
import java.io.PrintStream;

import de.joerg.diary.frontend.sdBasicComponents.ViewController;
import de.joerg.diary.shared.IRessources;
import de.joerg.diary.shared.dataSet.DataSet;
import de.joerg.diary.shared.dataSet.DataSetRowAlreadyExistsException;

/*
 *  SimpleDiary Version 2.0.0
    Copyright (C) 2010  Andreas Joerg (E-Mail: simpleDiary@freenet.de)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * ViewController fuer einen FehlerDialog.
 * 
 * @author Andreas Joerg
 */
public class ExceptionDialogVC extends ViewController implements ItemListener, ActionListener
{
	private ExceptionDialogV view = null;
	private String msg = null;
	private String classPath = null;
	private Exception ex = null;
	
	/**
	 * Erweiterter Konstruktor
	 * 
	 * @param msg
	 * @param classPath
	 * @throws DataSetRowAlreadyExistsException 
	 */
	public ExceptionDialogVC(String msg, String classPath) throws DataSetRowAlreadyExistsException
	{
		setMsg(msg);
		setClassPath(classPath);
		setView(new ExceptionDialogV(this));
		
		DataSet dataSet = new DataSet();
		dataSet.addRow("text", msg);
		
		view.init();
		view.setDataSet(dataSet);
	}
	
	/**
	 * Erweiterter Konstruktor mit Uebergabe Exception
	 * 
	 * @param msg
	 * @param classPath
	 * @param ex
	 * @throws DataSetRowAlreadyExistsException 
	 */
	public ExceptionDialogVC(String msg, String classPath, Exception ex) throws DataSetRowAlreadyExistsException
	{
		setMsg(msg);
		setClassPath(classPath);
		setEx(ex);
		
		DataSet dataSet = new DataSet();
		dataSet.addRow("text", msg);
		
		setView(new ExceptionDialogV(this));
		view.init();
		view.setDataSet(dataSet);
	}

	//-------------------------------------------------------------------------------------------
	//			GETTER/SETTER
	//-------------------------------------------------------------------------------------------
	public String getMsg() 
	{
		return msg;
	}
	public void setMsg(String msg) 
	{
		this.msg = msg;
	}

	public String getClassPath() 
	{
		return classPath;
	}
	public void setClassPath(String classPath) 
	{
		this.classPath = classPath;
	}

	public Exception getEx() 
	{
		return ex;
	}
	public void setEx(Exception ex) 
	{
		this.ex = ex;
	}

	public void setView(ExceptionDialogV view) 
	{
		this.view = view;
	}
	public ExceptionDialogV getView()
	{
		return view;
	}
	//-------------------------------------------------------------------------------------------

	@Override
	/**
	 * Wird ausgeloest, wenn die Checkbox Details im View gesetzt wird.
	 */
	public void itemStateChanged(ItemEvent evt) 
	{
		//Wenn Checkbox gesetzt ist -> Setzt DataSet des Views mit Details (msg, exception)
		if(evt.getStateChange() == ItemEvent.SELECTED)
		{
			DataSet dataSet = new DataSet();
			try 
			{
				dataSet.addRow("text", msg);
				
				dataSet.addRow("classPath", classPath);

				if(ex!=null)
				{
					ByteArrayOutputStream out = new ByteArrayOutputStream();
					PrintStream printStream = new PrintStream(out);
					ex.printStackTrace(printStream);
					dataSet.addRow("stackTrace", out.toString());
				}
				else
				{
					dataSet.addRow("stackTrace", IRessources.EMPTY);
				}

				view.showDetails();
				view.setDataSet(dataSet);
			} 
			catch (DataSetRowAlreadyExistsException ex) 
			{
				handleException(ex.getMessage(), getClass().getName(), "itemStateChanged", ex);
			}
			
		}
	}

	@Override
	/**
	 * Reagiert auf ActionEvents des Views
	 */
	public void actionPerformed(ActionEvent evt) 
	{
		//Schliesen
		if(evt.getActionCommand().equals("0014"))
		{
			view.dispose();
		}
	}
}
