package de.joerg.diary.frontend.general;

import java.awt.BorderLayout;
import java.awt.Dimension;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import de.joerg.diary.frontend.sdBasicComponents.BackGroundPane;
import de.joerg.diary.frontend.sdBasicComponents.EmptyPanel;
import de.joerg.diary.frontend.sdBasicComponents.View;
import de.joerg.diary.shared.IRessources;
import de.joerg.diary.shared.dataSet.DataSet;
import de.joerg.diary.shared.dataSet.DataSetRowAlreadyExistsException;

/*
 *  SimpleDiary Version 2.0.0
    Copyright (C) 2010  Andreas Joerg (E-Mail: simpleDiary@freenet.de)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Beschreibt einen View fuer den Dialog "Bildbeschreibung"
 * 
 * @author Andreas Joerg
 */
public class ImageDescriptionV extends View
{
	private static final long serialVersionUID = -5083752144518377438L;
	private DataSet dataSet = new DataSet();
	private ImageDescriptionVC controller = null;
	private JButton btnChange = null;
	private JTextArea text = null;
	private JButton btnSave = null;
	
	/**
	 * Constructor mit Uebergabe des Controllers.
	 * 
	 * @param controller
	 * @throws DataSetRowAlreadyExistsException
	 */
	public ImageDescriptionV(ImageDescriptionVC controller) throws DataSetRowAlreadyExistsException
	{
		setController(controller);
		
		init();
		
		initDataSet();
	}
	
	/**
	 * Komponenten initialisieren
	 */
	private void init()
	{
		setContentPane(new BackGroundPane(IRessources.BG_IMAGE));
		setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		setTitle(IRessources.res.getString("0189"));
		setLocation(100, 100);
		setLayout(new BorderLayout());
		addWindowListener(controller);
		
		text = new JTextArea();
		text.setEditable(false);
		JScrollPane descScrollPane = new JScrollPane(text);
		
		//Bearbeiten
		btnChange = new JButton(IRessources.res.getString("0025"));
		btnChange.setEnabled(false);
		btnChange.setActionCommand("0037");
		btnChange.addActionListener(controller);
		
		//Bestaetigen
		btnSave = new JButton(IRessources.res.getString("0190"));
		btnSave.setEnabled(false);
		btnSave.setActionCommand("0038");
		btnSave.addActionListener(controller);
		
		//--------LAYOUT--------------------------------------------------------------------------------------
		add(descScrollPane, BorderLayout.CENTER);
		
		EmptyPanel ePnl = new EmptyPanel();
		ePnl.add(btnChange);
		ePnl.add(btnSave);
		
		EmptyPanel ePnlOver = new EmptyPanel();
		ePnlOver.setLayout(new BorderLayout());
		ePnlOver.add(ePnl, BorderLayout.EAST);
		
		add(ePnlOver, BorderLayout.SOUTH);
		
		setPreferredSize(new Dimension(450,250));
		//----------------------------------------------------------------------------------------------------
		
		pack();
	}

	//------GETTER/SETTER---------------------------------------------------
	public DataSet getDataSet() 
	{
		return dataSet;
	}
	public void setDataSet(DataSet dataSet) 
	{
		this.dataSet = dataSet;
	}

	public ImageDescriptionVC getController() 
	{
		return controller;
	}
	public void setController(ImageDescriptionVC controller) 
	{
		this.controller = controller;
	}
	//----------------------------------------------------------------------
	
	/**
	 * DataSet initialisieren
	 */
	private void initDataSet() throws DataSetRowAlreadyExistsException
	{
		getDataSet().addRow("text", null);
	}
	
	/**
	 * Schiebt die Werte aus dem DataSet in die entsprechenden Controls
	 */
	public void updateView()
	{
		text.setText(dataSet.getString("text"));
	}
	
	/**
	 * Schieb die Werte aus den Controls ins DataSet
	 */
	public void updateDataSet()
	{
		dataSet.setValueOfRow("text", text.getText());
	}
	
	/**
	 * Setzt den Button "Bearbeiten" aktiv.
	 */
	public void setChangeAble()
	{
		btnChange.setEnabled(true);
	}
	
	/**
	 * Setzt die Controls auf den Zustand, wenn der Text geaendert werden soll.
	 */
	public void setInChange()
	{
		btnChange.setEnabled(false);
		btnSave.setEnabled(true);
		text.setEditable(true);
	}
}
