package de.joerg.diary.frontend.general;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import javax.swing.JOptionPane;
import de.joerg.diary.frontend.sdBasicComponents.ViewController;
import de.joerg.diary.frontend.sdUndoManager.SDUndoManager;
import de.joerg.diary.frontend.sdUndoManager.UndoActionForChangeImgDesc;
import de.joerg.diary.shared.IRessources;
import de.joerg.diary.shared.dataSet.DataSetRowAlreadyExistsException;

/*
 *  SimpleDiary Version 2.0.0
    Copyright (C) 2010  Andreas Joerg (E-Mail: simpleDiary@freenet.de)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Beschreibt einen Controller fuer den Dialog "Bildbeschreibung"
 * 
 * @author Andreas Joerg
 */
public class ImageDescriptionVC extends ViewController implements ActionListener, WindowListener
{
	private MainWindowVC controller = null;
	private ImageDescriptionV view = null;
	private int sub = -1;
	private int img = -1;
	
	/**
	 * Constructor
	 * 
	 * @param controller -> Controller des MainWindows
	 * @param sub -> Nummer des Untereintrags, wenn kein Untereintrag dann -1
	 * @param img -> Nummer des Bilds in der Bilderliste des (Unter)eintrags.
	 * @throws DataSetRowAlreadyExistsException
	 */
	public ImageDescriptionVC(MainWindowVC controller, int sub, int img)
		throws DataSetRowAlreadyExistsException
	{
		setController(controller);
		setSub(sub);
		setImg(img);
		
		//View erstellen.
		setView(new ImageDescriptionV(this));
		
		updateView();
		
		//Wenn der aktuelle Eintrag im Aenderungsstatus ist, darf die Bildbeschreibung auch geaendert werden.
		if(controller.isInChange())
		{
			setChangeAble();
		}
		
		view.setVisible(true);
	}

	//-------------GETTER/SETTER-------------------------------
	public void setController(MainWindowVC controller)
	{
		this.controller = controller;
	}
	public MainWindowVC getController() 
	{
		return controller;
	}

	public void setView(ImageDescriptionV view) 
	{
		this.view = view;
	}
	public ImageDescriptionV getView()
	{
		return view;
	}

	public int getSub() 
	{
		return sub;
	}
	public void setSub(int sub) 
	{
		this.sub = sub;
	}

	public int getImg() 
	{
		return img;
	}
	public void setImg(int img) 
	{
		this.img = img;
	}
	//---------------------------------------------------------

	/**
	 * Schliet den View.
	 */
	public void doCloseView()
	{
		view.dispose();
	}
	
	/**
	 * Setzt den Button "Bearbeiten" aktiv.
	 */
	public void setChangeAble()
	{
		view.setChangeAble();
	}
	
	/**
	 * Holt die Beschreibung aus dem Model, setzt sie ins View DataSet und updated den View.
	 */
	public void updateView()
	{
		//ParentEintrag
		if(sub < 0)
		{
			view.getDataSet().setValueOfRow("text", 
					controller.getModel().getPictures().get(img).getPictureDesc());
		}
		//ChildEintrag
		else
		{
			view.getDataSet().setValueOfRow("text", 
					controller.getModel().getSubEvent(sub).getPictures().get(img).getPictureDesc());
		}
		
		view.updateView();
	}
	
	@Override
	public void actionPerformed(ActionEvent evt) 
	{	
		//Bearbeiten
		if(evt.getActionCommand().equals("0037"))
		{
			view.setInChange();
		}
		//Bestaetigen
		else if(evt.getActionCommand().equals("0038"))
		{
			//View DataSet updaten
			view.updateDataSet();
			
			//Setzen copyModel im MainWindow um ggf wieder rueckgaengig machen zu koennen und um abspeichern
			//zu koennen.
			if(controller.getCopyModel()==null)
			{
				controller.setCopyModel(controller.getModel().getCopy());
			}
			
			//Parent Eintrag
			if(sub < 0)
			{
				//Aktion UndoManager adden
				SDUndoManager.getInstance().addAction(new UndoActionForChangeImgDesc(
						controller.getModel().getPictures().get(img).getPictureDesc(), sub, img));
				
				controller.getModel().getPictures().get(img).setPictureDesc(
						view.getDataSet().getString("text"));
			}
			//Untereintrag
			else
			{
				//Aktion UndoManager adden
				SDUndoManager.getInstance().addAction(new UndoActionForChangeImgDesc(
						controller.getModel().getSubEvent(sub).getPictures().get(img).getPictureDesc(),
						sub, img));
				
				controller.getModel().getSubEvent(sub).getPictures().get(img).setPictureDesc(
						view.getDataSet().getString("text"));
			}
			
			//Bildbeschreibung erfolgreich bearbeitet.
			JOptionPane.showMessageDialog(view, IRessources.res.getString("0191"),
					IRessources.res.getString("0003"), JOptionPane.INFORMATION_MESSAGE);
			
			controller.changeStatusLine(IRessources.res.getString("0191"));
		}
	}

	@Override
	public void windowActivated(WindowEvent arg0)
	{
	}

	@Override
	public void windowClosed(WindowEvent arg0) 
	{
		//Referenz auf this muss im ParentController auf null gesetzt werden.
		controller.handleImageDescDialogClose();
	}

	@Override
	public void windowClosing(WindowEvent arg0)
	{
	}

	@Override
	public void windowDeactivated(WindowEvent arg0) 
	{
	}

	@Override
	public void windowDeiconified(WindowEvent arg0) 
	{
	}

	@Override
	public void windowIconified(WindowEvent arg0)
	{
	}

	@Override
	public void windowOpened(WindowEvent arg0)
	{
	}
}
