package de.joerg.diary.frontend.general;

import java.awt.Color;
import java.awt.Font;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPasswordField;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import de.joerg.diary.frontend.sdBasicComponents.BackGroundPane;
import de.joerg.diary.frontend.sdBasicComponents.View;
import de.joerg.diary.shared.IRessources;
import de.joerg.diary.shared.dataSet.DataSet;
import de.joerg.diary.shared.dataSet.DataSetRowAlreadyExistsException;

/*
 *  SimpleDiary Version 2.0.0
    Copyright (C) 2010  Andreas Joerg (E-Mail: simpleDiary@freenet.de)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * View fuer einen LoginDialog
 * 
 * @author Andreas Joerg
 */
public class LoginDialogV extends View 
{
	private static final long serialVersionUID = -699043108533847324L;
	private JTextField tfUserName = null;
	private JPasswordField tfUserPw = null;
	private LoginDialogVC controller = null;

	/**
	 * Default Constructor
	 */
	public LoginDialogV()
	{
		
	}
	
	/**
	 * Erweiterter Konstruktor
	 * 
	 * @param controller
	 */
	public LoginDialogV(LoginDialogVC controller)
	{
		setController(controller);
	}
	
	//-------------Getter/Setter---
	public void setController(LoginDialogVC controller)
	{
		this.controller = controller;
	}
	public LoginDialogVC getController()
	{
		return controller;
	}
	//-----------------------------
	
	/**
	 * Initialisiert den View
	 */
	public void init() throws Exception
	{
		this.setSize(365, 375);
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.setTitle(IRessources.res.getString("0001"));
		this.setContentPane(new BackGroundPane(IRessources.BG_IMAGE));
		this.setLayout(null);
		this.setResizable(false);
		this.setLocation(300, 200);
		
		//Setzen des Windows Look&Feel
		//UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
		//Setzen des Nimbus Look&Feel (MANTIS 109)
		UIManager.setLookAndFeel("com.sun.java.swing.plaf.nimbus.NimbusLookAndFeel");
		SwingUtilities.updateComponentTreeUI(this);
		
		JLabel lblSimpleDiary = new JLabel(IRessources.res.getString("0003"));
		lblSimpleDiary.setFont(new Font("Verdana",Font.BOLD, 18));
		lblSimpleDiary.setForeground(Color.WHITE);
		lblSimpleDiary.setBounds(105, 20, 130, 20);
		
		JLabel lblUserName = new JLabel(IRessources.res.getString("0004"));
		lblUserName.setFont(new Font("Verdana", Font.BOLD, 14));
		lblUserName.setForeground(Color.WHITE);
		lblUserName.setBounds(10,98, 125, 20);
		
		tfUserName = new JTextField();
		tfUserName.setBounds(155, 95, 170, 26);
		
		JLabel lblUserPw = new JLabel(IRessources.res.getString("0005"));
		lblUserPw.setFont(new Font("Verdana", Font.BOLD, 14));
		lblUserPw.setForeground(Color.WHITE);
		lblUserPw.setBounds(10, 148, 125, 20);
		
		tfUserPw = new JPasswordField();
		tfUserPw.setBounds(155, 145, 170, 26);
		
		JButton btnOK = new JButton(IRessources.res.getString("0006"));
		btnOK.setBounds(10, 210, 315, 24);
		btnOK.setActionCommand("0001");
		btnOK.addActionListener(controller);
		
		JButton btnNewUser = new JButton(IRessources.res.getString("0007"));
		btnNewUser.setBounds(10,254,315,24);
		btnNewUser.setActionCommand("0002");
		btnNewUser.addActionListener(controller);
		
		this.add(btnNewUser);
		this.add(btnOK);
		this.add(tfUserPw);
		this.add(tfUserName);
		this.add(lblUserPw);
		this.add(lblUserName);
		this.add(lblSimpleDiary);
		this.setVisible(true);
	}
	
	/**
	 * Gibt das DataSet des Views zurueck
	 * 
	 * @return dataSet
	 * @throws DataSetRowAlreadyExistsException 
	 */
	public DataSet getDataSet() throws DataSetRowAlreadyExistsException
	{
		DataSet dataSet = new DataSet();
		
		dataSet.addRow("userName", tfUserName.getText());
		dataSet.addRow("userPw", new String(tfUserPw.getPassword()));
		
		return dataSet;
	}
	
	/**
	 * Setzt das DataSet des Views
	 * 
	 * @param dataSet
	 */
	public void setDataSet(DataSet dataSet)
	{
		tfUserName.setText((String) dataSet.getRowValueByName("userName"));
		tfUserPw.setText((String) dataSet.getRowValueByName("userPw"));
	}
}
