package de.joerg.diary.frontend.general;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.MenuItem;
import java.awt.PopupMenu;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.image.BufferedImage;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Enumeration;
import javax.swing.BorderFactory;
import javax.swing.DefaultComboBoxModel;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JTabbedPane;
import javax.swing.JToolBar;
import javax.swing.JTree;
import javax.swing.KeyStroke;
import javax.swing.ScrollPaneConstants;
import javax.swing.UIManager;
import javax.swing.border.EtchedBorder;
import javax.swing.text.DefaultEditorKit;
import javax.swing.text.JTextComponent;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeSelectionModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import de.joerg.diary.frontend.sdBasicComponents.BackGroundPane;
import de.joerg.diary.frontend.sdBasicComponents.EmptyPanel;
import de.joerg.diary.frontend.sdBasicComponents.TreeCellRendererWhiteNodes;
import de.joerg.diary.frontend.sdBasicComponents.View;
import de.joerg.diary.frontend.sdUndoManager.SDTextArea;
import de.joerg.diary.frontend.sdUndoManager.SDTextField;
import de.joerg.diary.frontend.sdUndoManager.SDUndoManager;
import de.joerg.diary.shared.IRessources;
import de.joerg.diary.shared.dataSet.DataSet;
import de.joerg.diary.shared.dataSet.DataSetRow;
import de.joerg.diary.shared.dataSet.DataSetRowAlreadyExistsException;
import de.joerg.diary.shared.models.DiaryEvent;

/*
 *  SimpleDiary Version 2.0.0
    Copyright (C) 2010  Andreas Joerg (E-Mail: simpleDiary@freenet.de)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * View fuer ein Hauptfenster
 * 
 * @author Andreas Joerg
 */
public class MainWindowV extends View 
{
	private static final long serialVersionUID = 3868694291858411393L;
	private MainWindowVC controller = null;
	private JScrollPane scrollPane = null;
	private SDTextField tfDate = null;
	private SDTextField tfPlace = null;
	private SDTextField tfTimeStart = null;
	private SDTextField tfTimeEnd = null;
	private SDTextArea text = null;
	private JTree tree = null;
	private DefaultMutableTreeNode rootTreeEntry = null;
	private JButton btnChange = null;
	private JButton btnDelete = null;
	private JButton btnSave = null;
	private JMenuItem save = null;
	private JMenuItem newEvent = null;
	private JMenuItem editEvent = null;
	private JMenuItem undoEvent = null;
	private JMenuItem delete = null;
	private JButton tBarSave = null;
	private JMenuItem options = null;
	private BackGroundPane standardContentPanel = null;
	private JPanel lockedContentPanel = null;
	private JMenuBar menuBar = null;
	private JMenuItem cutEvent = null;
	private JMenuItem copyEvent = null;
	private JMenuItem pasteEvent = null;
	private DataSet dsMenuBar = null;
	private JButton btnLocked = null;
	private JMenuItem showCalendar = null;
	private JMenuItem showTree = null;
	private JPanel entryPnl = null;
	private EmptyPanel calendarBottomPanel = null;
	private JLabel lblStatus = null;
	private JMenuItem print = null;
	private JButton tBarPrint = null;
	private MenuItem popSChange = null;
	private MenuItem popSDel = null;
	private MenuItem popMDelAll = null;
	private JButton btnShowKeyWords = null;
	private BackGroundPane imgPnl = null;
	private JComboBox cbImgs = null;
	private JButton btnAddImg = null;
	private JButton btnDeleteImg = null;
	private JButton btnImgDescription = null;
	private JMenuItem backup = null;
	private JMenuItem backupRedo = null;
	private JMenuItem search = null;
	
	/**
	 * Default Constructor
	 */
	public MainWindowV()
	{
		
	}
	
	/**
	 * Erweiterter Konstruktor
	 * 
	 * @param controller
	 */
	public MainWindowV(MainWindowVC controller)
	{
		setController(controller);
	}
	
	//----Getter/Setter---
	public void setController(MainWindowVC controller) 
	{
		this.controller = controller;
	}
	public MainWindowVC getController() 
	{
		return controller;
	}
	//--------------------

	/**
	 * Initialisiert den View
	 * @throws ParseException 
	 * @throws NumberFormatException 
	 */
	public void init() throws NumberFormatException, ParseException
	{		
		standardContentPanel = new BackGroundPane(IRessources.BG_IMAGE);
		
		setSize(1000, 768);
		setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		setContentPane(standardContentPanel);
		setLayout(new BorderLayout());
		setTitle(IRessources.res.getString("0003"));
		addWindowListener(getController());
		
		//RootPane darf den Focus nicht erhalten da es den Focus sonst erhaelt beim Wechseln zwischen
		//zwei Menuepunkten. Das Problem hierbei sind die Menuepunkte "Cut, Copy, Paste" die dann disabled
		//werden wuerden, weil keine TextComponente mehr den Focus hat.
		getRootPane().setFocusable(false);
		
		//---------------------------------------------------------------------------------
		//			Init der MenuBar
		//---------------------------------------------------------------------------------
		menuBar = new JMenuBar();
		
		JMenu file = new JMenu(IRessources.res.getString("0016"));
		
		newEvent = new JMenuItem(IRessources.res.getString("0017"));
		newEvent.setActionCommand("0008");
		newEvent.addActionListener(controller);
		newEvent.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, InputEvent.CTRL_MASK));
		newEvent.setIcon(getPicture(IRessources.NEW_IMG));

		save = new JMenuItem(IRessources.res.getString("0019"));
		save.setActionCommand("0007");
		save.addActionListener(controller);
		save.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.CTRL_MASK)); 
		save.setIcon(getPicture(IRessources.SAVE_IMG));
		
		print = new JMenuItem(IRessources.res.getString("0021"));
		print.setActionCommand("0030");
		print.addActionListener(controller);
		print.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_D, InputEvent.CTRL_MASK));
		print.setIcon(getPicture(IRessources.PRINT_IMG));
		
		JMenuItem close = new JMenuItem(IRessources.res.getString("0023"));
		close.setActionCommand("0009");
		close.addActionListener(controller);
		close.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_B, InputEvent.CTRL_MASK)); 
		close.setIcon(getPicture(IRessources.EXIT_IMG));
		
		file.add(newEvent);
		file.add(new JSeparator());
		file.add(save);
		file.add(new JSeparator());
		file.add(print);
		file.add(new JSeparator());
		file.add(close);
		menuBar.add(file);
		
		JMenu edit = new JMenu(IRessources.res.getString("0025"));
		edit.addMenuListener(controller);
		
		undoEvent = new JMenuItem(IRessources.res.getString("0026"));
		undoEvent.setActionCommand("0019");
		undoEvent.addActionListener(controller);
		undoEvent.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z, InputEvent.CTRL_MASK)); 
		undoEvent.setIcon(getPicture(IRessources.UNDO_IMG));
		
		cutEvent = new JMenuItem(new DefaultEditorKit.CutAction());
		cutEvent.setText(IRessources.res.getString("0028"));
		cutEvent.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X, InputEvent.CTRL_MASK));
		cutEvent.setIcon(getPicture(IRessources.CUT_IMG));
		
		copyEvent = new JMenuItem(new DefaultEditorKit.CopyAction());
		copyEvent.setText(IRessources.res.getString("0030"));
		copyEvent.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, InputEvent.CTRL_MASK)); 
		copyEvent.setIcon(getPicture(IRessources.COPY_IMG));
		
		pasteEvent = new JMenuItem(new DefaultEditorKit.PasteAction());
		pasteEvent.setText(IRessources.res.getString("0032"));
		pasteEvent.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V, InputEvent.CTRL_MASK));
		pasteEvent.setIcon(getPicture(IRessources.PASTE_IMG));
		
		editEvent = new JMenuItem(IRessources.res.getString("0034"));
		editEvent.setActionCommand("0005");
		editEvent.addActionListener(controller);
		editEvent.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q, InputEvent.CTRL_MASK)); 
		editEvent.setIcon(getPicture(IRessources.EDIT_IMG));
		
		delete = new JMenuItem(IRessources.res.getString("0036"));
		delete.setActionCommand("0006");
		delete.addActionListener(controller);
		delete.setAccelerator(KeyStroke.getKeyStroke("DELETE"));
		delete.setIcon(getPicture(IRessources.DEL_IMG));
		
		search = new JMenuItem(IRessources.res.getString("0208"));
		search.setActionCommand("0046");
		search.addActionListener(controller);
		search.setIcon(getPicture(IRessources.FIND_IMG));
		search.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F, InputEvent.CTRL_MASK));
		
		edit.add(undoEvent);
		edit.add(new JSeparator());
		edit.add(cutEvent);
		edit.add(copyEvent);
		edit.add(pasteEvent);
		edit.add(new JSeparator());
		edit.add(editEvent);
		edit.add(new JSeparator());
		edit.add(delete);
		edit.add(new JSeparator());
		edit.add(search);
		menuBar.add(edit);
		
		JMenu viewPoint = new JMenu(IRessources.res.getString("0038"));
		
		showCalendar = new JMenuItem(IRessources.res.getString("0039"));
		showCalendar.setActionCommand("0026");
		showCalendar.addActionListener(controller);
		showCalendar.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_K, InputEvent.CTRL_MASK));
		showCalendar.setIcon(getPicture(IRessources.SHOW_CAL_IMG));
		
		showTree = new JMenuItem(IRessources.res.getString("0041"));
		showTree.setEnabled(false);
		showTree.setActionCommand("0027");
		showTree.addActionListener(controller);
		showTree.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_L, InputEvent.CTRL_MASK));
		showTree.setIcon(getPicture(IRessources.SHOW_TREE_IMG));
		
		viewPoint.add(showCalendar);
		viewPoint.add(showTree);
		menuBar.add(viewPoint);
		
		JMenu extras = new JMenu(IRessources.res.getString("0043"));
		
		options = new JMenuItem(IRessources.res.getString("0044"));
		options.setActionCommand("0010");
		options.addActionListener(controller);
		options.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, InputEvent.CTRL_MASK));
		options.setIcon(getPicture(IRessources.OPTIONS_IMG));
		
		//Backup erstellen
		backup = new JMenuItem(IRessources.res.getString("0197"));
		backup.setActionCommand("0039");
		backup.addActionListener(controller);
		backup.setIcon(getPicture(IRessources.BKP_C_IMG));
		backup.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_E, InputEvent.CTRL_MASK));
		
		//Backup wiederherstellen
		backupRedo = new JMenuItem(IRessources.res.getString("0198"));
		backupRedo.setActionCommand("0040");
		backupRedo.addActionListener(controller);
		backupRedo.setIcon(getPicture(IRessources.BKP_R_IMG));
		backupRedo.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_W, InputEvent.CTRL_MASK));
		
		extras.add(options);
		extras.add(new JSeparator());
		extras.add(backup);
		extras.add(backupRedo);
		
		menuBar.add(extras);
		
		JMenu helpAndInfo = new JMenu(IRessources.res.getString("0046"));
		
		JMenuItem help = new JMenuItem(IRessources.res.getString("0047"));
		help.setActionCommand("0017");
		help.addActionListener(getController());
		help.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_H, InputEvent.CTRL_MASK));
		help.setIcon(getPicture(IRessources.HELP_IMG));
		
		JMenuItem info = new JMenuItem(IRessources.res.getString("0049"));
		info.setActionCommand("0018");
		info.addActionListener(getController());
		info.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_I, InputEvent.CTRL_MASK));
		info.setIcon(getPicture(IRessources.INFO_IMG));
		
		helpAndInfo.add(help);
		helpAndInfo.add(new JSeparator());
		helpAndInfo.add(info);
		
		menuBar.add(helpAndInfo);
		
		this.setJMenuBar(menuBar);
		//---------------------------------------------------------------------------------
		
		
		//---------------------------------------------------------------------------------
		//			Init der ToolBar
		//---------------------------------------------------------------------------------
		JToolBar toolBar = new JToolBar();
		toolBar.setPreferredSize(new Dimension(this.getWidth(),25));
		toolBar.setFloatable(false);
		toolBar.setOpaque(true);
		
		JButton tBarNew = new JButton(getPicture(IRessources.NEW_IMG));
		tBarNew.setToolTipText(IRessources.res.getString("0017"));
		tBarNew.setActionCommand("0008");
		tBarNew.addActionListener(getController());
		
		tBarSave = new JButton(getPicture(IRessources.SAVE_IMG));
		tBarSave.setToolTipText(IRessources.res.getString("0019"));
		tBarSave.setActionCommand("0007");
		tBarSave.addActionListener(getController());
		
		tBarPrint = new JButton(getPicture(IRessources.PRINT_IMG));
		tBarPrint.setToolTipText(IRessources.res.getString("0051"));
		tBarPrint.setActionCommand("0030");
		tBarPrint.addActionListener(getController());
		
		JButton tBarReload = new JButton(getPicture(IRessources.RELOAD_IMG));
		tBarReload.setToolTipText(IRessources.res.getString("0053"));
		tBarReload.setActionCommand("0015");
		tBarReload.addActionListener(getController());
		
		JButton tBarLockApplication = new JButton(getPicture(IRessources.LOCK_IMG));
		tBarLockApplication.setToolTipText(IRessources.res.getString("0055"));
		tBarLockApplication.setActionCommand("0023");
		tBarLockApplication.addActionListener(getController());
		
		toolBar.add(tBarNew);
		toolBar.addSeparator();
		
		toolBar.add(tBarSave);
		toolBar.addSeparator();
		
		toolBar.add(tBarPrint);
		toolBar.addSeparator();
		
		toolBar.add(tBarReload);
		toolBar.addSeparator();
		
		toolBar.add(tBarLockApplication);
		
		this.add(toolBar,BorderLayout.NORTH);
		
		//---------------------------------------------------------------------------------

		
		//------Init Eintragsbaum----------------------------------------------------------
		rootTreeEntry = new DefaultMutableTreeNode(IRessources.res.getString("0056"));
		
		tree = new JTree(rootTreeEntry);
		tree.setCellRenderer(new TreeCellRendererWhiteNodes());
		tree.setOpaque(false);
		tree.addTreeSelectionListener(controller);
		tree.addTreeExpansionListener(controller);
		tree.addTreeWillExpandListener(controller);
		tree.addMouseListener(controller);
		tree.getSelectionModel().setSelectionMode(DefaultTreeSelectionModel.SINGLE_TREE_SELECTION);
		
		scrollPane = new JScrollPane(tree);
		scrollPane.getViewport().setOpaque(false);
		scrollPane.setOpaque(false);
		scrollPane.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.RAISED));
		
		entryPnl = new EmptyPanel();
		entryPnl.setLayout(new BorderLayout());
		entryPnl.add(scrollPane, BorderLayout.CENTER);
		entryPnl.setMinimumSize(new Dimension(200,0));
		entryPnl.setPreferredSize(new Dimension(200,0));
		//---------------------------------------------------------------------------------

		
		//-------Init Eintragsdatenbereich--------------------------------------------------
		//Setzt TabbedPanes im gesamten Programm durchsichtig.
		UIManager.put("TabbedPane.contentOpaque", Boolean.FALSE);
		
		JTabbedPane entryTp = new JTabbedPane();
		entryTp.setOpaque(false);
		
		JPanel entryTText = new JPanel();
		entryTText.setOpaque(false);
		
		JPanel entryTImg = new JPanel();
		entryTImg.setOpaque(false);
		
		//EintragText+EintragBilder Tabs adden
		entryTp.addTab(IRessources.res.getString("0175"), entryTText);
		entryTp.addTab(IRessources.res.getString("0176"), entryTImg);
		
		JPanel workingArea = new JPanel();
		workingArea.setOpaque(false);
		workingArea.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.RAISED));
		
		Font font = new Font("Verdana", Font.BOLD, 14);
		
		JLabel date = new JLabel(IRessources.res.getString("0057"));
		date.setFont(font);
		date.setForeground(Color.WHITE);
		
		tfDate = new SDTextField("tfDate");
		
		JLabel place = new JLabel(IRessources.res.getString("0058"));
		place.setFont(font);
		place.setForeground(Color.WHITE);
		
		tfPlace = new SDTextField("tfPlace");
		
		JLabel timeStart = new JLabel(IRessources.res.getString("0059"));
		timeStart.setFont(font);
		timeStart.setForeground(Color.WHITE);
		
		tfTimeStart = new SDTextField("tfTimeStart");
		
		JLabel timeEnd = new JLabel(IRessources.res.getString("0060"));
		timeEnd.setFont(font);
		timeEnd.setForeground(Color.WHITE);
		
		tfTimeEnd = new SDTextField("tfTimeEnd");
		
		JLabel description = new JLabel(IRessources.res.getString("0061"));
		description.setFont(font);
		description.setForeground(Color.WHITE);
		
		text = new SDTextArea("text");
		text.setLineWrap(true);
		
		JScrollPane textScrollPane = new JScrollPane(text);
		textScrollPane.getViewport().setOpaque(false);
		textScrollPane.setOpaque(false);
		textScrollPane.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.RAISED));
		textScrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		
		btnShowKeyWords = new JButton(IRessources.res.getString("0066")); 
		btnShowKeyWords.setToolTipText(IRessources.res.getString("0177"));
		btnShowKeyWords.setActionCommand("0032");
		btnShowKeyWords.addActionListener(controller);
		btnShowKeyWords.setEnabled(false);
		
		
		//	Eintrag-Bilder Tab Controls
		
			cbImgs = new JComboBox();
			DefaultComboBoxModel cbModel = new DefaultComboBoxModel(new String[]{IRessources.NO_IMG});
			cbImgs.setModel(cbModel);
			cbImgs.setActionCommand("0036");
			cbImgs.addActionListener(controller);
			
			btnAddImg = new JButton(IRessources.res.getString("0178"));
			btnAddImg.setActionCommand("0033");
			btnAddImg.addActionListener(controller);
			
			btnDeleteImg = new JButton(IRessources.res.getString("0179"));
			btnDeleteImg.setActionCommand("0034");
			btnDeleteImg.addActionListener(controller);
			
			btnImgDescription = new JButton(IRessources.res.getString("0066"));
			btnImgDescription.setActionCommand("0035");
			btnImgDescription.addActionListener(controller);
			btnImgDescription.setToolTipText(IRessources.res.getString("0189"));
		
			imgPnl = new BackGroundPane();
		//---------------------------------------------------------------------------------

		
		//----- BUTTONS Aendern, Loeschen, Speichern ------------------------------------------
		EmptyPanel bottomPnl = new EmptyPanel();
		
		//Werden benoetigt um Leerraueme in GridLayout des bottomPnl darzustellen
		EmptyPanel botPnl1 = new EmptyPanel();
		EmptyPanel botPnl2 = new EmptyPanel();
		
		btnChange = new JButton(IRessources.res.getString("0034"));
		btnChange.setActionCommand("0005");
		btnChange.addActionListener(controller);
		
		btnDelete = new JButton(IRessources.res.getString("0036"));
		btnDelete.setActionCommand("0006");
		btnDelete.addActionListener(controller);
		
		btnSave = new JButton(IRessources.res.getString("0019"));
		btnSave.setActionCommand("0007");
		btnSave.addActionListener(controller);
		//--------------------------------------------------------------------------------------
		
		
		//----Eintraege PopUpMenues bei Rechtsklick auf Eintrag-------------
		popSChange = new MenuItem("\u00C4ndern");
		popSChange.setActionCommand("0005");
		popSChange.addActionListener(controller);
		
		popSDel = new MenuItem("L\u00F6schen");
		popSDel.setActionCommand("0006");
		popSDel.addActionListener(controller);
		
		popMDelAll = new MenuItem("Alle L\u00F6schen");
		popMDelAll.setActionCommand("0031");
		popMDelAll.addActionListener(controller);
		//------------------------------------------------------------------
		
		
		//Statusleiste
		JToolBar status = new JToolBar();
		status.setFloatable(false);
		status.setOpaque(true);
		lblStatus = new JLabel();
		lblStatus.setFont(new Font("Verdana", Font.PLAIN, 12));
		status.add(lblStatus);
		
		
		//------- Layout by Matisse --------------------------------------------------------------------------
		javax.swing.GroupLayout entrysPnlLayout = new javax.swing.GroupLayout(
				entryPnl);
		entryPnl.setLayout(entrysPnlLayout);
		entrysPnlLayout.setHorizontalGroup(entrysPnlLayout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addGap(0, 250,
				Short.MAX_VALUE));
		entrysPnlLayout.setVerticalGroup(entrysPnlLayout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addGap(0, 538,
				Short.MAX_VALUE));

		javax.swing.GroupLayout entryTTextLayout = new javax.swing.GroupLayout(
				entryTText);
		entryTText.setLayout(entryTTextLayout);
		entryTTextLayout
				.setHorizontalGroup(entryTTextLayout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addComponent(textScrollPane,
								javax.swing.GroupLayout.DEFAULT_SIZE, 600,
								Short.MAX_VALUE)
						.addGroup(
								javax.swing.GroupLayout.Alignment.TRAILING,
								entryTTextLayout
										.createSequentialGroup()
										.addContainerGap()
										.addComponent(
												btnShowKeyWords,
												javax.swing.GroupLayout.PREFERRED_SIZE,
												18,
												javax.swing.GroupLayout.PREFERRED_SIZE)));
		entryTTextLayout.setVerticalGroup(entryTTextLayout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addGroup(
				javax.swing.GroupLayout.Alignment.TRAILING,
				entryTTextLayout.createSequentialGroup().addComponent(
						textScrollPane, javax.swing.GroupLayout.DEFAULT_SIZE,
						348, Short.MAX_VALUE).addPreferredGap(
						javax.swing.LayoutStyle.ComponentPlacement.RELATED)
						.addComponent(btnShowKeyWords,
								javax.swing.GroupLayout.PREFERRED_SIZE, 18,
								javax.swing.GroupLayout.PREFERRED_SIZE)));

		javax.swing.GroupLayout entryTImgLayout = new javax.swing.GroupLayout(
				entryTImg);
		entryTImg.setLayout(entryTImgLayout);
		entryTImgLayout
				.setHorizontalGroup(entryTImgLayout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								javax.swing.GroupLayout.Alignment.TRAILING,
								entryTImgLayout
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												entryTImgLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.TRAILING)
														.addComponent(
																imgPnl,
																javax.swing.GroupLayout.Alignment.LEADING,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																580,
																Short.MAX_VALUE)
														.addGroup(
																entryTImgLayout
																		.createSequentialGroup()
																		.addComponent(
																				cbImgs,
																				javax.swing.GroupLayout.PREFERRED_SIZE,
																				113,//97->urspruenglich, Anpassung checkbox fuer
																				//Bilder, dass Text "keine Auswahl" vollstaendig
																				//angezeigt wird.
																				javax.swing.GroupLayout.PREFERRED_SIZE)
																		.addPreferredGap(
																				javax.swing.LayoutStyle.ComponentPlacement.RELATED)
																		.addComponent(
																				btnImgDescription,
																				javax.swing.GroupLayout.PREFERRED_SIZE,
																				21,
																				javax.swing.GroupLayout.PREFERRED_SIZE)
																		.addPreferredGap(
																				javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
																		.addComponent(
																				btnDeleteImg,
																				javax.swing.GroupLayout.PREFERRED_SIZE,
																				112,
																				javax.swing.GroupLayout.PREFERRED_SIZE)
																		.addPreferredGap(
																				javax.swing.LayoutStyle.ComponentPlacement.RELATED,
																				220,
																				Short.MAX_VALUE)
																		.addComponent(
																				btnAddImg,
																				javax.swing.GroupLayout.PREFERRED_SIZE,
																				114,
																				javax.swing.GroupLayout.PREFERRED_SIZE)))
										.addContainerGap()));
		entryTImgLayout
				.setVerticalGroup(entryTImgLayout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								entryTImgLayout
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												entryTImgLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																cbImgs,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE)
														.addComponent(btnAddImg)
														.addComponent(
																btnImgDescription)
														.addComponent(
																btnDeleteImg,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE))
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
										.addComponent(
												imgPnl,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												316, Short.MAX_VALUE)
										.addContainerGap()));

		javax.swing.GroupLayout workingAreaLayout = new javax.swing.GroupLayout(
				workingArea);
		workingArea.setLayout(workingAreaLayout);
		workingAreaLayout
				.setHorizontalGroup(workingAreaLayout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								javax.swing.GroupLayout.Alignment.TRAILING,
								workingAreaLayout
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												workingAreaLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.TRAILING)
														.addComponent(
																entryTp,
																javax.swing.GroupLayout.Alignment.LEADING,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																605,
																Short.MAX_VALUE)
														.addGroup(
																workingAreaLayout
																		.createSequentialGroup()
																		.addGroup(
																				workingAreaLayout
																						.createParallelGroup(
																								javax.swing.GroupLayout.Alignment.LEADING)
																						.addComponent(
																								date)
																						.addComponent(
																								place))
																		.addGap(
																				18,
																				18,
																				18)
																		.addGroup(
																				workingAreaLayout
																						.createParallelGroup(
																								javax.swing.GroupLayout.Alignment.LEADING)
																						.addGroup(
																								workingAreaLayout
																										.createSequentialGroup()
																										.addComponent(
																												tfDate,
																												javax.swing.GroupLayout.PREFERRED_SIZE,
																												97,
																												javax.swing.GroupLayout.PREFERRED_SIZE)
																										.addPreferredGap(
																												javax.swing.LayoutStyle.ComponentPlacement.RELATED,
																												224,
																												Short.MAX_VALUE)
																										.addComponent(
																												timeStart)
																										.addGap(
																												18,
																												18,
																												18)
																										.addComponent(
																												tfTimeStart,
																												javax.swing.GroupLayout.PREFERRED_SIZE,
																												97,
																												javax.swing.GroupLayout.PREFERRED_SIZE))
																						.addGroup(
																								workingAreaLayout
																										.createSequentialGroup()
																										.addComponent(
																												tfPlace,
																												javax.swing.GroupLayout.PREFERRED_SIZE,
																												157,
																												javax.swing.GroupLayout.PREFERRED_SIZE)
																										.addPreferredGap(
																												javax.swing.LayoutStyle.ComponentPlacement.RELATED,
																												231,
																												Short.MAX_VALUE)
																										.addComponent(
																												timeEnd)
																										.addGap(
																												18,
																												18,
																												18)
																										.addComponent(
																												tfTimeEnd,
																												javax.swing.GroupLayout.PREFERRED_SIZE,
																												97,
																												javax.swing.GroupLayout.PREFERRED_SIZE)))))
										.addContainerGap()));
		workingAreaLayout
				.setVerticalGroup(workingAreaLayout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								workingAreaLayout
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												workingAreaLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(date)
														.addComponent(
																tfDate,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE)
														.addComponent(
																tfTimeStart,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE)
														.addComponent(timeStart))
										.addGap(18, 18, 18)
										.addGroup(
												workingAreaLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(place)
														.addComponent(
																tfPlace,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE)
														.addComponent(
																tfTimeEnd,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE)
														.addComponent(timeEnd))
										.addGap(18, 18, 18)
										.addComponent(
												entryTp,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												400, Short.MAX_VALUE)
										.addContainerGap()));
		
		bottomPnl.setLayout(new java.awt.GridLayout(1, 0));

		bottomPnl.add(btnChange);

		javax.swing.GroupLayout emptyPnl1Layout = new javax.swing.GroupLayout(
				botPnl1);
		botPnl1.setLayout(emptyPnl1Layout);
		emptyPnl1Layout.setHorizontalGroup(emptyPnl1Layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addGap(0, 125,
				Short.MAX_VALUE));
		emptyPnl1Layout.setVerticalGroup(emptyPnl1Layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addGap(0, 28,
				Short.MAX_VALUE));

		bottomPnl.add(botPnl1);
		
		bottomPnl.add(btnDelete);

		javax.swing.GroupLayout emptyPnl2Layout = new javax.swing.GroupLayout(
				botPnl2);
		botPnl2.setLayout(emptyPnl2Layout);
		emptyPnl2Layout.setHorizontalGroup(emptyPnl2Layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addGap(0, 125,
				Short.MAX_VALUE));
		emptyPnl2Layout.setVerticalGroup(emptyPnl2Layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addGap(0, 28,
				Short.MAX_VALUE));

		bottomPnl.add(botPnl2);

		bottomPnl.add(btnSave);

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(
				getContentPane());
		getContentPane().setLayout(layout);
		layout
				.setHorizontalGroup(layout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addComponent(toolBar,
								javax.swing.GroupLayout.DEFAULT_SIZE, 911,
								Short.MAX_VALUE)
						.addComponent(status,
								javax.swing.GroupLayout.DEFAULT_SIZE, 911,
								Short.MAX_VALUE)
						.addGroup(
								layout
										.createSequentialGroup()
										.addGap(8, 8, 8)
										.addComponent(
												entryPnl,
												javax.swing.GroupLayout.PREFERRED_SIZE,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												javax.swing.GroupLayout.PREFERRED_SIZE)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
										.addGroup(
												layout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.LEADING)
														.addComponent(
																workingArea,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																Short.MAX_VALUE)
														.addComponent(
																bottomPnl,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																629,
																Short.MAX_VALUE))
										.addContainerGap()));
		layout
				.setVerticalGroup(layout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								layout
										.createSequentialGroup()
										.addComponent(
												toolBar,
												javax.swing.GroupLayout.PREFERRED_SIZE,
												25,
												javax.swing.GroupLayout.PREFERRED_SIZE)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addGroup(
												layout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.LEADING)
														.addGroup(
																layout
																		.createSequentialGroup()
																		.addComponent(
																				workingArea,
																				javax.swing.GroupLayout.DEFAULT_SIZE,
																				javax.swing.GroupLayout.DEFAULT_SIZE,
																				Short.MAX_VALUE)
																		.addPreferredGap(
																				javax.swing.LayoutStyle.ComponentPlacement.RELATED)
																		.addComponent(
																				bottomPnl,
																				javax.swing.GroupLayout.PREFERRED_SIZE,
																				32,
																				javax.swing.GroupLayout.PREFERRED_SIZE))
														.addComponent(
																entryPnl,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																Short.MAX_VALUE))
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addComponent(
												status,
												javax.swing.GroupLayout.PREFERRED_SIZE,
												25,
												javax.swing.GroupLayout.PREFERRED_SIZE)));
		//----------------------------------------------------------------------------------------------------

		setPreferredSize(new Dimension(1000,800));
		
		//minimal benoetigte Aufloesung zum Ausfuehren von SD
		setMinimumSize(new Dimension(770,365));
		
		//Eintragsbaum anzeigen
		setTreeVisible();
		
		setControlsAtApplicationStart();

		lblStatus.setText(IRessources.res.getString("0062"));
		
		pack();
		
		this.setVisible(true);
	}
	
	/**
	 * @param img -> vollstaendiger path zur Bilddatei (bsp.: "/de/project/images/img.png")
	 * @return ImageIcon aus Pfad geladen
	 */
	private Icon getPicture(String img)
	{
		return new ImageIcon(getClass().getResource(img));
	}

	/**
	 * Gibt das DataSet des Views zurueck
	 * 
	 * @return dataSet
	 * @throws DataSetRowAlreadyExistsException 
	 */
	public DataSet getDataSet() throws DataSetRowAlreadyExistsException
	{
		DataSet dataSet = new DataSet();
		
		dataSet.addRow("date", tfDate.getText());
		dataSet.addRow("place", tfPlace.getText());
		dataSet.addRow("timeStart", tfTimeStart.getText());
		dataSet.addRow("timeEnd", tfTimeEnd.getText());
		dataSet.addRow("text", text.getText());
		dataSet.addRow("imageCount", cbImgs.getItemCount()-1);
		
		return dataSet;
	}
	
	/**
	 * Setzt das DataSet des Views
	 * clearTree -> True = Eintraege im JTree werden geloescht.
	 * 
	 * @param dataSet
	 * @param clearTree
	 * @throws Exception
	 */
	public void setDataSet(DataSet dataSet, boolean clearTree) throws Exception
	{
		if(!(dataSet.getRowsCount()>=5))
		{
			throw new Exception(IRessources.res.getString("0063"));
		}
		
		//Setzen des Flag IgnoreChanges, um keine rueckgaengig machbaren Aktion im UndoManager zu
		//erstellen, da dies nur durch Eingaben durch den Benutzer geschehen soll.
		SDUndoManager.getInstance().setIgnoreChanges(true);
		
		tfDate.setText(dataSet.getString("date"));
		tfPlace.setText(dataSet.getString("place"));
		tfTimeStart.setText(dataSet.getString("timeStart"));
		tfTimeEnd.setText(dataSet.getString("timeEnd"));
		text.setText(dataSet.getString("text"));
		
		//Holen Anzahl Bilder aus DataSet falls mitgeliefert.
		if(dataSet.contains("imageCount")&&dataSet.getInteger("imageCount")!=null)
		{
			int imgs = dataSet.getInteger("imageCount");
			
			String[] imageBuffer = new String[imgs+1];
			imageBuffer[0]=IRessources.NO_IMG;
			
			for(int x = 1;x<=imgs;x++)
			{
				imageBuffer[x] = "Bild "+x;
			}
			
			cbImgs.setModel(new DefaultComboBoxModel(imageBuffer));
			btnDeleteImg.setEnabled(false);
			
			setPictureToShow(null);
		}
		//Keine Bilder zum anzeigen
		else
		{
			cbImgs.setModel(new DefaultComboBoxModel(new String[]{IRessources.NO_IMG}));
		}
		cbImgs.repaint();
		
		//Zuruecksetzen des Flag IgnoreChanges
		SDUndoManager.getInstance().setIgnoreChanges(false);
		
		//Holen Eintraege DataSet falls vorhanden
		DataSet dsEntrys = null;
		if(dataSet.contains("dsEntrys"))
		{
			dsEntrys = dataSet.getDataSet("dsEntrys");
		}
		
		//Tritt nur auf bei aktuallisieren, wenn kein Eintrag mehr da ist
		if(clearTree&&(dsEntrys==null||dsEntrys.getRowsCount()==0))
		{
			//Auswahlfenster schliessen, falls geoeffnet, da nichts mehr ausgewaehlt werden koennen soll,
			//wenn kein Eintrag mehr da ist.
			controller.doCloseCalenderSelectSubEntryDialog();
			
			//nur alles auf Anfangszustand setzen wenn nicht gerade ein neuer Eintrag erstellt werden soll.
			//MANTIS 107
			if( (getController().isInChange() == false && btnSave.isEnabled()) == false )
			{
				//Setzen des Flag IgnoreChanges, um keine rueckgaengig machbaren Aktion im UndoManager zu
				//erstellen, da dies nur durch Eingaben durch den Benutzer geschehen soll.
				SDUndoManager.getInstance().setIgnoreChanges(true);
				
				//Textfelder leeren und Controls auf Ursprungszustand zurueck setzen, da es nicht sein kann, dass
				//noch ein Eintrag angezeigt wird wenn keiner mehr vorhanden ist.
				tfDate.setText(IRessources.EMPTY);
				tfPlace.setText(IRessources.EMPTY);
				tfTimeStart.setText(IRessources.EMPTY);
				tfTimeEnd.setText(IRessources.EMPTY);
				text.setText(IRessources.EMPTY);
				
				//Zuruecksetzen flag
				SDUndoManager.getInstance().setIgnoreChanges(false);
				
				setControlsAtApplicationStart();
				
				//Setzt Model auf Null da rein logisch kein Eintrag mehr da ist der im Model stehen koennte
				controller.setModel(null);
			}
			
			controller.getCalendar().doRemoveAllEntrys();
			controller.getCalendar().doUpdate();
			rootTreeEntry.removeAllChildren();
			tree.expandPath(new TreePath(rootTreeEntry.getPath()));
			tree.updateUI();
		}
		
		//Setzt die Eintraege fuer den JTree
		if(dsEntrys!=null&&dsEntrys.getRowsCount()!=0)
		{	
			//Speicherung aller aufgeklappten Eintrage, um diese spaeter ggf. wieder aufzuklappen
			ArrayList<String> eventsToExpand = getExpandedEvents();
			
			rootTreeEntry.removeAllChildren();
			controller.getCalendar().doRemoveAllEntrys();
			
			for (DataSetRow row : dsEntrys.getRows())
			{
				String entry = (String) row.getAttribute();
				
				//Wenn es an diesem Tag mehrere Eintraege gibt
				if(entry.contains("!"))
				{
					//Bilden EintragName
					String entryName = entry.substring(entry.lastIndexOf("!")+1);
					entryName = entryName.substring(entryName.indexOf("_")+1, entryName.lastIndexOf("."));
					entryName = entryName.replace('_', '.');
					
					DefaultMutableTreeNode parent = new DefaultMutableTreeNode( entryName );
					
					parent.add(new DefaultMutableTreeNode( entryName + " - 1" ));
					
					rootTreeEntry.add(parent);
				}
				else
				{
					String entryName = entry.substring(entry.indexOf("_")+1, entry.lastIndexOf("."));
					entryName = entryName.replace('_', '.');
					
					rootTreeEntry.add(new DefaultMutableTreeNode( entryName ));
					
					//Schleife zur Ermittlung ob ein Pfad, der aufgeklappt werden soll dem aktuellen Pfad
					//des DataSets entspricht. Wenn ja -> Entfernen aus Liste, da er keine SubEvents
					//									  (mehr) hat.
					for(int x=0;x<eventsToExpand.size();x++)
					{				
						if(eventsToExpand.get(x).equals(entryName))
						{
							eventsToExpand.remove(x);
						}
					}

				}
				controller.getCalendar().addEntry(entry);
			}
			
			
			//Calendar updaten
			controller.getCalendar().doUpdate();
			
			tree.expandPath(new TreePath(rootTreeEntry.getPath()));
			
			//Klappt alle Eintraege, die aufgeklappt waren wieder auf.
			if(eventsToExpand.size()>0)
			{
				doExpandPaths(eventsToExpand);
			}

			tree.updateUI();
		}
	}
	    
	/**
	 * Fuegt je nach Wert der Variable numberOfSubEvents Untereintraege zum uebergebenem Eintrag
	 * (treeStock) hinzu.
	 * 
	 * @param treeStock
	 * @param numberOfSubEvents
	 */
	public void expandTreeStock(String treeStock, int numberOfSubEvents)
	{
		TreeNode node = null;
		
		//Schleife fuer alle Eintraege im Baum
		for (Enumeration<?> childs = rootTreeEntry.children(); childs.hasMoreElements();) 
		{	
			node = (TreeNode)childs.nextElement();
			//Wenn der aktuelle Zweig der gesuchte ist
			if(node.toString().equals(treeStock))
			{
				//Fuegt fuer jedes SubEvent ein Blatt zu dem Zweig hinzu
				for(int x = 1;x<=numberOfSubEvents;x++)
				{
					DefaultMutableTreeNode defNode = (DefaultMutableTreeNode) node;
					defNode.add(new DefaultMutableTreeNode(treeStock + " - " + (x+1)));
				}
			}
		}
	}
	
	/**
	 * Setzt die Controls nachdem ein Eintrag geladen wurde
	 */
	public void setControlsAtLoadingEvent()
	{
		save.setEnabled(false);
		editEvent.setEnabled(true);
		delete.setEnabled(true);
		
		tBarSave.setEnabled(false);
		
		tfDate.setEditable(false);
		tfPlace.setEditable(false);
		tfTimeEnd.setEditable(false);
		tfTimeStart.setEditable(false);
		text.setEditable(false);
		
		btnChange.setEnabled(true);
		btnDelete.setEnabled(true);
		btnSave.setEnabled(false);
		
		print.setEnabled(true);
		tBarPrint.setEnabled(true);
		
		popSChange.setEnabled(true);
		
		btnAddImg.setEnabled(false);
		btnDeleteImg.setEnabled(false);
		btnImgDescription.setEnabled(false);
		
		btnShowKeyWords.setEnabled(true);
	}
	
	/**
	 * Setzt die Controls nach Klick auf Neu
	 */
	public void setControlsAtNewEvent()
	{
		save.setEnabled(true);
		editEvent.setEnabled(false);
		delete.setEnabled(false);
		
		tBarSave.setEnabled(true);
		
		tfDate.setEditable(true);
		tfPlace.setEditable(true);
		tfTimeEnd.setEditable(true);
		tfTimeStart.setEditable(true);
		text.setEditable(true);
		
		btnChange.setEnabled(false);
		btnDelete.setEnabled(false);
		btnSave.setEnabled(true);
		
		print.setEnabled(true);
		tBarPrint.setEnabled(true);
		
		btnAddImg.setEnabled(true);
		btnDeleteImg.setEnabled(false);
		btnImgDescription.setEnabled(false);
		
		btnShowKeyWords.setEnabled(true);
	}
	
	/**
	 * Setzt die Controls wenn ein Eintrag geaendert werden soll
	 */
	public void setControlsAtEditEvent(boolean isPictureSelected)
	{
		save.setEnabled(true);
		editEvent.setEnabled(false);
		delete.setEnabled(true);
		
		tBarSave.setEnabled(true);
		
		tfDate.setEditable(true);
		tfPlace.setEditable(true);
		tfTimeEnd.setEditable(true);
		tfTimeStart.setEditable(true);
		text.setEditable(true);
		
		btnChange.setEnabled(false);
		btnDelete.setEnabled(true);
		btnSave.setEnabled(true);
		
		popSChange.setEnabled(false);
		
		btnAddImg.setEnabled(true);
		btnDeleteImg.setEnabled(isPictureSelected);
		btnImgDescription.setEnabled(isPictureSelected);
	}
	
	/**
	 * Setzt die Controls nach Anwendungsstart (=Anfangszustand)
	 */
	public void setControlsAtApplicationStart()
	{
		save.setEnabled(false);
		editEvent.setEnabled(false);
		delete.setEnabled(false);
		
		//Wenn im UndoManager keine Aktionen gespeichert sind die rueckgaengig machbar sind
		// -> Disablen des MenuItems undoEvent
		if(!(SDUndoManager.getInstance().getActions().size()>0))
		{
			undoEvent.setEnabled(false);
		}
		
		tBarSave.setEnabled(false);
		
		tfDate.setEditable(false);
		tfPlace.setEditable(false);
		tfTimeEnd.setEditable(false);
		tfTimeStart.setEditable(false);
		text.setEditable(false);
		
		btnChange.setEnabled(false);
		btnDelete.setEnabled(false);
		btnSave.setEnabled(false);
		
		print.setEnabled(false);
		tBarPrint.setEnabled(false);
		
		btnAddImg.setEnabled(false);
		btnDeleteImg.setEnabled(false);
		btnImgDescription.setEnabled(false);
		
		btnShowKeyWords.setEnabled(false);
	}
	
	/**
	 * True -> Menuepunkt Extras->Option enabled
	 * 
	 * @param enable
	 */
	public void setControlOptionsEnable(boolean enable)
	{
		options.setEnabled(enable);
	}
	
	/**
	 * Methode um eine Komponente (c) zu einem Container (cnt) mit GridBagLayout (gbl) hinzuzufuegen.
	 * 
	 * @param cnt
	 * @param gbl
	 * @param c
	 * @param x
	 * @param y
	 * @param width
	 * @param height
	 * @param weightx
	 * @param weighty
	 */
	private void addComponent(Container cnt,GridBagLayout gbl, Component c, int x, int y, int width, 
			int height, double weightx, double weighty )
	{
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.fill = GridBagConstraints.BOTH;
		gbc.gridx = x; 
		gbc.gridy = y;
		gbc.gridwidth = width; 
		gbc.gridheight = height;
		gbc.weightx = weightx; 
		gbc.weighty = weighty;
		gbl.setConstraints( c, gbc );
		cnt.add( c );
	}
	
	/**
	 * Gibt eine Liste aller Eintraege zurueck, die aufgeklappt sind.
	 * 
	 * @return expandedEvents
	 */
	private ArrayList<String> getExpandedEvents()
	{
		ArrayList<String> expandedEvents = new ArrayList<String>();
			
		//Geht alle Eintrage durch
		for(int x = 0;x<rootTreeEntry.getChildCount();x++)
		{
			//Holt sich childElement an Stelle x
			DefaultMutableTreeNode childNote = (DefaultMutableTreeNode) rootTreeEntry.getChildAt(x);
			
			//Wenn aktuelles ChildElement aufgeklappt ist -> Speichern des Names in ArrayList
			if(tree.isExpanded(new TreePath(childNote.getPath())))
			{
				//Hohlt sich den Namen des aktuellen childElements an Stelle x
				String path = new TreePath(childNote.getPath()).getLastPathComponent().toString();
				expandedEvents.add(path);
			}
		}
		return expandedEvents;
	}
	
	/**
	 * Klappt alle in der Liste enthaltene Pfade (sofern im Tree vorhanden) auf.
	 * 
	 * @param eventsToExpand
	 * @throws Exception 
	 */
	public void doExpandPaths(ArrayList<String> eventsToExpand) throws Exception
	{
		//Speichert Zustand der Anwendung, da dieser durch das Aufklappen der Eintraege geaendert wird
		// (Model wird neu gesetzt, DataSet des Views wird leer gesetzt, Zustaende der Controls werden
		//  geaendert).    - MANTIS 87 -
		DataSet dsSaveStatus = saveApplicationStatus();
		
		for(int x=0;x<eventsToExpand.size();x++)
		{
			//Geht alle Eintraege im Tree durch
			for(int y = 0;y<rootTreeEntry.getChildCount();y++)
			{
				//Aktuelles ChildElement an Stelle y
				DefaultMutableTreeNode childNote = (DefaultMutableTreeNode) rootTreeEntry.getChildAt(y);
				//Namen des aktuellen Eintrages an Stelle y
				String path = new TreePath(childNote.getPath()).getLastPathComponent().toString();
					
				//Wenn der aktuelle Eintrag im Tree dem aktuellen Eintrag in der Liste entspricht
				// -> Eintrag aufklappen
				if(path.equals(eventsToExpand.get(x)))
				{
					tree.expandPath(new TreePath(childNote.getPath()));
				}
			}
		}
		
		//Zuruecksetzen des Anwendungszustands vor Ausklappen der Eintraege
		setBackApplicationStatus(dsSaveStatus);
	}

	/**
	 * Setzt das MenuItem "Rueckgaengig" aktiv.
	 * 
	 * @param active
	 */
	public void setUndoActive(boolean active)
	{
		this.undoEvent.setEnabled(active);
	}
	
	/**
	 * Sperrt/Entsperrt die Anwendung.
	 * 
	 * @param locked
	 * @throws DataSetRowAlreadyExistsException
	 */
	public void setApplicationLocked(boolean locked) throws DataSetRowAlreadyExistsException
	{
		if(locked)
		{
			this.setVisible(false);
			this.setContentPane(getLockedContentPanel());
			
			//Speichern der Zustaende der Eintraege in der Menubar.
			dsMenuBar = new DataSet();
			
			dsMenuBar.addRow("save", save.isEnabled());
			dsMenuBar.addRow("undoEvent", undoEvent.isEnabled());
			dsMenuBar.addRow("cutEvent", cutEvent.isEnabled());
			dsMenuBar.addRow("copyEvent", copyEvent.isEnabled());
			dsMenuBar.addRow("pasteEvent", pasteEvent.isEnabled());
			dsMenuBar.addRow("editEvent", editEvent.isEnabled());
			dsMenuBar.addRow("delete", delete.isEnabled());
			dsMenuBar.addRow("options", options.isEnabled());
			dsMenuBar.addRow("showCalendar", showCalendar.isEnabled());
			dsMenuBar.addRow("showTree", showTree.isEnabled());
			dsMenuBar.addRow("backup", backup.isEnabled());
			dsMenuBar.addRow("backupRedo", backupRedo.isEnabled());
			dsMenuBar.addRow("print", print.isEnabled());
			
			newEvent.setEnabled(false);
			save.setEnabled(false);
			undoEvent.setEnabled(false);
			cutEvent.setEnabled(false);
			copyEvent.setEnabled(false);
			pasteEvent.setEnabled(false);
			editEvent.setEnabled(false);
			delete.setEnabled(false);
			options.setEnabled(false);
			showCalendar.setEnabled(false);
			showTree.setEnabled(false);
			backup.setEnabled(false);
			backupRedo.setEnabled(false);
			search.setEnabled(false);
			print.setEnabled(false);
			
			this.setVisible(true);
		}
		else
		{
			//Wiederherstellen der Zustaende der Eintraege der Menubar.
			newEvent.setEnabled(true);
			save.setEnabled((Boolean) dsMenuBar.getRowValueByName("save"));
			undoEvent.setEnabled((Boolean) dsMenuBar.getRowValueByName("undoEvent"));
			cutEvent.setEnabled((Boolean) dsMenuBar.getRowValueByName("cutEvent"));
			copyEvent.setEnabled((Boolean) dsMenuBar.getRowValueByName("copyEvent"));
			pasteEvent.setEnabled((Boolean) dsMenuBar.getRowValueByName("pasteEvent"));
			editEvent.setEnabled((Boolean) dsMenuBar.getRowValueByName("editEvent"));
			delete.setEnabled((Boolean) dsMenuBar.getRowValueByName("delete"));
			options.setEnabled((Boolean) dsMenuBar.getRowValueByName("options"));
			showCalendar.setEnabled((Boolean)dsMenuBar.getRowValueByName("showCalendar"));
			showTree.setEnabled((Boolean)dsMenuBar.getRowValueByName("showTree"));
			backup.setEnabled(dsMenuBar.getBoolean("backup"));
			backupRedo.setEnabled(dsMenuBar.getBoolean("backupRedo"));
			search.setEnabled(true);
			print.setEnabled(dsMenuBar.getBoolean("print"));
			
			dsMenuBar = null;
			
			setVisible(false);
			setContentPane(standardContentPanel);
			setVisible(true);
		}
	}
	
	/**
	 * Gibt das Panel zurck, dass angezeigt werden soll, wenn die Anwendung gesperrt wird.
	 * 
	 * @return lockedContentPanel
	 */
	private JPanel getLockedContentPanel()
	{
		if(lockedContentPanel==null)
		{
			lockedContentPanel = new BackGroundPane(IRessources.BG_IMAGE);
			
			GridBagLayout grid = new GridBagLayout();
			
			lockedContentPanel.setLayout(grid);
			
			JLabel lblLocked = new JLabel(IRessources.res.getString("0064"));
			lblLocked.setFont(new Font("Verdana", Font.BOLD, 14));
			lblLocked.setForeground(Color.WHITE);
			
			btnLocked = new JButton(IRessources.res.getString("0065"));
			btnLocked.setActionCommand("0024");
			btnLocked.addActionListener(controller);
			
			addComponent(lockedContentPanel, grid, new EmptyPanel(), 0, 0, 3, 1, 1.0, 0.47);
			
			addComponent(lockedContentPanel, grid, new EmptyPanel(), 0, 1, 1, 1, 0.495, 0.03);
			addComponent(lockedContentPanel, grid, lblLocked,    1, 1, 1, 1, 0.01, 0.03);
			addComponent(lockedContentPanel, grid, new EmptyPanel(), 2, 1, 1, 1, 0.495, 0.03);
			
			addComponent(lockedContentPanel, grid, new EmptyPanel(), 0, 2, 1, 1, 0.495, 0.03);
			addComponent(lockedContentPanel, grid, btnLocked,    1, 2, 1, 1, 0.01, 0.03);
			addComponent(lockedContentPanel, grid, new EmptyPanel(), 2, 2, 1, 1, 0.495, 0.03);
			
			addComponent(lockedContentPanel, grid, new EmptyPanel(), 0, 3, 3, 1, 1.0, 0.47);
		}
		
		//Muss auf enabled gesetzt werden, da er sonst ab dem zweiten mal sperren immer deaktiviert ist,
		//da er beim ersten Mal deaktiviert wird, sobald sich der Dialog zur Eingabe der Userdaten oeffnet
		btnLocked.setEnabled(true);
		
		return lockedContentPanel;
	}
	
	/**
	 * Setzt ob der Button "Entsperren" enabled ist oder nicht.
	 * 
	 * @param enabled
	 */
	public void setUnlockButtonEnabled(boolean enabled)
	{
		btnLocked.setEnabled(enabled);
	}
	
	/**
	 * Entfernt den Tree aus dem View und zeigt den Kalender an
	 */
	public void setCalendarVisible()
	{
		entryPnl.removeAll();
		entryPnl.setLayout(new GridLayout(2,1));
		entryPnl.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.RAISED));
		entryPnl.add(controller.getCalendar().getPanel());
		entryPnl.add(getCalendarBottomPanel());
		showCalendar.setEnabled(false);
		showTree.setEnabled(true);
		repaint();
	}
	
	/**
	 * Entfernt den Kalender aus dem View und zeigt den Tree an.
	 */
	public void setTreeVisible()
	{
		entryPnl.removeAll();
		entryPnl.setLayout(new BorderLayout());
		entryPnl.setBorder(null);
		entryPnl.add(scrollPane, BorderLayout.CENTER);
		showTree.setEnabled(false);
		showCalendar.setEnabled(true);
		repaint();
	}
	
	/**
	 * Gibt das Panel zurueck, das ganz unten in dem Panel des Kalenders angezeigt wird.
	 * Dieses Panel erklaert was die Farben der einzelnen Datums Kaestchen zu bedeuten haben.
	 * 
	 * @return calendarBottomPanel
	 */
	public EmptyPanel getCalendarBottomPanel()
	{
		//Erzeugen Instanz, falls noch nicht geschehen
		if(calendarBottomPanel == null)
		{
			calendarBottomPanel = new EmptyPanel();
			calendarBottomPanel.setLayout(new BorderLayout());
			
			JPanel pnlExplainColors = new JPanel();
			pnlExplainColors.setLayout(new BorderLayout());
			
			
			JLabel lblSingleEntry = new JLabel(IRessources.res.getString("0066"));
			lblSingleEntry.setBackground(Color.green);
			lblSingleEntry.setOpaque(true);
			
			JLabel lblExplSingleEntry = new JLabel(IRessources.res.getString("0067"));
			
			JPanel singleEntryPnl = new JPanel();
			singleEntryPnl.add(lblSingleEntry);
			singleEntryPnl.add(lblExplSingleEntry);
			
			
			JLabel lblMultiEntry = new JLabel(IRessources.res.getString("0066"));
			lblMultiEntry.setBackground(Color.orange);
			lblMultiEntry.setOpaque(true);
			
			JLabel lblExplMultiEntry = new JLabel(IRessources.res.getString("0068"));
			
			JPanel multiEntryPnl = new JPanel();
			multiEntryPnl.add(lblMultiEntry);
			multiEntryPnl.add(lblExplMultiEntry);
			
			
			pnlExplainColors.add(singleEntryPnl, BorderLayout.NORTH);
			pnlExplainColors.add(multiEntryPnl, BorderLayout.CENTER);
			
			calendarBottomPanel.add(pnlExplainColors, BorderLayout.SOUTH);
		}
		return calendarBottomPanel;
	}
	
	/**
	 * Setzt den Text in der Statusleiste.
	 * 
	 * @param text
	 */
	public synchronized void setStatusLineText(String text, boolean bold)
	{
		if(bold)
		{
			lblStatus.setFont(new Font("Verdana", Font.BOLD, 14));
		}
		else
		{
			lblStatus.setFont(new Font("Verdana", Font.PLAIN, 12));
		}
		
		lblStatus.setText(" "+text);
	}
	
	/**
	 * Setzt ob die entsprechenden MenuItems aktiv sind oder nicht.
	 * 
	 * @param isCutEnabeld
	 * @param isCopyEnabled
	 * @param isPasteEnabled
	 */
	public void setCutCopyPasteEnabled(boolean isCutEnabeld, boolean isCopyEnabled, 
			boolean isPasteEnabled)
	{
		cutEvent.setEnabled(isCutEnabeld);
		copyEvent.setEnabled(isCopyEnabled);
		pasteEvent.setEnabled(isPasteEnabled);
	}
	
	/**
	 * Gibt die TextComponent, die den Focus hat zurueck.
	 * Hat keine TextComponent den Focus -> return null
	 * 
	 * @return JTextComponent/null
	 */
	public JTextComponent getFocusedTextComponent()
	{
		JTextComponent comps[] = new JTextComponent[]
		{
			tfDate, tfPlace, tfTimeEnd, tfTimeStart, text	
		};
		
		for (JTextComponent actComp : comps)
		{
			if(actComp.hasFocus())
			{
				return actComp;
			}
		}
		
		return null;
	}
	
	/**
	 * Speichert den Zustand der Anwendung in ein DataSet
	 * D.h.: DataSet des Views, Model und die Zustaende der Controls
	 * 
	 * @return dataSet
	 * @throws DataSetRowAlreadyExistsException
	 */
	private DataSet saveApplicationStatus() throws DataSetRowAlreadyExistsException
	{
		DataSet dataSet = new DataSet();
		
		//DataSet des Views speichern + Model des Controllers
		dataSet.addRow("orgDataSet", getDataSet());
		dataSet.addRow("model", controller.getModel());
		
		//Zustaende der Controls speichern
		dataSet.addRow("save", save.isEnabled());
		dataSet.addRow("editEvent", editEvent.isEnabled());
		dataSet.addRow("delete", delete.isEnabled());
		dataSet.addRow("tBarSave", tBarSave.isEnabled());
		dataSet.addRow("btnChange", btnChange.isEnabled());
		dataSet.addRow("btnDelete", btnDelete.isEnabled());
		dataSet.addRow("btnSave", btnSave.isEnabled());
		dataSet.addRow("tfDate", tfDate.isEditable());
		dataSet.addRow("tfPlace", tfPlace.isEditable());
		dataSet.addRow("tfTimeEnd", tfTimeEnd.isEditable());
		dataSet.addRow("tfTimeStart", tfTimeStart.isEditable());
		dataSet.addRow("text", text.isEditable());
		dataSet.addRow("print", print.isEnabled());
		dataSet.addRow("tBarPrint", tBarPrint.isEnabled());
		dataSet.addRow("popSChange", popSChange.isEnabled());
		dataSet.addRow("btnAddImg", btnAddImg.isEnabled());
		dataSet.addRow("btnDeleteImg", btnDeleteImg.isEnabled());
		dataSet.addRow("btnImgDescription", btnImgDescription.isEnabled());
		dataSet.addRow("btnShowKeyWords", btnShowKeyWords.isEnabled());
		
		return dataSet;
	}
	
	/**
	 * Setzt einen in ein DataSet gespeicherten Zustand der Anwendung zurueck.
	 * D.h.: DataSet des Views, Model und die Zustaende der Controls
	 * 
	 * @param dataSet
	 * @throws Exception
	 */
	private void setBackApplicationStatus(DataSet dataSet) throws Exception
	{
		setDataSet(dataSet.getDataSet("orgDataSet"), false);
		controller.setModel((DiaryEvent)dataSet.getRowValueByName("model"));
		
		save.setEnabled(dataSet.getBoolean("save"));
		editEvent.setEnabled(dataSet.getBoolean("editEvent"));
		delete.setEnabled(dataSet.getBoolean("delete"));
		tBarSave.setEnabled(dataSet.getBoolean("tBarSave"));
		btnChange.setEnabled(dataSet.getBoolean("btnChange"));
		btnDelete.setEnabled(dataSet.getBoolean("btnDelete"));
		btnSave.setEnabled(dataSet.getBoolean("btnSave"));
		tfDate.setEditable(dataSet.getBoolean("tfDate"));
		tfPlace.setEditable(dataSet.getBoolean("tfPlace"));
		tfTimeEnd.setEditable(dataSet.getBoolean("tfTimeEnd"));
		tfTimeStart.setEditable(dataSet.getBoolean("tfTimeStart"));
		text.setEditable(dataSet.getBoolean("text"));
		print.setEnabled(dataSet.getBoolean("print"));
		tBarPrint.setEnabled(dataSet.getBoolean("tBarPrint"));
		popSChange.setEnabled(dataSet.getBoolean("popSChange"));
		btnAddImg.setEnabled(dataSet.getBoolean("btnAddImg"));
		btnDeleteImg.setEnabled(dataSet.getBoolean("btnDeleteImg"));
		btnImgDescription.setEnabled(dataSet.getBoolean("btnImgDescription"));
		btnShowKeyWords.setEnabled(dataSet.getBoolean("btnShowKeyWords"));
	}
	
	/**
	 * Zeigt das PopUp Menue an, wenn auf einen einzelnen Eintrag mit der rechten Maustaste geklickt wird.
	 * Als einzelner Eintrag zaehlt hier auch ein Untereintrag.
	 * 
	 * @param x
	 * @param y
	 */
	public void setPopupMenuForSingleEntry(int x, int y)
	{
		PopupMenu popSingle = new PopupMenu();
		
		popSingle.add(popSChange);
		popSingle.add(popSDel);
		
		add(popSingle);
		
		//Tree wird angezeigt
		if(showCalendar.isEnabled())
		{
			popSingle.show(tree, x, y);
		}
		//Kalendar wird angezeigt
		else
		{
			popSingle.show(controller.getCalendar().getPanel(), x, y);
		}
	}

	/**
	 * Zeigt das PopUp Menue an, wenn auf einen Ueberpunkt im Tree mit der rechten Maustaste geklickt wurde.
	 * 
	 * @param x
	 * @param y
	 */
	public void setPopupMenuForMultiEntry(int x, int y) 
	{
		PopupMenu popMulti = new PopupMenu();
		
		popMulti.add(popMDelAll);
		
		add(popMulti);
		
		//Tree wird angezeigt
		if(showCalendar.isEnabled())
		{
			popMulti.show(tree, x, y);
		}
		//Kalendar wird angezeigt
		else
		{
			popMulti.show(controller.getCalendar().getPanel(), x, y);
		}
	}
	
	/**
	 * Zeigt das uebergebene Bild im Tab Eintrag-Bild im ImagePanel an.
	 * 
	 * @param img
	 */
	public void setPictureToShow(BufferedImage img)
	{
		if(img == null)
		{
			btnDeleteImg.setEnabled(false);
			btnImgDescription.setEnabled(false);
		}
		else
		{
			//Button "Bild loeschen" nur aktiv setzen wenn Eintrag im  Aenderungsstatus/Neustatus ist.
			if(btnChange.isEnabled()==false)
			{
				btnDeleteImg.setEnabled(true);
			}
			
			//Button "Bildbeschreibung" immer aktiv setzen, da Beschreibung auch angesehen werden soll koennen
			//wenn Eintrag sicht nicht im ChangeStatus befindet.
			btnImgDescription.setEnabled(true);
		}
		
		imgPnl.setImg(img);
		imgPnl.repaint();
	}
	
	/**
	 * @return ausgewaehltes Bild in der ComboBox
	 */
	public int getSelectedImage()
	{
		return cbImgs.getSelectedIndex();
	}
	
	/**
	 * Setzt das ausgewaehlte Bild in der ComboBox
	 * 
	 * @param index
	 */
	public void setSelectedImage(int index)
	{
		cbImgs.setSelectedIndex(index);
	}

	/**
	 * Gibt true zurueck, wenn der aktuelle Eintrag gespeichert werden kann.
	 * D.h. er wird entweder geaendert oder es wird ein neuer erstellt.
	 * 
	 * @return
	 */
	public boolean canEntryBeSaved() 
	{
		return btnSave.isEnabled();
	}
}