package de.joerg.diary.frontend.general;

import java.awt.Desktop;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import javax.imageio.ImageIO;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JTree;
import javax.swing.SwingWorker;
import javax.swing.event.MenuEvent;
import javax.swing.event.MenuListener;
import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeExpansionListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.event.TreeWillExpandListener;
import javax.swing.filechooser.FileFilter;
import javax.swing.text.JTextComponent;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.ExpandVetoException;
import sun.misc.BASE64Decoder;
import sun.misc.BASE64Encoder;
import de.joerg.diary.backend.exceptions.FileForThisUserInvalidException;
import de.joerg.diary.backend.exceptions.FileInvalidException;
import de.joerg.diary.backend.fileManagement.DiaryEventFileManager;
import de.joerg.diary.frontend.sdBasicComponents.ViewController;
import de.joerg.diary.frontend.sdCalendar.ICalendarListener;
import de.joerg.diary.frontend.sdCalendar.SDCalendar;
import de.joerg.diary.frontend.sdCalendar.SDCalendarClickEvent;
import de.joerg.diary.frontend.sdCalendar.SDCalendarRightClickEvent;
import de.joerg.diary.frontend.sdCalendar.SDCalendarSelectSubEntryVC;
import de.joerg.diary.frontend.sdUndoManager.SDUndoManager;
import de.joerg.diary.frontend.sdUndoManager.UndoActionForAddPicture;
import de.joerg.diary.frontend.sdUndoManager.UndoActionForChange;
import de.joerg.diary.frontend.sdUndoManager.UndoActionForDelete;
import de.joerg.diary.frontend.sdUndoManager.UndoActionForDeletePicture;
import de.joerg.diary.frontend.sdUndoManager.UndoActionForMultiEntrySave;
import de.joerg.diary.frontend.sdUndoManager.UndoActionForSingleEntrySave;
import de.joerg.diary.frontend.tools.SDPrinter;
import de.joerg.diary.shared.DiaryLogger;
import de.joerg.diary.shared.IRessources;
import de.joerg.diary.shared.SDSharedServices;
import de.joerg.diary.shared.dataSet.DataSet;
import de.joerg.diary.shared.dataSet.DataSetRowAlreadyExistsException;
import de.joerg.diary.shared.models.DiaryEvent;
import de.joerg.diary.shared.models.SDPicture;
import de.joerg.diary.shared.models.User;

/*
 *  SimpleDiary Version 2.0.0
    Copyright (C) 2010  Andreas Joerg (E-Mail: simpleDiary@freenet.de)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * ViewController fuer ein HauptFenster
 * 
 * @author Andreas Joerg
 */
public class MainWindowVC extends ViewController implements ActionListener, TreeSelectionListener, 
TreeExpansionListener, TreeWillExpandListener, WindowListener, ICalendarListener, MenuListener, MouseListener
{
	private OptionsVC optionsCtrl = null;
	private MainWindowV view = null;
	private User user = null;
	private String savePath = null;
	private DiaryEvent model = null;
	private boolean isInChange = false;
	private SDCalendar calendar = null;
	private boolean isCalendarSelectionDialogOpenend = false;
	private Thread setStatusBackThread = null;
	private SDCalendarSelectSubEntryVC calSelectSubEntryCtrl = null;
	private String selectedEvent = null;
	private boolean viewLocked = false;
	private String selectedMultiEvent = null;
	boolean isValueChangeFiredByRightClick = false;
	private DiaryEvent copyModel = null;
	private ImageDescriptionVC imgDescVC = null;
	private EntryKeywordsVC entryKeywordsVC = null;
	
	/**
	 * Erweiterter Konstruktor
	 * 
	 * @param user
	 */
	public MainWindowVC(User user)
	{
		try 
		{
			setCalendar(new SDCalendar(this));
			
			setUser(user);

			setSavePath(user.getSavePath());
			
			setView(new MainWindowV(this));
			view.init();
			updateView();
			
			SDUndoManager.getInstance().setController(this);
		}
		catch (Exception ex) 
		{
			handleException(ex.getMessage(), getClass().getName(), "constructor", ex);
		} 
	}
	
	//-------------Getter/Setter------------------------------
	public void setView(MainWindowV view) 
	{
		this.view = view;
	}
	public MainWindowV getView() 
	{
		return view;
	}

	public User getUser() 
	{
		return user;
	}
	public void setUser(User user) 
	{
		this.user = user;
	}

	public String getSavePath() 
	{
		return savePath;
	}
	public void setSavePath(String savePath) 
	{
		//Wenn der Speicherpfad geandert wird und ein Calendar Auswahl Dialog offen ist muss dieser
		//geschlossen werden, da er sich auf einen Eintrag im falschen Pfad bezieht.
		if(!savePath.equals(this.savePath))
		{
			if(isCalendarSelectionDialogOpenend)
			{
				doCloseCalenderSelectSubEntryDialog();
			}
		}
		
		this.savePath = savePath;
	}

	public DiaryEvent getModel() 
	{
		return model;
	}
	public void setModel(DiaryEvent model) 
	{
		this.model = model;
	}

	public boolean isInChange() 
	{
		return isInChange;
	}
	public void setInChange(boolean isInChange) 
	{
		this.isInChange = isInChange;
	}
	
	public DiaryEvent getCopyModel()
	{
		return copyModel;
	}
	public void setCopyModel(DiaryEvent copyModel) 
	{
		this.copyModel = copyModel;
	}

	public void setCalendar(SDCalendar calendar)
	{
		this.calendar = calendar;
	}
	public SDCalendar getCalendar() 
	{
		return calendar;
	}
	
	public void setCalendarSelectionDialogOpenend(boolean isCalendarSelectionDialogOpenend) 
	{
		this.isCalendarSelectionDialogOpenend = isCalendarSelectionDialogOpenend;
	}
	public boolean isCalendarSelectionDialogOpenend() 
	{
		return isCalendarSelectionDialogOpenend;
	}
	
	public boolean isViewLocked()
	{
		return viewLocked;
	}
	
	public ImageDescriptionVC getImgDescVC()
	{
		return imgDescVC;
	}
	
	public SDCalendarSelectSubEntryVC getCalSelectSubEntryCtrl() 
	{
		return calSelectSubEntryCtrl;
	}
	public void setCalSelectSubEntryCtrl(SDCalendarSelectSubEntryVC calSelectSubEntryCtrl) 
	{
		this.calSelectSubEntryCtrl = calSelectSubEntryCtrl;
	}
	
	public String getSelectedEvent()
	{
		return selectedEvent;
	}
	//---------------------------------------------------------------

	

	/**
	 * Aktualisiert die Eintraege im View
	 */
	public void updateView()
	{
		try 
		{
			DataSet dsEntrys = new DataSet();
			DataSet dsView = view.getDataSet();
			
			DiaryEventFileManager dEvtFileMgr = new DiaryEventFileManager();
			ArrayList<String> evtFiles;
			
			//Holen der EventFiles fuer den aktuellen User
			evtFiles = dEvtFileMgr.getListOfEventFiles(user, savePath);
			//Holen der PartenFiles fuer das Verzeichniss des Users
			ArrayList<String> parentFiles = dEvtFileMgr.loadParentObjectsFile(savePath);
			boolean isMultiple = false;
			
			//Schleife fuer alle Eintraege
			for(int x = 0;x<evtFiles.size();x++)
			{
				isMultiple = false;
				
				//Schleife fuer alle Eintraege der parentFiles
				for (int y = 0;y<parentFiles.size();y++)
				{
					//Wenn der aktuelle Eintrag ein ParentFile ist
					if(evtFiles.get(x).equals(parentFiles.get(y)))
					{
						isMultiple = true;
						continue;
					}
				}
				
				//Wenn der aktuelle Eintrag ein ParentFile ist
				if(isMultiple)
				{
					dsEntrys.addRow("entry"+x, "!"+evtFiles.get(x));
				}
				else
				{
					dsEntrys.addRow("entry"+x, evtFiles.get(x));
				}
			}
		
			dsView.addRow("dsEntrys", dsEntrys);
			view.setDataSet(dsView, true);
			
			//Pruefung nach Aktualisierung ob geoeffneter Eintrag noch existiert. Wenn nicht Anwendung auf
			//Anfangszustand setzen. (s. MANTIS 114)
			if(model!=null&&selectedEvent!=null)
			{
				DiaryEvent tempEvt = null;
				
				try
				{
					tempEvt = dEvtFileMgr.loadDiaryEventFromFile(savePath, 
							SDSharedServices.getFileName(user.getUserID(), model.getDateOfEvent()),
							user);
				}
				catch(FileNotFoundException ex)
				{
					//Setzen des DataSets des Views
					doResetViewDataSet();
					
					view.setControlsAtApplicationStart();
					setInChange(false);
					model = null;
					selectedEvent = null;
					copyModel = null;
					view.setPictureToShow(null);
					
					updateView();
					
					return;
				}
				
				if(model.equals(tempEvt)==false)
				{
					//Setzen des DataSets des Views
					doResetViewDataSet();
					
					view.setControlsAtApplicationStart();
					setInChange(false);
					model = null;
					selectedEvent = null;
					copyModel = null;
					view.setPictureToShow(null);
					
					updateView();
				}
			}
		} 
		catch (Exception ex) 
		{
			handleException(ex.getMessage(), getClass().getName(), "updateView", ex);
		}
	}
	
	@Override
	/**
	 * Reagiert auf ActionEvents des Views
	 */
	public void actionPerformed(ActionEvent evt) 
	{
		try
		{
			//Aendern
			if(evt.getActionCommand().equals("0005"))
			{
				doChange();
			}
			//Loeschen
			else if(evt.getActionCommand().equals("0006"))
			{			
				doDelete();
			}
			//Speichern
			else if(evt.getActionCommand().equals("0007"))
			{
				doSave();
			}
			//Neu
			else if(evt.getActionCommand().equals("0008"))
			{
				doSetApplicationForNewEntry();
			}
			//Beenden
			else if(evt.getActionCommand().equals("0009"))
			{
				doExit();
			}
			//Optionen
			else if(evt.getActionCommand().equals("0010"))
			{
				doShowOptions();
			}
			//ToolBar Reload
			else if(evt.getActionCommand().equals("0015"))
			{
				doToolBarReload();
			}
			//Hilfe
			else if(evt.getActionCommand().equals("0017"))
			{
				doShowHelp();
			}
			//Info
			else if(evt.getActionCommand().equals("0018"))
			{
				doShowInfo();
			}
			//Rueckgaengig
			else if(evt.getActionCommand().equals("0019"))
			{
				doUndo();
			}
			//Anwendung sperren
			else if(evt.getActionCommand().equals("0023"))
			{
				doLock();
			}
			//Anwendung entsperren
			else if(evt.getActionCommand().equals("0024"))
			{
				doShowUnlockDialog();
			}
			//Kalender anzeigen
			else if(evt.getActionCommand().equals("0026"))
			{
				doShowCalendar();
			}
			//Liste anzeigen
			else if(evt.getActionCommand().equals("0027"))
			{
				doShowTree();
			}
			//Drucken
			else if(evt.getActionCommand().equals("0030"))
			{
				doPrintCurrentEntry();
			}
			//Alle Eintraege zu einem Tag loeschen
			else if(evt.getActionCommand().equals("0031"))
			{
				doDeleteAllEntrysOnSelectedDay();
			}
			//Stichwoerter zum Eintrag
			else if(evt.getActionCommand().equals("0032"))
			{
				doShowKeywordsDialog();
			}
			//Bild hinzufuegen
			else if(evt.getActionCommand().equals("0033"))
			{
				doAddPicture();
			}
			//Bild loeschen
			else if(evt.getActionCommand().equals("0034"))
			{
				doDeletePicture(view.getSelectedImage()-1, false);
			}
			//Bildbeschreibung
			else if(evt.getActionCommand().equals("0035"))
			{
				doShowImageDescriptionDialog();
			}
			//SelectChange in ComboBox Bilder
			else if(evt.getActionCommand().equals("0036"))
			{
				doHandlePicturesComboBoxChange(evt);
			}
			//Backup erstellen
			else if(evt.getActionCommand().equals("0039"))
			{
				doCreateBackup();
			}
			//Backup wiederherstellen
			else if(evt.getActionCommand().equals("0040"))
			{
				doRecreateBackup();
			}
			//suchen
			else if(evt.getActionCommand().equals("0046"))
			{
				doShowSearchDialog();
			}
		}
		catch(Exception ex)
		{
			DiaryLogger log = new DiaryLogger(ex);
			log.start();
			
			ex.printStackTrace();
		}
		
	}

	/**
	 * Gibt ein DiaryEvent zusammensetzend aus dem DataSet des Views zurueck.
	 * Im Fehlerfall -> return null
	 * 
	 * @return newEvt/null
	 */
	private void updateViewValuesToModel()
	{
		try 
		{
			DataSet dataSet = view.getDataSet();
			
			//View DataSet validieren, wenn ungueltig wird hier Exception geworfen
			validateViewDataSet(dataSet);
			
			SimpleDateFormat sdf = new SimpleDateFormat(IRessources.DATE_FORMAT);
			sdf.setLenient(false);

			model.setDateOfEvent(sdf.parse((String) dataSet.getRowValueByName("date")));
			model.setPlace((String) dataSet.getRowValueByName("place"));
			model.setTimeStart((String) dataSet.getRowValueByName("timeStart"));
			model.setTimeEnd((String) dataSet.getRowValueByName("timeEnd"));
			model.setDescription((String) dataSet.getRowValueByName("text"));
			
			//Return, da sonst das Model auf null gesetzt wird.
			return;
		} 
		catch (ParseException ex) 
		{
			handleException(IRessources.res.getString("0070"), getClass().getName(), 
					"updateViewValuesToModel", ex);
		}
		catch (Exception ex) 
		{
			handleException(ex.getMessage(), getClass().getName(), "updateViewValuesToModel", ex);
		}
		model = null;
	}
	
	private void updateViewValuesToModel(int subEvent)
	{
		try 
		{
			DataSet dataSet = view.getDataSet();
			
			//View DataSet validieren, wenn ungueltig wird hier Exception geworfen
			validateViewDataSet(dataSet);
			
			SimpleDateFormat sdf = new SimpleDateFormat(IRessources.DATE_FORMAT);
			sdf.setLenient(false);

			model.getSubEvent(subEvent).setDateOfEvent(sdf.parse(dataSet.getString("date")));
			model.getSubEvent(subEvent).setPlace(dataSet.getString("place"));
			model.getSubEvent(subEvent).setTimeStart(dataSet.getString("timeStart"));
			model.getSubEvent(subEvent).setTimeEnd(dataSet.getString("timeEnd"));
			model.getSubEvent(subEvent).setDescription(dataSet.getString("text"));
			
			//Return, da sonst das Model auf null gesetzt wird.
			return;
		} 
		catch (ParseException ex) 
		{
			handleException(IRessources.res.getString("0070"), getClass().getName(), 
					"updateViewValuesToModel", ex);
		}
		catch (Exception ex) 
		{
			handleException(ex.getMessage(), getClass().getName(), "updateViewValuesToModel", ex);
		}
		model = null;
	}
	
	/**
	 * Methode zur Validierung des DataSets des Views
	 * 
	 * @param dataSet
	 * @throws Exception 
	 */
	private void validateViewDataSet(DataSet dataSet) throws Exception
	{
		if(((String) dataSet.getRowValueByName("date")).isEmpty())
		{
			DiaryLogger errLog = new DiaryLogger(IRessources.res.getString("0071")+
					"\nAT: "+getClass().getName()+" <validateViewDataSet");
			errLog.start();

			throw new Exception(IRessources.res.getString("0071"));
		}
		if(((String) dataSet.getRowValueByName("date")).length()!=10)
		{
			DiaryLogger errLog = new DiaryLogger(IRessources.res.getString("0072")+
					"\nAT: "+getClass().getName()+" <validateViewDataSet");
			errLog.start();

			throw new Exception(IRessources.res.getString("0072"));
		}
	}
	
	/**
	 * Entsperrt die Anwendung.
	 */
	public void doUnlock()
	{
		try 
		{
			getView().setApplicationLocked(false);
			viewLocked = false;
		} 
		catch (DataSetRowAlreadyExistsException ex) 
		{
			handleException(ex.getMessage(), getClass().getName(), "doUnlock", ex);
		}
		
		//Wenn ein Optionen Dialog geoeffnet ist, wird er wieder hergestellt.
		if(optionsCtrl != null)
		{
			optionsCtrl.getView().setVisible(true);
		}
		
		if(isCalendarSelectionDialogOpenend)
		{
			calSelectSubEntryCtrl.getView().setVisible(true);
		}
	}
	
	/**
	 * Wird aufgerufen, wenn ein Optionen Dialog geschlossen wird. Dieser wird im MainController auf
	 * null gesetzt, da er sonst bestehen bleibt und immer ein Optionen Dialog angezeigt wird nach
	 * der Entsperrung der Anwendung, unabhaengig davon, ob davor ein solcher geoeffnet wurde oder nicht.
	 */
	public void doOptionsClose()
	{
		optionsCtrl = null;
	}
	
	/**
	 * Oeffnet einen einzelnen Eintrag.
	 * 
	 * @param date
	 * @throws Exception
	 */
	private void doOpenSingleEntry(String date) throws Exception
	{
		DiaryEventFileManager dEvtFileMgr = new DiaryEventFileManager();
		
		date = user.getUserID() + "_" + date.replace('.', '_') + IRessources.FILE_EXT;
		
		//Setzen des models
		model = dEvtFileMgr.loadDiaryEventFromFile(savePath, date, user);
		
		selectedEvent = new SimpleDateFormat(IRessources.DATE_FORMAT).format(model.getDateOfEvent());
		
		//Tritt auf wenn ein ParentFile irrtuemlicher Weise als einzelnes Event angezeigt
		//wird. Kann zB passieren, wenn File aus anderem Speicherpfad kopiert wird, da dann
		//kein Eintrag in der ParentFiles Datei vorhanden ist.
		if(model.getSubEvents()!=null&&model.getSubEvents().size()!=0)
		{
			//Hinzufgen der Datei zum ParentFile und aktualisieren des Views
			dEvtFileMgr.markFileAsParent(savePath, new File(date));
			
			updateView();
			
			//Leeres DataSet setzen plus Controls disablen
			doResetViewDataSet();
			view.setControlsAtApplicationStart();
			
			//Entfernt alle TextAenderungs Aktion des UndoManagers, da diese nur bei Anlage eines neuen
			//Eintrags oder einer Aenderung eines bestehenden Eintrags rueckgaengig gemacht werden
			//koennen.
			SDUndoManager.getInstance().removeUndoActionsForEntryChanges();
			
			//Exit aus Methode um DataSet des Views nicht zu setzen, da es sonst zu Folgefehlern kommen kann
			return;
		}
		
		DataSet dataSet = new DataSet();
		
		dataSet.addRow("date", new SimpleDateFormat(IRessources.DATE_FORMAT).format(model.getDateOfEvent()));
		dataSet.addRow("place", model.getPlace());
		dataSet.addRow("timeStart", model.getTimeStart());
		dataSet.addRow("timeEnd", model.getTimeEnd());
		dataSet.addRow("text", model.getDescription());
		if(model.getPictures()!=null&&model.getPictures().size()!=0)
		{
			dataSet.addRow("imageCount", model.getPictures().size());
		}
		
		view.setDataSet(dataSet,false);
		
		view.setControlsAtLoadingEvent();
		
		view.setSelectedImage(0);
		
		//Entfernt alle TextAenderungs Aktion des UndoManagers, da diese nur bei Anlage eines neuen
		//Eintrags oder einer Aenderung eines bestehenden Eintrags rueckgaengig gemacht werden
		//koennen.
		SDUndoManager.getInstance().removeUndoActionsForEntryChanges();
		
		//Statusleiste Text anzeigen
		changeStatusLine(IRessources.res.getString("0073"));
	}
	
	/**
	 * Oeffnet ein SubEvent, das im Kalendar ausgewaehlt wurde.
	 * 
	 * @param pos
	 */
	public void doOpenSubEventFromCalendar(int pos, String date)
	{
		try
		{
			pos-=1;
			DataSet dataSet = new DataSet();
			SimpleDateFormat sdf = new SimpleDateFormat(IRessources.DATE_FORMAT);
			
			//Wenn Model falsch gesetzt ist -> neu laden
			if(model == null || sdf.format(model.getDateOfEvent()).equals(date) == false)
			{
				model = new DiaryEventFileManager().loadDiaryEventFromFile(savePath,
						SDSharedServices.getFileName(user.getUserID(), date),user);
			}
			
			selectedEvent = sdf.format(model.getDateOfEvent())+" - "+(pos+2);

			//pos = -1 -> ParentEvent
			if(pos == -1)
			{
				dataSet.addRow("date", sdf.format(model.getDateOfEvent()));
				dataSet.addRow("place", model.getPlace());
				dataSet.addRow("timeStart", model.getTimeStart());
				dataSet.addRow("timeEnd", model.getTimeEnd());
				dataSet.addRow("text", model.getDescription());
				
				if(model.getPictures()!=null&&model.getPictures().size()!=0)
				{
					dataSet.addRow("imageCount", model.getPictures().size());
				}
			}
			//sonst -> SubEvent
			else
			{
				dataSet.addRow("date", sdf.format(model.getSubEvent(pos).getDateOfEvent()));
				dataSet.addRow("place", model.getSubEvent(pos).getPlace());
				dataSet.addRow("timeStart", model.getSubEvent(pos).getTimeStart());
				dataSet.addRow("timeEnd", model.getSubEvent(pos).getTimeEnd());
				dataSet.addRow("text", model.getSubEvent(pos).getDescription());
				
				if(model.getSubEvent(pos).getPictures()!=null&&model.getSubEvent(pos).getPictures().size()!=0)
				{
					dataSet.addRow("imageCount", model.getSubEvent(pos).getPictures().size());
				}
			}
			
			view.setDataSet(dataSet, false);
			view.setControlsAtLoadingEvent();
			
			view.setSelectedImage(0);
			
			ArrayList<String> tempList = new ArrayList<String>();
			tempList.add(sdf.format(model.getDateOfEvent()));
			view.doExpandPaths(tempList);
			
			//Entfernt alle TextAenderungs Aktion des UndoManagers, da diese nur bei Anlage eines neuen
			//Eintrags oder einer Aenderung eines bestehenden Eintrags rueckgaengig gemacht werden
			//koennen.
			SDUndoManager.getInstance().removeUndoActionsForEntryChanges();
			
			//Statusleiste Text anzeigen
			changeStatusLine(IRessources.res.getString("0073"));
		}
		catch(Exception ex)
		{
			handleException(ex.getMessage(), getClass().getName(), "doOpenSubEventFromCalendar", ex);
		}
	}
	
	/**
	 * Setzt den Zustand der Anwendung, wenn ein neuer Eintrag erstellt werden soll
	 */
	private void doSetApplicationForNewEntry()
	{
		try 
		{
			//Es sind ungespeicherte Aenderungen vorhanden
			if(SDUndoManager.getInstance().hasChangesOnEntry())
			{
				//Wollen Sie wirklich...
				if(JOptionPane.showConfirmDialog(null, IRessources.res.getString("0173"),
						IRessources.res.getString("0017"),0)!=JOptionPane.YES_OPTION)
				{
					//Wenn User nicht auf Ja klickt wird der Vorgang beendet.
					return;
				}
			}
			
			model = new DiaryEvent();
			copyModel = null;
			selectedEvent = null;

			doResetViewDataSet();
			
			view.setControlsAtNewEvent();
			setInChange(false);
			view.setPictureToShow(null);
			
			//Entfernt alle TextAenderungen Aktion des UndoManagers, da ein neuer Eintrag angelegt
			//werden soll und rein logisch hierbei nichts rueckgaengig gemacht werden kann.
			SDUndoManager.getInstance().removeUndoActionsForEntryChanges();
			
			//Statusleiste Inhalt setzen
			changeStatusLine(IRessources.res.getString("0074"));
		} 
		catch (Exception ex) 
		{
			handleException(ex.getMessage(), getClass().getName(), "doSetApplicationForNewEntry", ex);
		}
	}
	
	/**
	 * Speichert den Eintrag aus dem View
	 */
	private void doSave()
	{
		DiaryEventFileManager diaryEvtFileMgr = new DiaryEventFileManager();
		
		//Temp Speichern Eintrag vor delete um es dem UndoManager hinzufuegen zu koennen und um den Eintrag
		//aendern zu koennen da die neuen Daten vom View beim Aendern direkt ins Model geschrieben werden.
		//Zustaetzlich muss geprueft werden ob Bilder hinzugefuegt wurden beim Aendern. Wenn ja muss das
		//copyModel als tempEvt gesetzt werden, da hier die durch die Aenderung hinzugefuegten Bilder noch
		//nicht enthalten sind.
		DiaryEvent tempEvt = null;
		if(copyModel!=null)
		{
			tempEvt = copyModel.getCopy();
		}
		else
		{
			//in manchenen Fehlerfaellen wird das Model auf null gesetzt. Beim speichern muss immer eins da sein
			//sonst kommt es u.a. hier zu Exceptions.
			if(model==null)
			{
				model = new DiaryEvent();
			}
			
			tempEvt = model.getCopy();
		}
		
		
		//Wenn das Event geaendert werden soll
		if(isInChange)
		{
			Date oldDate = model.getDateOfEvent();

			//einzelnes DiaryEvent oder ParentEvent
			if(model.getSubEvents()==null||model.getSubEvents().size()==0||
					selectedEvent.contains(" - 1"))
			{
				updateViewValuesToModel();
				
				//model ist nur im Falle einer Exception in der Methode
				//updateViewValuesToModel() null
				if(model!=null)
				{
					DiaryEvent setBackModel = null;
					
					try 
					{
						SimpleDateFormat sdf = new SimpleDateFormat(IRessources.DATE_FORMAT2);
						
						//Kopie des tempEvts erstellen, da das Model im Fehlerfall wieder zurueckgesetzt
						//werden muss. Im naechsten Schritt werden alle Subevents aus dem tempEvt entfernt,
						//daher ist es nicht mehr brauchbar zum zuruecksetzen (s.a. Mantis 117 Notiz 150)
						setBackModel = tempEvt.getCopy();
						
						//Entfernen aller Untereintraege da nur der geanderte Eintrag an sich wichtig ist
						//fuer den UndoManager. Bleiben die Untereintraege kommt es zu Fehlern.
						tempEvt.doRemoveAllSubEvents();
						
						diaryEvtFileMgr.changeDiaryEvent(tempEvt, model, user, savePath);
						
						//Ausgewaehlten Eintrag neu setzen, falls Datum des Eintrags geaendert wurde
						//(s. MANTIS 91)
						if(!oldDate.equals( model.getDateOfEvent() ) )
						{
							sdf = new SimpleDateFormat(IRessources.DATE_FORMAT);
							selectedEvent = sdf.format(model.getDateOfEvent());
							
							//Model neu laden
							model = diaryEvtFileMgr.loadDiaryEventFromFile(savePath, 
									SDSharedServices.getFileName(user.getUserID(), model.getDateOfEvent()),
									user);
							
							//Wenn Model Untereintraege hat muss das letzte als gewaehlter Eintrag
							//gesetzt werden.
							if(!(model.getSubEvents()==null||model.getSubEvents().size()==0))
							{
								selectedEvent += " - "+(model.getSubEvents().size()+1);
							}
						}
						
						updateView();
						
						//Fuegt die Speicherung dem UndoManager als Aktion hinzu.
						SDUndoManager.getInstance().addAction(new UndoActionForChange(tempEvt, model));
						
						//Statusleiste Text anzeigen
						changeStatusLine(IRessources.res.getString("0075"));
					} 
					catch (Exception ex) 
					{
						//Zuruecksetzen Model
						model = setBackModel;
						
						handleException(ex.getMessage(), getClass().getName(), "doSave", ex);
						return;
					}
				}
				else
				{
					//Zuruecksetzen Model
					model = tempEvt;
					
					return;
				}	
			}
			//ChildEvent
			else
			{
				int eventNumber = Integer.valueOf(
						selectedEvent.substring(selectedEvent.indexOf(" - ")+3))-2;
				
				 updateViewValuesToModel(eventNumber);
				
				//model ist nur im Falle einer Exception in der Methode
				//updateViewValuesToModel null
				if(model!=null)
				{
					try 
					{
						SimpleDateFormat sdf = new SimpleDateFormat(IRessources.DATE_FORMAT2);
						
						//Zwischenspeichern Eintrag, wird benoetigt um es dem UndoManager
						//hinzuzufuegen, da es sonst zu Exceptions kommen kann wenn das Datum
						//des Eintrags auf ein andres Datum geaendert wird.
						DiaryEvent evtToChange = tempEvt.getSubEvent(eventNumber);
						
						//Ueberpruefung ob Kalendar Auswahl Fenster zu diesem Eintrag geoeffnet
						//ist
						//-> Wenn ja beenden, da es sonst zu Exceptions fuehren kann.
						SimpleDateFormat sdf2 = new SimpleDateFormat(IRessources.DATE_FORMAT);
						if(isCalendarSelectionDialogOpenend&&
								sdf2.format(evtToChange.getDateOfEvent()).equals(
										calSelectSubEntryCtrl.getDate() ) )
						{
							doCloseCalenderSelectSubEntryDialog();
						}
						
						diaryEvtFileMgr.changeDiaryEvent(evtToChange, model.getSubEvent(eventNumber),
								user, savePath);
						
						//Kopie des neuen geanderten Eintrags erstellen fuer UndoMgr (MANTIS 69 Notiz 182)
						DiaryEvent newEvt = model.getSubEvent(eventNumber).getCopy();
						
						//Model neu setzen
						model = diaryEvtFileMgr.loadDiaryEventFromFile(savePath, user.getUserID()+"_"+
								sdf.format(model.getSubEvent(eventNumber).getDateOfEvent())+
								IRessources.FILE_EXT, user);
						
						//Ausgewaehlten Eintrag neu setzen, falls Datum des Eintrags geaendert wurde
						//(s. MANTIS 91)
						if(!oldDate.equals( model.getDateOfEvent() ) )
						{
							sdf = new SimpleDateFormat(IRessources.DATE_FORMAT);
							selectedEvent = sdf.format(model.getDateOfEvent());
							
							//Wenn Model Untereintraege hat muss das letzte als gewaehlter Eintrag
							//gesetzt werden.
							if(!(model.getSubEvents()==null||model.getSubEvents().size()==0))
							{
								selectedEvent += " - "+(model.getSubEvents().size()+1);
							}
						}
						
						updateView();
						
						//Fuegt dem UndoManager die Aktion hinzu
						SDUndoManager.getInstance().addAction( new UndoActionForChange(
								evtToChange, newEvt));
						
						//Statusleiste Text anzeigen
						changeStatusLine(IRessources.res.getString("0075"));
					} 
					catch(FileForThisUserInvalidException ex)
					{
						//Zuruecksetzen Model
						model = tempEvt;
						
						diaryEvtFileMgr.markFileAsInvalid(savePath, 
								SDSharedServices.getFileName(user.getUserID(), selectedEvent.substring(0,9)));
						
						updateView();
						
						handleException(ex.getMessage(), getClass().getName(), "doSave", ex);
						return;
					}
					catch(FileInvalidException ex)
					{
						//Zuruecksetzen Model
						model = tempEvt;
						
						diaryEvtFileMgr.markFileAsInvalid(savePath, 
								SDSharedServices.getFileName(user.getUserID(), selectedEvent.substring(0,9)));
						
						updateView();
						
						handleException(ex.getMessage(), getClass().getName(), "doSave", ex);
						return;
					}
					catch (Exception ex) 
					{
						//Zuruecksetzen Model
						model = tempEvt;
						
						handleException(ex.getMessage(), getClass().getName(), "doSave", ex);
						return;
					}
				}
				else
				{
					//Zuruecksetzen Model
					model = tempEvt;
					
					return;
				}
			}
		}
		//Neu Speicherung
		else
		{
			try 
			{
				//Speichern des DiaryEvents aus dem View
				updateViewValuesToModel();
				
				//Model ist nur im Fehlerfall in updateViewValuesToModel null
				if(model!=null)
				{
					model.setUser(user);
					
					File file = new File(SDSharedServices.getFileName(user.getUserID(), 
							model.getDateOfEvent()));
					
					diaryEvtFileMgr.saveDiaryEvent(model, user, savePath);
					setInChange(true);
					updateView();
					
					//Temp Speichern model um es den UndoManager zu adden falls es als Untereintrag gespeichert
					//wird. Muss zwischengespeichert werden, da model im naechsten Schritt neu geladen wird.
					DiaryEvent tempEvent = model.getCopy();
					
					//Eintrag MUSS neu geladen werden, falls es zu dem Tag mehrere Eintraege
					//gibt. Sonst -> Exception (s. Mantis 86)
					setModel(diaryEvtFileMgr.loadDiaryEventFromFile(savePath, file.getName(), user));
					
					selectedEvent = new SimpleDateFormat(IRessources.DATE_FORMAT).
						format(model.getDateOfEvent());
					
					//Wenn es zu dem Tag bereits ein oder mehrere Eintraege gab
					if(model.getSubEvents()!=null&&model.getSubEvents().size()!=0)
					{
						//Als Selected den letzen Untereintrag im Model setzen
						selectedEvent += " - "+(model.getSubEvents().size()+1);
						
						//Fuegt dem UndoManager die Aktion hinzu
						SDUndoManager.getInstance().addAction(new UndoActionForMultiEntrySave(tempEvent));
					}
					//Erster Eintrag an Tag
					else
					{
						//kompletten Pfad zum File setzen
						file = new File(savePath+"\\"+file.toString());
						
						//Fuegt dem UndoManager die Aktion hinzu
						SDUndoManager.getInstance().addAction(new UndoActionForSingleEntrySave(file));
					}
					
					//Statusleiste Text anzeigen
					changeStatusLine(IRessources.res.getString("0076"));
				}
				else
				{
					//Zuruecksetzen Model
					model = tempEvt;
					
					return;
				}
			} 
			catch(FileForThisUserInvalidException ex)
			{				
				//Zuruecksetzen Model
				model = tempEvt;
				
				diaryEvtFileMgr.markFileAsInvalid(savePath, 
						SDSharedServices.getFileName(user.getUserID(), model.getDateOfEvent()));
				
				view.setControlsAtNewEvent();
				updateView();
				
				handleException(ex.getMessage(), getClass().toString(), "doSave", ex);
				return;
			}
			catch(FileInvalidException ex)
			{
				//Zuruecksetzen Model
				model = tempEvt;
				
				diaryEvtFileMgr.markFileAsInvalid(savePath, 
						SDSharedServices.getFileName(user.getUserID(), model.getDateOfEvent()));
				
				view.setControlsAtNewEvent();
				updateView();
				
				handleException(ex.getMessage(), getClass().getName(), "doSave", ex);
				return;
			}
			catch (Exception ex) 
			{
				view.setControlsAtNewEvent();
				
				handleException(ex.getMessage(), getClass().getName(), "doSave", ex);
				return;
			}
		}
		
		//Eintrag mit Untereintraegen -> Eintrag wird ausgeklappt (MANTIS 90)
		if(model.getSubEvents()!=null&&model.getSubEvents().size()!=0)
		{
			try 
			{
				ArrayList<String> eventToExpand = new ArrayList<String>();
				eventToExpand.add(new SimpleDateFormat(IRessources.DATE_FORMAT).format(model.getDateOfEvent()));
				
				view.doExpandPaths(eventToExpand);
			}
			catch (Exception ex)
			{
				handleException(ex.getMessage(), getClass().getName(), "doSave", ex);
			}
		}
		
		copyModel = null;
		
		JOptionPane.showMessageDialog(getView(), IRessources.res.getString("0076"), 
				IRessources.res.getString("0003"), 1);
		
		//Entfernt alle TextAenderungs Aktionen des UndoManagers, da der Eintrag bereits so
		//gespeichert ist und eine Rueckgaengige Aktion dadurch sinnlos ist.
		SDUndoManager.getInstance().removeUndoActionsForEntryChanges();
	}
	
	/**
	 * Beendet die Anwendung
	 */
	private void doExit()
	{
		if(JOptionPane.showConfirmDialog(null, IRessources.res.getString("0077"),
				IRessources.res.getString("0078"),0)==JOptionPane.YES_OPTION)
		{
			//Es sind ungespeicherte Aenderungen vorhanden
			if(SDUndoManager.getInstance().hasChangesOnEntry())
			{
				//Wollen Sie wirklich...
				if(JOptionPane.showConfirmDialog(null, IRessources.res.getString("0173"),
						IRessources.res.getString("0078"),0)!=JOptionPane.YES_OPTION)
				{
					//Wenn User nicht auf Ja klickt wird der Vorgang beendet.
					return;
				}
			}
			
			System.exit(0);
		}
	}
	
	/**
	 * Macht die letzte Aktion rueckgaengig
	 */
	private void doUndo()
	{
		try 
		{
			//Macht die letzte dem UndoManager hinzugefuegte Aktion rueckgaengig
			SDUndoManager.getInstance().makeLastActionUndo();
			
			//Statusleiste Text setzen
			changeStatusLine(IRessources.res.getString("0079"));
		} 
		catch (Exception ex) 
		{
			handleException(IRessources.res.getString("0080"), getClass().getName(), "doUndo", ex);
		}
	}
	
	/**
	 * Setzt den Zustand der Anwendung, dass der ausgewaehlte Eintrag geandert werden kann
	 */
	private void doChange()
	{
		setInChange(true);
		
		//Es wurde kein Bild ausgewaehlt
		if(view.getSelectedImage()==0)
		{
			view.setControlsAtEditEvent(false);
		}
		else
		{
			view.setControlsAtEditEvent(true);
		}
		
		//Dialog "Bildbeschreibung" ist geoeffnet -> Button "Bearbeiten" muss aktiviert werden
		if(imgDescVC!=null)
		{
			imgDescVC.setChangeAble();
		}
		
		//Dialog "Stichwoerter zum Eintrag" ist geoeffnet -> View muss auf Aenderungszustand gesetzt werden
		if(entryKeywordsVC!=null)
		{
			entryKeywordsVC.setChangeAble();
		}
		
		//Statusleiste Text anzeigen
		changeStatusLine(IRessources.res.getString("0081"));
	}
	
	/**
	 * Loescht den ausgewaehlten Eintrag
	 */
	private void doDelete()
	{
		if(JOptionPane.showConfirmDialog(null, 
				IRessources.res.getString("0082"),IRessources.res.getString("0083"),
				0)==JOptionPane.YES_OPTION)
		{
			DiaryEventFileManager diaryEvtFileMgr = new DiaryEventFileManager();

			String fileName = SDSharedServices.getFileName(user.getUserID(), model.getDateOfEvent());
			
			try 
			{
				//Ueberpruefung ob Kalendar Auswahl Fenster zu diesem Eintrag geoeffnet ist
				//-> Wenn ja beenden, da es sonst zu Exceptions fuehren kann.
				SimpleDateFormat sdf2 = new SimpleDateFormat(IRessources.DATE_FORMAT);
				if(isCalendarSelectionDialogOpenend&&
						sdf2.format(model.getDateOfEvent()).equals(
								calSelectSubEntryCtrl.getDate() ) )
				{
					doCloseCalenderSelectSubEntryDialog();
				}
				
				//Einzelnes DiaryEvent
				if(model.getSubEvents() == null||model.getSubEvents().size()==0)
				{	
					diaryEvtFileMgr.deleteDiaryEvent(fileName, model, user, savePath);
					//Fuegt dem UndoManager die Aktion hinzu
					SDUndoManager.getInstance().addAction(new UndoActionForDelete(model));
				}
				//Event mit SubEvents
				else
				{
					int subEventNr = Integer.valueOf(selectedEvent.substring(
							selectedEvent.lastIndexOf(" ")+1))-2;

					//subEventNr = -1 wenn es das HauptEvent ist da 1-2 = -1
					if(subEventNr!=-1)
					{
						diaryEvtFileMgr.deleteDiaryEvent(fileName, model.getSubEvent(subEventNr), 
								user, savePath);
						//Fuegt dem UndoManager die Aktion hinzu
						SDUndoManager.getInstance().addAction(new UndoActionForDelete(
								model.getSubEvent(subEventNr)));
					}
					else
					{
						diaryEvtFileMgr.deleteDiaryEvent(fileName, model, user, savePath);
						//Fuegt dem UndoManager die Aktion hinzu
						SDUndoManager.getInstance().addAction(new UndoActionForDelete(model));
					}
				}
				
				//Setzen des DataSets des Views
				doResetViewDataSet();
				
				view.setControlsAtApplicationStart();
				setInChange(false);
				model = null;
				selectedEvent = null;
				copyModel = null;
				view.setPictureToShow(null);
				
				updateView();
				
				JOptionPane.showMessageDialog(null, IRessources.res.getString("0084"),
						IRessources.res.getString("0003"), 1);
				
				//Entfernt alle rueckgaengig machbaren Aktionen im UndoManager, da diese nach einem
				//Loeschvorgang ueberfluessig sind.
				SDUndoManager.getInstance().removeUndoActionsForEntryChanges();
				
				//Statusleiste Text anzeigen
				changeStatusLine(IRessources.res.getString("0084"));
			} 
			catch(FileForThisUserInvalidException ex)
			{
				diaryEvtFileMgr.markFileAsInvalid(savePath, fileName);
				updateView();
				handleException(ex.getMessage(), getClass().getName(), "doDelete", ex);
				return;
			}
			catch(FileInvalidException ex)
			{
				diaryEvtFileMgr.markFileAsInvalid(savePath, fileName);
				updateView();
				handleException(ex.getMessage(), getClass().getName(), "doDelete", ex);
				return;
			}
			catch (Exception ex) 
			{
				handleException(ex.getMessage(), getClass().getName(), "doDelete", ex);
				return;
			}		
		}
	}
	
	/**
	 * Fuegt dem Eintrag ein Bild hinzu, dass der User zuvor auswaehlen muss.
	 */
	private void doAddPicture()
	{
		//neuer JFileChooser Dialog
		JFileChooser chooseImg = new JFileChooser(); 
		chooseImg.setDialogType(JFileChooser.OPEN_DIALOG); 
		
		//Verzeichnisse und Dateien koennen angezeigt werden
		chooseImg.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES); 
        
		//Bitte Bild waehlen
        chooseImg.setDialogTitle(IRessources.res.getString("0181"));
        
        //Standard Filter "Alle Dateien" entfernen
        chooseImg.removeChoosableFileFilter(chooseImg.getChoosableFileFilters()[0]);
        
        //Filter setzen, dass nur Bilddateien angezeigt werden.
        chooseImg.setFileFilter(new FileFilter()
        {
			@Override
			public String getDescription() 
			{
				//Bilddateien
				return IRessources.res.getString("0182");
			}
			
			@Override
			public boolean accept(File f) 
			{
				String name = f.getName();
				
				if(f.isDirectory()||
						name.endsWith(".jpg")||name.endsWith(".JPG")||
						name.endsWith(".gif")||name.endsWith(".GIF")||
						name.endsWith(".png")||name.endsWith(".PNG")||
						name.endsWith(".jpeg")||name.endsWith(".JPEG"))
				{
					return true;
				}
				return false;
			}
		});
        
        chooseImg.setVisible(true);
        
        if(chooseImg.showOpenDialog(getView())==JFileChooser.APPROVE_OPTION)
        {
        	File imgFile = new File(chooseImg.getSelectedFile().getPath());
        	
        	//Validierung ob Pfad vorhanden (Evtl. Pfad in Zwischenzeit geloescht oder User hat
        	//zum eingentlichen Pfad (aus Versehen!?) noch etwas dazu getippt).
        	if(!imgFile.exists())
        	{
        		//Datei existiert nicht
        		handleException(IRessources.res.getString("0183"), getClass().getName(), 
        				"doAddPicture");
				return;
        	}
        	
        	try 
        	{	
        		//Einlesen Bilddatei in byteArray
				byte[] imagePuffer = new byte[(int) imgFile.length()];
				DataInputStream in = new DataInputStream(new FileInputStream(imgFile));
				in.readFully(imagePuffer);
				in.close();
				
				//Speichern des Inhalts der Datei in String
				String imageData = new BASE64Encoder().encode(imagePuffer);
				
				//Model muss zwischen gespeichert werden, in dem Zustand bevor ein Bild hinzugefuegt wurde,
				//da es sonst zu Fehlern kommt beim Aendern wenn das Bild bereits im Model gesetzt ist.
				if(copyModel==null)
				{
					copyModel = model.getCopy();
				}
				
				//Speichern des Bilds in Model und setzen der Combobox im View.
				//Returnvalue = Anzahl der Bilder im Model nach Speicherung
				int picturesSize = addPictureToModelAndView(new SDPicture(imageData));
				
				//Gespeichertes Bild setzen
				view.setSelectedImage(picturesSize);
				
				//Dem UndoManager die Aktion hinzufuegen
				SDUndoManager.getInstance().addAction(new UndoActionForAddPicture(picturesSize-1));
				
				//Bild added
				changeStatusLine(IRessources.res.getString("0184"));
        	}
        	catch (IOException ex) 
        	{
        		//Fehler bei IO
				handleException(IRessources.res.getString("0185"), getClass().getName(), "doAddPicture", ex);
			}
        	catch(Exception ex)
        	{
        		handleException(ex.getMessage(), getClass().getName(), "doAddPicture", ex);
        	}
        }
        chooseImg.setVisible(false);
	}
	
	/**
	 * Loescht das uebergene Bild im View aus dem Model.
	 */
	public void doDeletePicture(int selImg, boolean isUndoAction)
	{
		try 
		{
			int imgCount = 0;
			
			//Model muss zwischen gespeichert werden, in dem Zustand bevor ein Bild hinzugefuegt wurde,
			//da es sonst zu Fehlern kommt beim Aendern wenn das Bild bereits im Model gesetzt ist.
			if(copyModel==null)
			{
				copyModel = model.getCopy();
			}
			
			//MultiEintrag
			if(selectedEvent!=null&&selectedEvent.contains(" - "))
			{
				int eventNumber = Integer.valueOf(
						selectedEvent.substring(selectedEvent.indexOf(" - ")+3))-2;
				
				//PARENT
				if(eventNumber == -1)
				{
					//Wenn das Loeschen keine UndoAction ist wird es dem UndoManager als neue Action
					//hinzugefuegt.
					if(isUndoAction == false)
					{
						SDUndoManager.getInstance().addAction(new UndoActionForDeletePicture(
							model.getPictures().get(selImg)));
					}
					
					//Bild aus Model loeschen
					model.getPictures().remove(selImg);
					
					imgCount = model.getPictures().size();
				}
				//CHILD
				else
				{
					//Wenn das Loeschen keine UndoAction ist wird es dem UndoManager als neue Action
					//hinzugefuegt.
					if(isUndoAction == false)
					{
						SDUndoManager.getInstance().addAction(new UndoActionForDeletePicture(
							model.getSubEvent(eventNumber).getPictures().get(selImg)));
					}
					
					//Bild aus Model Untereintrag loeschen
					model.getSubEvent(eventNumber).getPictures().remove(selImg);
					
					imgCount = model.getSubEvent(eventNumber).getPictures().size();
				}
			}
			//SingleEintrag
			else
			{
				//Wenn das Loeschen keine UndoAction ist wird es dem UndoManager als neue Action
				//hinzugefuegt.
				if(isUndoAction == false)
				{
					SDUndoManager.getInstance().addAction(new UndoActionForDeletePicture(
						model.getPictures().get(selImg)));
				}
				
				//Bild aus Model loeschen
				model.getPictures().remove(selImg);
				
				imgCount = model.getPictures().size();
			}
			
			
			//DataSet View neu setzen, um Eintraege in Combobox entsprechend darzustellen.
			DataSet dsView = view.getDataSet();
			dsView.setValueOfRow("imageCount", imgCount);
			view.setDataSet(dsView, false);
			
			//In der ComboBox "keine Auswahl" selecten
			view.setSelectedImage(0);
			
			//Wenn Del von UndoManager kommt uebernimmt dieser die Anzeige im statuslabel
			if(isUndoAction == false)
			{
				//Bild wurde geloescht.
				changeStatusLine(IRessources.res.getString("0180"));
			}
		} 
		catch (Exception ex)
		{
			handleException(ex.getMessage(), getClass().getName(), "doDeletePicture", ex);
		}
	}
	
	/**
	 * Reagiert auf Aenderung in der ComboBox fuer die Bilder des Eintrags.
	 */
	private void doHandlePicturesComboBoxChange(ActionEvent evt)
	{
		try
		{
			JComboBox cb = (JComboBox)evt.getSource();
			String item = (String) cb.getSelectedItem();
			
			//Selection: "keine Auswahl" -> kein Bild anzeigen.
			if(IRessources.NO_IMG.equals(item))
			{
				//boolean spielt keine Rolle da es sowieso nicht ueberprueft wird wenn img null ist aber sicher ist
				//sicher.
				view.setPictureToShow(null);
			}
			else
			{
				Integer selImg = Integer.valueOf(item.substring(item.indexOf(" ")+1))-1;
				String imageData = null;
					
				//MultiEintrag
				if(selectedEvent!=null&&selectedEvent.contains(" - "))
				{
					int eventNumber = Integer.valueOf(
							selectedEvent.substring(selectedEvent.indexOf(" - ")+3))-2;
					
					//Parent
					if(eventNumber==-1)
					{
						imageData = model.getPictures().get(selImg).getPictureData();
					}
					//child
					else
					{
						imageData = model.getSubEvent(eventNumber).getPictures().get(selImg).getPictureData();
					}
				}
				//SingleEintrag
				else
				{
					imageData = model.getPictures().get(selImg).getPictureData();
				}
				
				byte[] imageDataArr = new BASE64Decoder().decodeBuffer(imageData);
					
				//Zeigt das Bild im View an und setzt den Button "Aendern" aktiv je nachdem ob der
				//Eintrag geaendert werden soll oder nicht.
				view.setPictureToShow(ImageIO.read(new ByteArrayInputStream(imageDataArr)));
			}
			
			//Es war ein Dialog "Bildbeschreibung" offen
			if(imgDescVC != null)
			{
				imgDescVC.doCloseView();
				imgDescVC = null;
			}
		}
		catch(Exception ex)
		{
			handleException(ex.getMessage(), getClass().getName(), "doHandlePicturesComboBoxChange", ex);
		}	
	}
	
	/**
	 * Zeigt den Kalendar an
	 */
	private void doShowCalendar()
	{
		view.setCalendarVisible();
	}
	
	/**
	 * Zeigt den Tree an
	 */
	private void doShowTree()
	{
		view.setTreeVisible();
	}
	
	/**
	 * Sperrt die gesamte Anwendung
	 */
	private void doLock()
	{
		try 
		{
			getView().setApplicationLocked(true);
			viewLocked = true;
		} 
		catch (DataSetRowAlreadyExistsException ex)
		{
			handleException(ex.getMessage(), getClass().getName(), "doLock", ex);
		}
		
		//Wenn ein Optionen Dialog offen ist, wird er auf unsichtbar gesetzt.
		if(optionsCtrl != null)
		{
			optionsCtrl.getView().setVisible(false);
		}
		
		//Wenn ein Kalendar Auswahl Dialog offen ist, wird er auf unsichtbar gesetzt
		if(isCalendarSelectionDialogOpenend)
		{
			calSelectSubEntryCtrl.getView().setVisible(false);
		}
	}
	
	/**
	 * Erstellt einen Dialog um die Anwendung zu entsperren
	 */
	private void doShowUnlockDialog()
	{
		new UnlockDialogVC(this);
		view.setUnlockButtonEnabled(false);
	}
	
	/**
	 * Zeit den Optionen Dialog an
	 */
	private void doShowOptions()
	{
		try 
		{
			optionsCtrl = new OptionsVC(this);
		} 
		catch (DataSetRowAlreadyExistsException ex)
		{
			handleException(ex.getMessage(), getClass().getName(), "doShowOptions", ex);
		}
	}
	
	/**
	 * Wird ausgefuehrt bei einem Klick auf den Reload Button in der Toolbar.
	 * Die Eintraege werden aktualisiert.
	 */
	private void doToolBarReload()
	{
		updateView();
		
		changeStatusLine(IRessources.res.getString("0069"));
	}
	
	/**
	 * Oeffnet die Hilfe im Browser
	 */
	private void doShowHelp()
	{
		try 
		{
			URI uri = new URI("help.html");
			Desktop.getDesktop().browse(uri);
		} 
		catch (URISyntaxException ex) 
		{
			handleException(ex.getMessage(), getClass().getName(), "doShowHelp", ex);
		} 
		catch (IOException ex) 
		{
			handleException(IRessources.res.getString("0085"), getClass().getName(), "doShowHelp", ex);
		}
	}
	
	/**
	 * Zeigt den Infodialog an.
	 */
	private void doShowInfo()
	{
		JOptionPane.showMessageDialog(null, IRessources.INFO, IRessources.res.getString("0049"), 1);
	}
	
	/**
	 * Zeigt den Dialog "Bildbeschreibung" an.
	 */
	private void doShowImageDescriptionDialog()
	{
		//Es wurde noch kein Bildbeschreibungs Dialog erstellt.
		if(imgDescVC==null)
		{
			try 
			{
				int eventNumber = -1;
				
				//MultiEintrag
				if(selectedEvent.contains(" - "))
				{
					eventNumber = Integer.valueOf(
							selectedEvent.substring(selectedEvent.indexOf(" - ")+3))-2;
				}

				imgDescVC = new ImageDescriptionVC(this, eventNumber, view.getSelectedImage()-1);
			} 
			catch (Exception ex)
			{
				handleException(ex.getMessage(), getClass().getName(), "doShowImageDescriptionDialog", ex);
			}
		}
	}
	
	/**
	 * Oeffnet den Dialog "Stichwoerter zum Eintrag"
	 */
	private void doShowKeywordsDialog()
	{
		if(entryKeywordsVC==null)
		{
			try 
			{
				int eventNumber = -1;
				
				//MultiEintrag
				if(selectedEvent!=null && selectedEvent.contains(" - "))
				{
					eventNumber = Integer.valueOf(
							selectedEvent.substring(selectedEvent.indexOf(" - ")+3))-2;
				}
				
				entryKeywordsVC = new EntryKeywordsVC(this, eventNumber);
			} 
			catch (Exception ex) 
			{
				handleException(ex.getMessage(), getClass().getName(), "doShowKeywordsDialog", ex);
			}
		}	
	}
	
	/**
	 * Oeffnet den Dialog "Suche"
	 */
	private void doShowSearchDialog()
	{
		new SearchVC(this);
	}
	
	/**
	 * Wenn der Dialog "Bildbeschreibung" geschlossen wird, muss die Referenz auf den ChildCOntroller auf
	 * null gesetzt werden, da sonst kein neuer Dialog geoeffnet werden kann.
	 */
	public void handleImageDescDialogClose() 
	{
		imgDescVC = null;
	}
	
	/**
	 * Wenn der Dialog "Stichwoerter zum Eintrag" geschlossen wird, muss die Referenz auf den ChildCOntroller
	 * auf null gesetzt werden, da sonst kein neuer Dialog geoeffnet werden kann.
	 */
	public void handleKeywordDialogClose()
	{
		entryKeywordsVC = null;
	}
	
	/**
	 * Stoesst den Druck des aktuellen Eintrags im View an.
	 */
	private void doPrintCurrentEntry()
	{
		try 
		{
			//Wenn das DataSet des Views gueltig ist, wird der Druck angestossen
			//Sonst wird Exception abgefangen
			validateViewDataSet(view.getDataSet());

			updateViewValuesToModel();
			
			//model ist null im Fehlerfall in updateViewValuesToModel
			//falls null ist darf nicht weiter gemacht werden.
			if(model==null)
			{
				return;
			}
			
			new SDPrinter(model).doPrint();
		}
		catch (Exception ex)
		{
			handleException(ex.getMessage(), getClass().getName(), "doPrintCurrentEntry", ex);
		} 
	}
	
	/**
	 * Loescht alle Eintraege an einem Tag.
	 */
	private void doDeleteAllEntrysOnSelectedDay()
	{
		//Fehlerbehandlung falls kein Eintrag mit Untereintraegen ausgewaehlt wurde.
		if(selectedMultiEvent==null)
		{
			handleException(IRessources.res.getString("0123"),
					getClass().getName(), "doDeleteAllEntrysOnSelectedDay", new Exception());
			return;
		}
		
		try 
		{
			//"Wollen Sie wirklich...?"
			if(JOptionPane.showConfirmDialog(null, 
					IRessources.res.getString("0124"),IRessources.res.getString("0083"),
					0)==JOptionPane.YES_OPTION)
			{
				//Ueberpruefung ob Kalendar Auswahl Fenster zu diesem Eintrag geoeffnet ist
				//-> Wenn ja beenden, da es sonst zu Exceptions fuehren kann.
				if(isCalendarSelectionDialogOpenend&&
						selectedMultiEvent.equals(calSelectSubEntryCtrl.getDate() ) )
				{
					doCloseCalenderSelectSubEntryDialog();
				}
				
				DiaryEventFileManager evtFileMgr = new DiaryEventFileManager();
				
				//Eintrag muss temporaer geladen werden, bevor er geloescht wird um ihn dem UndoManager nach
				//erfolgreichem Loeschvorgang hinzufuegen zu koennen.
				DiaryEvent tempEvt = evtFileMgr.loadDiaryEventFromFile(savePath, 
						user.getUserID()+"_"+selectedMultiEvent.replace('.', '_')+IRessources.FILE_EXT, user);
				
				//Alle Eintraege loeschen
				evtFileMgr.deleteAllEntrysOnDay(selectedMultiEvent, savePath, user);
				
				//Wenn geloeschter Eintrag noch angezeit wird -> View cleanen (Mantis 54 Notiz 174)
				//auser wenn model null ist (s. MANTIS 54 Notiz 213)
				//Ausserdem undoActions entfernen, falls Eintrag in Aenderung war (Mantis 54 Notiz 246)
				if(model!=null&&tempEvt.getDateOfEvent().equals(model.getDateOfEvent()))
				{
					DataSet dataSet = new DataSet();
					
					dataSet.addRow("date");
					dataSet.addRow("place");
					dataSet.addRow("timeStart");
					dataSet.addRow("timeEnd");
					dataSet.addRow("text");
					
					view.setDataSet(dataSet, false);
					
					SDUndoManager.getInstance().removeUndoActionsForEntryChanges();
				}
				
				//View updaten
				updateView();
				
				//Fuegt den Eintrag zum UndoManager hinzu um ihn ggf. wieder herzustellen.
				SDUndoManager.getInstance().addAction(new UndoActionForDelete(tempEvt));
			}
		} 
		catch (Exception ex) 
		{
			handleException(ex.getMessage(), getClass().getName(), "doDeleteAllEntrysOnSelectedDay", ex);
		}
	}
	
	/**
	 * Fuegt dem Model und dem ViewDataSet das uebergebene Bild hinzu.
	 * 
	 * @param img
	 * @return Position des Bildes in der Bilderliste des ausgewaehlten (Unter)Eintrags.
	 * @throws Exception
	 */
	public int addPictureToModelAndView(SDPicture img) throws Exception
	{
		//ViewDataSet holen um die Eintraege in der Comobox anzuzeigen
		DataSet dsView = view.getDataSet();
		
		//Variable zur Speicherung der Groesse der Bilderliste um Bild anzuzeigen. Wird benoetigt
		//weil man sonst wieder ueberpruefen muesste ob es sich um parent oder subevent handelt um dann
		//wiederum die Groesse zu erfragen.
		int picturesSize = 0;
		
		//Multieintrag ausgewaehlt
		if(selectedEvent!=null&&selectedEvent.contains(" - "))
		{
			int eventNumber = Integer.valueOf(
					selectedEvent.substring(selectedEvent.indexOf(" - ")+3))-2;
			
			//ParentEntry
			if(eventNumber==-1)
			{
				//Bild Model hinzufuegen
				model.addPicture(img);
				
				picturesSize = model.getPictures().size();
				
				dsView.setValueOfRow("imageCount", picturesSize);
			}
			//ChildEntry
			else
			{
				//Bild Untereintrag im Model adden
				model.getSubEvent(eventNumber).addPicture(img);
				
				picturesSize = model.getSubEvent(eventNumber).getPictures().size();
				
				dsView.setValueOfRow("imageCount", picturesSize);
			}
		}
		else
		{
			//Bild Model hinzufuegen
			model.addPicture(img);
			
			picturesSize = model.getPictures().size();
			
			dsView.setValueOfRow("imageCount", picturesSize);
		}
		
		//DataSet setzen zwecks anzahl Bilder in ComboBox Pictures
		view.setDataSet(dsView, false);
		
		return picturesSize;
	}
	
	/**
	 * Erstellt ein BackupFile dessen Pfad vom Benutzer gewaehlt werden muss.
	 * In diesem sind alle Eintraege des Users im aktuellen Verzeichnis enthalten.
	 */
	private void doCreateBackup()
	{
		try
		{
			//neuer JFileChooser Dialog
			JFileChooser chooseBkpFile = new JFileChooser(); 
			chooseBkpFile.setDialogType(JFileChooser.OPEN_DIALOG); 
			
			//Verzeichnisse und Dateien koennen angezeigt werden
			chooseBkpFile.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES); 
	        
			//Bitte Datei waehlen
	        chooseBkpFile.setDialogTitle(IRessources.res.getString("0192"));
	        
	        //Text des Bestaetigen Button setzen auf "Erstellen" (Standard: "Oeffnen") s. Mantis 29 Notiz 224
	        chooseBkpFile.setApproveButtonText(IRessources.res.getString("0229"));
	        
	        //Standard Filter "Alle Dateien" entfernen
	        chooseBkpFile.removeChoosableFileFilter(chooseBkpFile.getChoosableFileFilters()[0]);
	        
	        //Filter setzen, dass nur Backupdaten zulaessig sind.
	        chooseBkpFile.setFileFilter(new FileFilter()
	        {
				@Override
				public String getDescription() 
				{
					//Backup Dateien
					return IRessources.res.getString("0193");
				}
				
				@Override
				public boolean accept(File f) 
				{
					String name = f.getName();
					
					if(f.isDirectory()||name.endsWith(".bkp")||name.endsWith(".BKP"))
					{
						return true;
					}
					return false;
				}
			});
	        
	        chooseBkpFile.setVisible(true);
	        
	        //User hat Datei ausgewaehlt
	        if(chooseBkpFile.showOpenDialog(getView())==JFileChooser.APPROVE_OPTION)
	        {
	        	File file = new File(chooseBkpFile.getSelectedFile().getPath());
	        	
	        	new DiaryEventFileManager().doBackUp(savePath, user, file.toString());
	        	
	        	//BackupDatei wurde erstellt
	        	changeStatusLine(IRessources.res.getString("0194"));
	        }
	        
	        chooseBkpFile.setVisible(false);
		}
		catch(Exception ex)
		{
			handleException(ex.getMessage(), getClass().getName(), "doCreateBackup", ex);
		}
	}
	
	/**
	 * Stellt ein Backup anhand einer Backupdatei wieder her. Diese muss vom User gewaehlt werden.
	 * 
	 * ACHTUNG: Bereits vorhandene Eintraege werden dabei ggf. ueberschrieben!!!
	 */
	private void doRecreateBackup()
	{
		try
		{
			//neuer JFileChooser Dialog
			JFileChooser chooseBkpFile = new JFileChooser(); 
			chooseBkpFile.setDialogType(JFileChooser.OPEN_DIALOG); 
			
			//Verzeichnisse und Dateien koennen angezeigt werden
			chooseBkpFile.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES); 
	        
			//Bitte Datei waehlen
	        chooseBkpFile.setDialogTitle(IRessources.res.getString("0192"));
	        
	        //Standard Filter "Alle Dateien" entfernen
	        chooseBkpFile.removeChoosableFileFilter(chooseBkpFile.getChoosableFileFilters()[0]);
	        
	        //Filter setzen, dass nur Backupdaten zulaessig sind.
	        chooseBkpFile.setFileFilter(new FileFilter()
	        {
				@Override
				public String getDescription() 
				{
					//Backupdateien
					return IRessources.res.getString("0193");
				}
				
				@Override
				public boolean accept(File f) 
				{
					String name = f.getName();
					
					if(f.isDirectory()||name.endsWith(".bkp")||name.endsWith(".BKP"))
					{
						return true;
					}
					return false;
				}
			});
	        
	        chooseBkpFile.setVisible(true);
	        
	        //User hat Datei ausgewaehlt
	        if(chooseBkpFile.showOpenDialog(getView())==JFileChooser.APPROVE_OPTION)
	        {
	        	File file = new File(chooseBkpFile.getSelectedFile().getPath());
	        	
	        	//Validierung ob Pfad vorhanden (Evtl. Pfad in Zwischenzeit geloescht oder User hat
	        	//zum eingentlichen Pfad (aus Versehen!?) noch etwas dazu getippt).
	        	if(file.exists() == false)
	        	{
	        		//Datei existiert nicht
	        		handleException(IRessources.res.getString("0183"), getClass().getName(), 
	        				"doRecreateBackup");
					return;
	        	}
	        	
	        	//Achtung... Eintraege werden ueberschrieben...
	        	if(JOptionPane.showConfirmDialog(view, 
	        			IRessources.res.getString("0195"),
	        			"Backup wiederherstellen", JOptionPane.WARNING_MESSAGE) == JOptionPane.OK_OPTION)
	        	{
	        		new DiaryEventFileManager().doBackUpRedo(savePath, file.toString(), user);
	        		
	        		//View updaten um hergestellte Eintraege anzuzeigen
	        		updateView();
	        		
	        		//Backup wurde wieder hergestellt
	        		changeStatusLine(IRessources.res.getString("0196"));
	        	}
	        }
	        
	        chooseBkpFile.setVisible(false);
		}
		catch(Exception ex)
		{
			handleException(ex.getMessage(), getClass().getName(), "doRecreateBackup", ex);
		}
	}
	
	/**
	 * DataSet des Views wieder auf Ursprungszustand setzen
	 * 
	 * @throws Exception
	 */
	private void doResetViewDataSet() throws Exception
	{
		DataSet dataSet = new DataSet();
		dataSet.addRow("date", IRessources.EMPTY);
		dataSet.addRow("place", IRessources.EMPTY);
		dataSet.addRow("timeStart", IRessources.EMPTY);
		dataSet.addRow("timeEnd", IRessources.EMPTY);
		dataSet.addRow("text", IRessources.EMPTY);
		
		view.setDataSet(dataSet, false);
	}
	
	/**
	 * Aendert den Text der Statusleiste.
	 * Dieser wird 5 Sekunden angezeigt. Danach wird wieder der Standarttext ("SimpleDiary bereit.")
	 * angezeigt.
	 * 
	 * @param text
	 */
	public void changeStatusLine(String text)
	{
		//setStatusBackThread ist nur beim allerersten Aufruf null
		if(setStatusBackThread!=null)
		{
			//Der Thread muss beendet werden, falls er noch laeuft. Sonst wird der Text uU zu
			//schnell zurueck gesetzt.
			setStatusBackThread.interrupt();
		}
		
		//Neuen (setStatusBack)Thread erzeugen
		setStatusBackThread = new Thread()
		{
			@Override
			public void run()
			{
				try 
				{
					//Wartet 5 Sekunden
					Thread.sleep(5000);
				} 
				catch (InterruptedException ex) 
				{
					DiaryLogger msgLog = new DiaryLogger(IRessources.res.getString("0086"));
					msgLog.start();
					//Wird benoetigt um Text nicht vorzeitug zurueckzusetzen
					return;
				}
				//Setzt nach Ablauf der 5 Sekunden Wartezeit den Text der Statuszeile zurueck.
				view.setStatusLineText(IRessources.res.getString("0062"), false);
			}
		};
		
		//Setzt Text fuer die Statusleiste
		view.setStatusLineText(text, true);
		
		//Startet den Thread, der den Text der Statusleiste nach 10 Sek wieder zurueck setzt.
		setStatusBackThread.start();
	}
	
	/**
	 * Schliesst einen Calendar Eintragsauswahl Dialog, falls geoeffnet
	 */
	public void doCloseCalenderSelectSubEntryDialog()
	{
		if(calSelectSubEntryCtrl!=null)
		{
			calSelectSubEntryCtrl.getView().dispose();
			calSelectSubEntryCtrl = null;
		}
		setCalendarSelectionDialogOpenend(false);
	}
	
	/**
	 * Behandelt einen Rechtsklick auf einen Eintrag im Tree.
	 * 
	 * @param entry
	 * @param y 
	 * @param x 
	 */
	private void handleRightClickOnEntry(String entry, int x, int y)
	{
		//Wenn nicht auf Eintraege geklickt wurde
		if(!entry.equals(IRessources.res.getString("0056")))
		{
			//SubEvent
			if(entry.contains(" - "))
			{
				view.setPopupMenuForSingleEntry(x,y);
			}
			//Einzelner Eintrag oder Ueberpunkt im Tree
			else
			{
				//Ueberpunkt im Tree
				if(model==null)
				{
					view.setPopupMenuForMultiEntry(x,y);
				}
				//einzelner Eintrag
				else
				{
					view.setPopupMenuForSingleEntry(x,y);
				}
			}
		}
	}
	
	/**
	 * Gibt true zurueck, wenn gerade ein neuer Eintrag angelegt wird.
	 * 
	 * @return boolean
	 */
	public boolean isNewEntryBeingCreated()
	{
		return !isInChange && getView().canEntryBeSaved();
	}
	
	@Override
	/**
	 * Wird ausgefuehrt, wenn im View ein Eintrag selektiert wird
	 */
	public void valueChanged(TreeSelectionEvent evt)
	{
		//Es sind ungespeicherte Aenderungen vorhanden
		//und Event wurde nicht ausgeloest durch einen Rechtsklick. Muss geprueft werden, da sonst zwei mal
		//die Userabfrage "Wollen Sie wirklich..." kommen wuerde.
		if(!isValueChangeFiredByRightClick&&SDUndoManager.getInstance().hasChangesOnEntry())
		{
			//Wollen Sie wirklich...
			if(JOptionPane.showConfirmDialog(null, IRessources.res.getString("0173"),
					IRessources.res.getString("0174"),0)!=JOptionPane.YES_OPTION)
			{
				//Wenn User nicht auf Ja klickt wird der Vorgang beendet.
				return;
			}
		}
		//Zuruecksetzen auf false egal ob es true oder schon false war.
		isValueChangeFiredByRightClick = false;
		
		String tempSelectedEvent = evt.getPath().getLastPathComponent().toString();
		
		//Wenn der User entweder auf Eintraege geklickt hat oder auf ein ParentEvent
		if(tempSelectedEvent.contains(IRessources.res.getString("0056"))||
				((DefaultMutableTreeNode)evt.getPath().getLastPathComponent()).getChildCount()>=1)
		{
			view.setControlsAtApplicationStart();
			
			try 
			{
				//Setzen des DataSets des Views (leer)
				doResetViewDataSet();
				
				//Entfernt alle TextAenderungs Aktion des UndoManagers, da diese nur bei Anlage eines neuen
				//Eintrags oder einer Aenderung eines bestehenden Eintrags rueckgaengig gemacht werden
				//koennen.
				SDUndoManager.getInstance().removeUndoActionsForEntryChanges();
				model = null;
				copyModel = null;
				selectedEvent = null;
				
				//Setzen Bilder Combobox keine Eintraege + kein Bild anzeigen
				DataSet dsView = view.getDataSet();
				dsView.setValueOfRow("imageCount", 0);
				view.setDataSet(dsView, false);
				view.setSelectedImage(0);
				
				selectedMultiEvent = tempSelectedEvent;
				
				//Muss zurueckgesetzt werden, da nichts (mehr) geaendert wird.
				setInChange(false);
				
				return;
			} 
			catch (Exception ex)
			{
				handleException(ex.getMessage(), getClass().getName(), "valueChanged", ex);
			}
		}

		DiaryEventFileManager dEvtFileMgr = new DiaryEventFileManager();
		try 
		{
			//Wenn es sich um ein SubEvent/ParentEvent mit SubEvents handelt
			if(tempSelectedEvent.contains("-"))
			{
				int pos = Integer.valueOf(tempSelectedEvent.substring(
						tempSelectedEvent.lastIndexOf(" ")+1))-2;
				DataSet dataSet = new DataSet();
				SimpleDateFormat sdf = new SimpleDateFormat(IRessources.DATE_FORMAT);
				
				//Wenn model null ist (Tritt auf wenn vorher auf Eintraege geklick wurde) oder ->
				//wenn im View ein Eintrag ausgewaehlt wurde und dieser nicht das Datum des models 
				//enthaelt
				//oder -> wenn dateOfEvent null ist (neuer Eintrag) s. Mantis 110
				//-> Holen des Dateinames von selecteden Eintrag und laden des entsprechenden
				//Eintrags in Model
				if(model==null||model.getDateOfEvent()==null||
						(tempSelectedEvent!=null&&!tempSelectedEvent.contains
						(sdf.format(model.getDateOfEvent())) ) )
				{
					//Setzen des Models
					model = dEvtFileMgr.loadDiaryEventFromFile(savePath, SDSharedServices.getFileName(
							user.getUserID(), tempSelectedEvent.substring(0, tempSelectedEvent.indexOf(" ")))
							, user);
				}

				//pos = -1 -> ParentEvent
				if(pos == -1)
				{
					dataSet.addRow("date", sdf.format(model.getDateOfEvent()));
					dataSet.addRow("place", model.getPlace());
					dataSet.addRow("timeStart", model.getTimeStart());
					dataSet.addRow("timeEnd", model.getTimeEnd());
					dataSet.addRow("text", model.getDescription());
					
					if(model.getPictures()!=null&&model.getPictures().size()!=0)
					{
						dataSet.addRow("imageCount", model.getPictures().size());
					}
				}
				//sonst -> SubEvent
				else
				{
					dataSet.addRow("date", sdf.format(model.getSubEvent(pos).getDateOfEvent()));
					dataSet.addRow("place", model.getSubEvent(pos).getPlace());
					dataSet.addRow("timeStart", model.getSubEvent(pos).getTimeStart());
					dataSet.addRow("timeEnd", model.getSubEvent(pos).getTimeEnd());
					dataSet.addRow("text", model.getSubEvent(pos).getDescription());
					
					if(model.getSubEvent(pos).getPictures()!=null&&
							model.getSubEvent(pos).getPictures().size()!=0)
					{
						dataSet.addRow("imageCount", model.getSubEvent(pos).getPictures().size());
					}
				}

				selectedEvent = tempSelectedEvent;
				
				view.setDataSet(dataSet, false);
				
				view.setControlsAtLoadingEvent();
				
				view.setSelectedImage(0);
				
				//Entfernt alle TextAenderungs Aktion des UndoManagers, da diese nur bei Anlage eines neuen
				//Eintrags oder einer Aenderung eines bestehenden Eintrags rueckgaengig gemacht werden
				//koennen.
				SDUndoManager.getInstance().removeUndoActionsForEntryChanges();
				
				//Statusleiste Text anzeigen
				changeStatusLine(IRessources.res.getString("0073"));
			}
			//Wenn es sich um ein einzelnes Event handelt
			else
			{
				doOpenSingleEntry(evt.getPath().getLastPathComponent().toString());
			}
			
			setInChange(false);
		}
		catch(FileForThisUserInvalidException ex)
		{
			dEvtFileMgr.markFileAsInvalid(savePath, SDSharedServices.getFileName(user.getUserID(), 
					evt.getPath().getLastPathComponent().toString()));
			
			updateView();
			
			handleException(ex.getMessage(), getClass().getName(), "valueChanged", ex);
		}
		catch(FileInvalidException ex)
		{
			dEvtFileMgr.markFileAsInvalid(savePath, SDSharedServices.getFileName(user.getUserID(), 
					evt.getPath().getLastPathComponent().toString()));
			
			updateView();
			
			handleException(ex.getMessage(), getClass().getName(), "valueChanged", ex);
		}
		catch(FileNotFoundException ex)
		{
			updateView();
			
			handleException(ex.getMessage(), getClass().getName(), "valueChanged", ex);
		}
		catch (Exception ex) 
		{
			handleException(ex.getMessage(), getClass().getName(), "valueChanged", ex);
		}
	}

	@Override
	public void treeCollapsed(TreeExpansionEvent event) 
	{
	}

	@Override
	public void treeExpanded(TreeExpansionEvent event) 
	{
	}

	@Override
	public void treeWillCollapse(TreeExpansionEvent event) throws ExpandVetoException 
	{
	}

	@Override
	/**
	 * Wird ausgefuehrt, wenn im View ein ParentEvent aufgeklappt wird.
	 */
	public void treeWillExpand(TreeExpansionEvent event) throws ExpandVetoException 
	{
		//Wenn es sich nicht um den Punkt "[Eintraege]" im View handelt
		if(!event.getPath().toString().equals(IRessources.res.getString("0087")))
		{
			DiaryEventFileManager dEvtFileMgr = new DiaryEventFileManager();
			
			try 
			{
				String fileName = SDSharedServices.getFileName(user.getUserID(), 
						event.getPath().getLastPathComponent().toString());		
				
				//Setzen des Models
				model = dEvtFileMgr.loadDiaryEventFromFile(savePath, fileName, user);
				
				//Wenn ein einzelner Eintrag irrtumlicherweise als MultiEintrag interpretiert wird
				//-> Muss aus parentObjects File geloescht werden und View muss aktualisiert werden
				//MANTIS 85
				if(model.getSubEvents()==null||model.getSubEvents().size()==0)
				{
					dEvtFileMgr.doRemarkFileAsParent(savePath, fileName);
					updateView();
				}
				//Sonst -> Alles OK Node wird aufgeklappt
				else
				{
					//Aufruf der Methode expandTreeStock des Views mit Uebergabe des Eintragsnamen
					//im JTree und der Anzahl der SubEvents
					view.expandTreeStock(event.getPath().getLastPathComponent().toString(), 
							model.getSubEvents().size());
				}
				
				view.setControlsAtApplicationStart();
				
				doResetViewDataSet();
			}
			catch(FileForThisUserInvalidException ex)
			{
				dEvtFileMgr.markFileAsInvalid(savePath, SDSharedServices.getFileName(user.getUserID(), 
						event.getPath().getLastPathComponent().toString()));
				
				updateView();
				
				handleException(ex.getMessage(), getClass().getName(), "treeWillExpand", ex);
			}
			catch(FileInvalidException ex)
			{
				dEvtFileMgr.markFileAsInvalid(savePath, SDSharedServices.getFileName(user.getUserID(), 
						event.getPath().getLastPathComponent().toString()));
				
				updateView();
				
				handleException(ex.getMessage(), getClass().getName(), "treeWillExpand", ex);
			}
			catch(FileNotFoundException ex)
			{
				updateView();
				handleException(ex.getMessage(), getClass().getName(), "treeWillExpand", ex);
			}
			catch (Exception ex) 
			{
				handleException(ex.getMessage(), getClass().getName(), "treeWillExpand", ex);
			} 
		}
		
	}

	//-------------------------WINDOW LISTENER-----------------------------------------------------------
	@Override
	public void windowActivated(WindowEvent e) 
	{
	}

	@Override
	public void windowClosed(WindowEvent e) 
	{
	}

	@Override
	/**
	 * Wird ausgeloest wenn der View geschlossen werden soll
	 */
	public void windowClosing(WindowEvent e) 
	{
		if(JOptionPane.showConfirmDialog(null, IRessources.res.getString("0077"),
				IRessources.res.getString("0078"),0)==JOptionPane.YES_OPTION)
		{
			System.exit(0);
		}
	}

	@Override
	public void windowDeactivated(WindowEvent e) 
	{
	}

	@Override
	public void windowDeiconified(WindowEvent e) 
	{
	}

	@Override
	public void windowIconified(WindowEvent e)
	{
	}

	@Override
	public void windowOpened(WindowEvent e) 
	{
	}
	//---------------------------------------------------------------------------------------------------

	
	//--------------------ICalendarListener--------------------------------------------------------------
	@Override
	/**
	 * Handelt Klicks auf Datumsfelder im Kalendar
	 */
	public void doHandleCalendarClick(SDCalendarClickEvent evt) 
	{
		try
		{
			//Wenn angeklickter Eintrag bereits im Model gesetzt ist und keine Untereintraege hat
			//-> Beenden (s. MANTIS 89)
			//
			//Wenn im model kein Datum gesetzt ist -> weiter
			//es handelt sich um einen neuen Eintrag dieser kann verworfen werden (Mantis 33 Notiz 179)
			if(model!=null&&model.getDateOfEvent()!=null&&new SimpleDateFormat(IRessources.DATE_FORMAT).format(model.getDateOfEvent()).
					equals(evt.getDate())&&(model.getSubEvents()==null||model.getSubEvents().size()==0))
			{
				return;
			}
			
			//Es sind ungespeicherte Aenderungen vorhanden
			//und Event wurde nicht durch Rechtsklick ausgeloest. Muss ueberprueft werden da der Dialog
			//"Wollen Sie wirklich..." sonst zwei mal erscheinen wuerde.
			if(!isValueChangeFiredByRightClick&&SDUndoManager.getInstance().hasChangesOnEntry())
			{
				//Wollen Sie wirklich...
				if(JOptionPane.showConfirmDialog(null, IRessources.res.getString("0173"),
						IRessources.res.getString("0174"),0)!=JOptionPane.YES_OPTION)
				{
					//Wenn User nicht auf Ja klickt wird der Vorgang beendet.
					return;
				}
			}
			//Zuruecksetzen egal ob vorher true oder false war.
			isValueChangeFiredByRightClick = false;
			
			//Zu diesem Datum gibt es mehrere Eintraege
			if(evt.isMulti())
			{
				DiaryEventFileManager dEvtFileMgr = new DiaryEventFileManager();

				//Setzen des Models
				model = dEvtFileMgr.loadDiaryEventFromFile(savePath, 
						SDSharedServices.getFileName(user.getUserID(), evt.getDate()), user);
				
				copyModel = null;
				
				view.setControlsAtApplicationStart();
				
				doResetViewDataSet();
				
				//Wenn nicht schon ein Auswahlfenster geoeffnet ist.
				//Sonst GEFAHR -> Exception zB durch Loeschung Eintrag 3. In einem anderem Dialog ist
				//Eintrag 3 aber noch da. Beim Versuch diesen zu loeschen wird eine Exception ausgeloest.
				if(!isCalendarSelectionDialogOpenend)
				{
					//neuer Dialog zur Auswahl des gewuenschten SubEvents
					calSelectSubEntryCtrl = new SDCalendarSelectSubEntryVC(evt.getDate(), 
							model.getSubEvents().size()+1, this);
				}
			}
			//nur ein Eintrag
			else
			{
				try
				{
					doOpenSingleEntry(evt.getDate());
				}
				catch(FileInvalidException ex)
				{
					new DiaryEventFileManager().markFileAsInvalid(savePath, SDSharedServices.getFileName(user.getUserID(), 
							evt.getDate()));
					
					updateView();
					
					handleException(ex.getMessage(), getClass().getName(), "doHandleCalendarClick", ex);
				}
				
			}
		} 
		catch (Exception ex) 
		{
			handleException(ex.getMessage(), getClass().getName(), "doHandleCalendarClick", ex);
		}
	}
	
	@Override
	/**
	 * Handelt Rechtsklicks auf ein Datumsfeld im Kalendar
	 */
	public void doHandleCalendarRightClick(SDCalendarRightClickEvent evt) 
	{
		//Mehrere Eintraege/Tag
		if(evt.isMulti())
		{
			//Angeklicktes Datumsfeld merken
			selectedMultiEvent = evt.getDate();
			
			view.setPopupMenuForMultiEntry(evt.getX(), evt.getY());
		}
		//Einzelner Eintrag
		else
		{
			//Es sind ungespeicherte Aenderungen vorhanden
			if(SDUndoManager.getInstance().hasChangesOnEntry())
			{
				//Wollen Sie wirklich...
				if(JOptionPane.showConfirmDialog(null, IRessources.res.getString("0173"),
						IRessources.res.getString("0174"),0)!=JOptionPane.YES_OPTION)
				{
					//Wenn User nicht auf Ja klickt wird der Vorgang beendet.
					return;
				}
			}
			
			isValueChangeFiredByRightClick = true;
			
			//Eintrag anzeigen, nur bei einzelnen Eintrag da bei MultiEintrag Auswahlfenster kommen muesste
			doHandleCalendarClick(evt);
			
			
			//-------------------------------------------------------------------------------------------
			//		Workaround mit SwingWorker um den Eintrag erst anzuzeigen
			//		bevor das PopUp Menue erscheint.
			//-------------------------------------------------------------------------------------------
			final int x = evt.getX();
			final int y = evt.getY();
			
			SwingWorker<Object, Object> worker = new SwingWorker<Object, Object>()
			{
				@Override
				protected Object doInBackground() throws Exception 
				{
					try
					{
						//Laesst den Thread 0,1 Sekunde ruhen, damit der Eintrag
						//im View dargestellt wird.
						Thread.sleep(100);
						
						//PopUp Menu anzeigen
						view.setPopupMenuForSingleEntry(x, y);
					}
					catch(Exception ex)
					{
						handleException(ex.getMessage(), getClass().getName(), "doHandleCalendarRightClick",
								ex);
					}
					
					return null;
				}
			};
			
			worker.execute();
			//-----------------------------------------------------------------------------------------
		}
	}
	//---------------------------------------------------------------------------------------------------
	

	//-----------------------------MenuListener----------------------------------------------------------
	@Override
	public void menuCanceled(MenuEvent arg0) 
	{
	}

	@Override
	public void menuDeselected(MenuEvent arg0) 
	{
	}

	@Override
	/**
	 * Handelt Klicks auf einen Menuepunkt.
	 * Einziger Menuepunkt der dieses Event ausloesen kann is "Bearbeiten" -> Daher keine weitere
	 * Pruefung der Source noetig.
	 */
	public void menuSelected(MenuEvent evt) 
	{
		//Holen der TextKomponente des Views die den Focus hat
		JTextComponent comp = view.getFocusedTextComponent();
		
		// comp==null wenn keine TextKomponente den Focus hat -> Setzt alle MenuItems disabled
		if(comp==null)
		{
			view.setCutCopyPasteEnabled(false, false, false);
		}
		else
		{
			//Wenn ein Text markiert wurde
			if(comp.getSelectedText()!=null&&!comp.getSelectedText().isEmpty())
			{
				//Wenn Komponente editierbar ist
				// -> Setzen aller MenuItems auf enabled
				if(comp.isEditable())
				{
					view.setCutCopyPasteEnabled(true, true, true);
				}
				//Komponente ist nicht editierbar -> Setzen MenuItems:
				// Cut - disabled (Kann nichts ausgeschnitten werden)
				// Copy - enabled (Kopiert kann der Text werden)
				// Paste - disabled (Kann nichts eingefuegt werden)
				else
				{
					view.setCutCopyPasteEnabled(false, true, false);
				}
			}
			//Es wurde kein Text markiert
			else
			{
				//Die Komponente ist editierbar -> Setzen MenuItems:
				// Cut - disabled
				// Copy - disabled
				// Paste - enabeld
				if(comp.isEditable())
				{
					view.setCutCopyPasteEnabled(false, false, true);
				}
				//Komponente ist nicht editierbar
				// -> Setzen aller MenuItems auf disabled
				else
				{
					view.setCutCopyPasteEnabled(false, false, false);
				}
			}
		}
	}
	//--------------------------------------------------------------------------------------------------
	

	//------------------------------MouseListener-------------------------------------------------------
	@Override
	public void mouseClicked(MouseEvent e) 
	{
	}

	@Override
	public void mouseEntered(MouseEvent e) 
	{
	}

	@Override
	public void mouseExited(MouseEvent e) 
	{
	}

	@Override
	/**
	 * Wird ausgefuehrt bei einem rechten Mausklick auf den Tree.
	 */
	public void mousePressed(MouseEvent evt) 
	{
		//Rechtsklick auf Tree
		if(evt.getButton()==3&&evt.getSource() instanceof JTree)
		{
			JTree tree = (JTree)evt.getSource();
			
			//Holen angeklickten Eintrag
			int selectedRow = tree.getRowForLocation(evt.getX(), evt.getY());
			
			//Wenn ueberhaupt auf einen Eintrag geklickt wurde und es ein einzelner Klick war
			if(selectedRow != -1&&evt.getClickCount()==1) 
			{
				//Wenn der angeklickte Eintrag nicht bereits gesetzt ist.
				if(!(selectedEvent!=null&&selectedEvent.equals(tree.getPathForLocation(evt.getX(), evt.getY()).
						getLastPathComponent().toString())))
				{
					//Es sind ungespeicherte Aenderungen vorhanden
					if(SDUndoManager.getInstance().hasChangesOnEntry())
					{
						//Wollen Sie wirklich...
						if(JOptionPane.showConfirmDialog(null, IRessources.res.getString("0173"),
								IRessources.res.getString("0174"),0)!=JOptionPane.YES_OPTION)
						{
							//Wenn User nicht auf Ja klickt wird der Vorgang beendet.
							return;
						}
					}
					
					isValueChangeFiredByRightClick = true;
					
					//Eintrag selecten -> Bewirkt, dass die Methode valueChanged ausgefuehrt wird.
					tree.setSelectionRow(selectedRow);
				}
				
				
				//-------------------------------------------------------------------------------------------
				//		Workaround mit SwingWorker um den Eintrag erst zu selektieren und anzuzeigen
				//		bevor das PopUp Menue erscheint.
				//-------------------------------------------------------------------------------------------
				final int x_worker = evt.getX();
				final int y_worker = evt.getY();
				final JTree tree_worker = tree;
				
				SwingWorker<Object, Object> worker = new SwingWorker<Object, Object>()
				{
					@Override
					protected Object doInBackground() throws Exception 
					{
						try
						{
							//Laesst den Thread 0,1 Sekunde ruhen, damit der Eintrag selectiert wird im Tree
							//und der Eintrag selbst im View dargestellt wird.
							Thread.sleep(100);
							
							//Rechtsklick behandeln mit Uebergabe des Eintrags.
							handleRightClickOnEntry(tree_worker.getPathForLocation(x_worker, y_worker).
														getLastPathComponent().toString(), x_worker, y_worker);
						}
						catch(Exception ex)
						{
							handleException(ex.getMessage(), getClass().getName(), "mousePressed", ex);
						}
					
						return null;
					}
				};
				
				worker.execute();
				//-------------------------------------------------------------------------------------------
			} 
		}
	}

	@Override
	public void mouseReleased(MouseEvent e) 
	{
	}
	//-------------------------------------------------------------------------------------------------------
}