package de.joerg.diary.frontend.general;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import javax.swing.JOptionPane;

import de.joerg.diary.backend.userManagement.UserAccountsManager;
import de.joerg.diary.frontend.sdBasicComponents.ViewController;
import de.joerg.diary.shared.IRessources;
import de.joerg.diary.shared.dataSet.DataSet;
import de.joerg.diary.shared.models.User;

/*
 *  SimpleDiary Version 2.0.0
    Copyright (C) 2010  Andreas Joerg (E-Mail: simpleDiary@freenet.de)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * ViewController fuer einen Dialog um einen neuen User zu erstellen
 * 
 * @author Andreas Joerg
 */
public class NewUserDialogVC extends ViewController implements WindowListener, ActionListener
{
	private NewUserDialogV view = null;
	private LoginDialogVC parentView = null;
	
	/**
	 * Erweiterter Konstruktor
	 * 
	 * @param parentView
	 */
	public NewUserDialogVC(LoginDialogVC parentView)
	{
		setParentView(parentView);
		setView(new NewUserDialogV(this));
		view.init();
	}

	//---------Getter/Setter---
	public NewUserDialogV getView()
	{
		return view;
	}
	public void setView(NewUserDialogV view) 
	{
		this.view = view;
	}

	public LoginDialogVC getParentView() 
	{
		return parentView;
	}
	public void setParentView(LoginDialogVC parentView) 
	{
		this.parentView = parentView;
	}
	//----------------------
	
	@Override
	public void windowActivated(WindowEvent e) 
	{
	}

	@Override
	/**
	 * Wird ausgefuehrt wenn der View geschlossen wird
	 */
	public void windowClosed(WindowEvent e) 
	{
		parentView.getView().setVisible(true);
	}

	@Override
	public void windowClosing(WindowEvent e) 
	{
	}

	@Override
	public void windowDeactivated(WindowEvent e)
	{
	}

	@Override
	public void windowDeiconified(WindowEvent e) 
	{
	}

	@Override
	public void windowIconified(WindowEvent e) 
	{
	}

	@Override
	public void windowOpened(WindowEvent e) 
	{
	}

	@Override
	/**
	 * Reagiert auf ActionEvents des Views
	 */
	public void actionPerformed(ActionEvent evt)
	{
		//Abbrechen
		if(evt.getActionCommand().equals("0004"))
		{
			view.dispose();
			parentView.getView().setVisible(true);
		}
		//User anlegen
		else if(evt.getActionCommand().equals("0003"))
		{
			UserAccountsManager usrAccMgr = new UserAccountsManager();
			try 
			{
				DataSet dataSet = view.getDataSet();
				
				if(((String) dataSet.getRowValueByName("userName")).isEmpty())
				{
					throw new Exception(IRessources.res.getString("0012"));
				}
				
				if(((String) dataSet.getRowValueByName("userPw")).isEmpty())
				{
					throw new Exception(IRessources.res.getString("0013"));
				}
				if(((String) dataSet.getRowValueByName("userPwRep")).isEmpty())
				{
					throw new Exception(IRessources.res.getString("0091"));
				}
				if(!((String)dataSet.getRowValueByName("userPw")).equals(
						(String)dataSet.getRowValueByName("userPwRep") ))
				{
					throw new Exception(IRessources.res.getString("0092"));
				}

				//Speichern des Users
				usrAccMgr.addUser(new User((String) view.getDataSet().getRowValueByName("userName"),
						String.valueOf(view.getDataSet().getRowValueByName("userPw").hashCode())));
				
				JOptionPane.showMessageDialog(view, IRessources.res.getString("0093"),
							IRessources.res.getString("0003"),1);
				
				view.dispose();
			}
			catch (Exception ex) 
			{
				handleException(ex.getMessage(), getClass().getName(), "actionPerformed",ex);
			}
		}
	}
}
