package de.joerg.diary.frontend.general;

import javax.swing.JButton;
import javax.swing.JTabbedPane;
import de.joerg.diary.frontend.sdBasicComponents.BackGroundPane;
import de.joerg.diary.frontend.sdBasicComponents.JPanelWithDataSet;
import de.joerg.diary.frontend.sdBasicComponents.View;
import de.joerg.diary.shared.IRessources;
import de.joerg.diary.shared.dataSet.DataSet;
import de.joerg.diary.shared.dataSet.DataSetRowAlreadyExistsException;

/*
 *  SimpleDiary Version 2.0.0
    Copyright (C) 2010  Andreas Joerg (E-Mail: simpleDiary@freenet.de)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * View fuer einen OptionenDialog
 * 
 * @author Andreas Joerg
 */
public class OptionsV extends View
{
	private static final long serialVersionUID = -7650399906240260860L;
	private OptionsVC controller = null;
	private JTabbedPane tabPane = null;
	
	/**
	 * Default Constructor
	 */
	public OptionsV()
	{
		
	}
	
	/**
	 * Erweiterter Konstruktor
	 * 
	 * @param controller
	 */
	public OptionsV(OptionsVC controller)
	{
		setController(controller);
	}

	//-------Getter/Setter
	public void setController(OptionsVC controller) 
	{
		this.controller = controller;
	}
	public OptionsVC getController() 
	{
		return controller;
	}
	//----------------------
	
	/**
	 * Initialisierung des Views
	 */
	public void init()
	{
		this.setSize(760, 540);
		this.setContentPane(new BackGroundPane(IRessources.BG_IMAGE));
		this.setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		this.addWindowListener(getController());
		this.setTitle(IRessources.res.getString("0044"));
		this.setLayout(null);
		this.setResizable(false);
		
		tabPane = new JTabbedPane();
		tabPane.setBounds(10, 10, 730, 450);
		tabPane.setOpaque(false);
		
		JButton btnOK = new JButton(IRessources.res.getString("0010"));
		btnOK.setBounds(510, 470, 230, 30);
		btnOK.setActionCommand("0016");
		btnOK.addActionListener(getController());
		
		this.add(btnOK);
		this.add(tabPane);
		this.setVisible(true);
	}
	
	/**
	 * Fuegt dem TabPane einen Tab hinzu.
	 * 
	 * @param desc
	 * @param tab
	 */
	public void addTab(String desc, JPanelWithDataSet tab)
	{
		tabPane.addTab(desc, tab);
	}
	
	/**
	 * Gibt das DataSet des aktiven Panels zurueck
	 * 
	 * @return ((JPanelWithDataSet)tabPane.getSelectedComponent()).getDataSet()
	 * @throws DataSetRowAlreadyExistsException 
	 */
	public DataSet getDataSet() throws DataSetRowAlreadyExistsException
	{
		return ((JPanelWithDataSet)tabPane.getSelectedComponent()).getDataSet();
	}
	
	/**
	 * Setzt das DataSet des Views. In dem Fall das DataSets des aktuellen Tabs in der JTabbedPane
	 * 
	 * @param dataSet
	 * @throws Exception
	 */
	public void setDataSet(DataSet dataSet) throws Exception
	{
		((JPanelWithDataSet)tabPane.getSelectedComponent()).setDataSet(dataSet);
	}
}
