package de.joerg.diary.frontend.general;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.File;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import de.joerg.diary.backend.userManagement.UserAccountsManager;
import de.joerg.diary.frontend.sdBasicComponents.ViewController;
import de.joerg.diary.frontend.sdUndoManager.SDUndoManager;
import de.joerg.diary.shared.IRessources;
import de.joerg.diary.shared.dataSet.DataSet;
import de.joerg.diary.shared.dataSet.DataSetRowAlreadyExistsException;
import de.joerg.diary.shared.models.User;

/*
 *  SimpleDiary Version 2.0.0
    Copyright (C) 2010  Andreas Joerg (E-Mail: simpleDiary@freenet.de)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * ViewController fuer einen OptionenDialog
 * 
 * @author Andreas Joerg
 */
public class OptionsVC extends ViewController implements ActionListener, WindowListener
{
	private OptionsV view = null;
	private String savePath = null;
	private User user = null;
	private MainWindowVC mainWindow = null;
	
	/**
	 * Erweiterter Konstruktor
	 * 
	 * @param mainWindow
	 * @throws DataSetRowAlreadyExistsException 
	 */
	public OptionsVC(MainWindowVC mainWindow) throws DataSetRowAlreadyExistsException
	{
		setMainWindow(mainWindow);
		setUser(mainWindow.getUser());
		setSavePath(mainWindow.getSavePath());
		mainWindow.getView().setControlOptionsEnable(false);
		
		view = new OptionsV(this);
		view.init();
		UserAccountPnl usrAccPnl = new UserAccountPnl(this);
		
		DataSet dataSet = new DataSet();
		dataSet.addRow("userName", user.getUserName());
		dataSet.addRow("oldPw", IRessources.EMPTY);
		dataSet.addRow("newPw", IRessources.EMPTY);
		dataSet.addRow("newPwRep", IRessources.EMPTY);

		try
		{
			usrAccPnl.setDataSet(dataSet);
		} 
		catch (Exception ex) 
		{
			handleException(ex.getMessage(), getClass().getName(), "constructor", ex);
		}
		
		//Fuegt die Panels dem TabPane des Views hinzu
		view.addTab("Benutzerkonto", usrAccPnl);
		view.addTab("Speicherpfad", new SavePathPnl(this,savePath));
	}

	//-----------Getter/Setter--------
	public void setSavePath(String savePath) 
	{
		this.savePath = savePath;
	}
	public String getSavePath() 
	{
		return savePath;
	}

	public User getUser() 
	{
		return user;
	}
	public void setUser(User user) 
	{
		this.user = user;
	}

	public void setMainWindow(MainWindowVC mainWindow) 
	{
		this.mainWindow = mainWindow;
	}
	public MainWindowVC getMainWindow()
	{
		return mainWindow;
	}
	
	public OptionsV getView()
	{
		return view;
	}
	//---------------------------

	@Override
	/**
	 * Reagiert auf ActionEvents des Views
	 */
	public void actionPerformed(ActionEvent evt) 
	{
		//Speicherpfad aendern
		if(evt.getActionCommand().equals("0011"))
		{
			try
			{
				String savePath = (String) view.getDataSet().getRowValueByName("savePath");
				
				//Speicherpfad bereits gesetzt
				if(savePath.equals(mainWindow.getSavePath()))
				{
					throw new Exception(IRessources.res.getString("0231"));
				}
				
				File validPath = new File(savePath);
				
				if(!validPath.exists())
				{
					throw new Exception(IRessources.res.getString("0094"));
				}
				
				new UserAccountsManager().changeUserSavePath(user, savePath);
				
				//alle undoactions cleanen
				SDUndoManager.getInstance().getActions().clear();
				
				mainWindow.setSavePath(savePath);
				mainWindow.updateView();
				
				JOptionPane.showMessageDialog(view, IRessources.res.getObject("0095"),
						IRessources.res.getString("0003"), 1);
			} 
			catch (Exception ex) 
			{
				handleException(ex.getMessage(), getClass().getName(), "actionPerformed",ex);
			}
		}
		//Passwort aendern
		else if(evt.getActionCommand().equals("0012"))
		{
			DataSet dataSet;
			try 
			{
				dataSet = view.getDataSet();
				
				String oldPw = ((String) dataSet.getRowValueByName("oldPw"));
				String newPw = ((String) dataSet.getRowValueByName("newPw"));
				String newPwRep = ((String) dataSet.getRowValueByName("newPwRep"));
				
				if(dataSet.getRowsCount()!=4)
				{
					throw new Exception(IRessources.res.getString("0096"));
				}
				if(oldPw.isEmpty())
				{
					throw new Exception(IRessources.res.getString("0097"));
				}
				if(!String.valueOf(oldPw.hashCode()).equals(user.getUserPW()))
				{
					throw new Exception(IRessources.res.getString("0098"));
				}
				if(newPw.isEmpty())
				{
					throw new Exception(IRessources.res.getString("0099"));
				}
				if(newPwRep.isEmpty())
				{
					throw new Exception(IRessources.res.getString("0100"));
				}
				if(!newPw.equals(newPwRep))
				{
					throw new Exception(IRessources.res.getString("0101"));
				}
				if(oldPw.equals(newPw))
				{
					throw new Exception(IRessources.res.getString("0102"));
				}
				
				if(JOptionPane.showConfirmDialog(view, IRessources.res.getString("0103"),
						IRessources.res.getString("0104"),0)==JOptionPane.YES_OPTION)
				{
					UserAccountsManager usrAccMgr = new UserAccountsManager();

					usrAccMgr.changeUserPassword(user, String.valueOf(newPw.hashCode()), savePath);
					user = mainWindow.getUser();
					user.setUserPW(String.valueOf(newPw.hashCode()));
					mainWindow.updateView();
					JOptionPane.showMessageDialog(view, IRessources.res.getString("0105"),
							IRessources.res.getString("0003"), 1);
				}	
			} 
			catch (Exception ex) 
			{
				handleException(ex.getMessage(), getClass().getName(), "actionPerformed", ex);
			}	
		}
		
		//User Account loeschen
		else if(evt.getActionCommand().equals("0013"))
		{
			if(JOptionPane.showConfirmDialog(view, IRessources.res.getString("0106"),
					IRessources.res.getString("0107"),0)==JOptionPane.YES_OPTION)
			{
				try 
				{
					UserAccountsManager usrAccMgr = new UserAccountsManager();
					usrAccMgr.delUser(user, savePath);
					view.dispose();
					mainWindow.getView().dispose();
					new LoginDialogVC();
				} 
				catch (Exception ex) 
				{
					handleException(ex.getMessage(), getClass().getName(), "actionPerformed", ex);
				}
			}
		}
		//OK
		else if(evt.getActionCommand().equals("0016"))
		{
			view.dispose();
		}
		//Verzeichnisauswahl Dialog anzeigen
		else if(evt.getActionCommand().equals("0029"))
		{
			//neuer JFileChooser Dialog
			JFileChooser choosePath = new JFileChooser(); 
			choosePath.setDialogType(JFileChooser.OPEN_DIALOG); 
			//Nur Verzeichnisse koennen ausgewaehlt werden
	        choosePath.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY); 
	        choosePath.setDialogTitle(IRessources.res.getString("0108"));
	        
	        choosePath.setVisible(true);
	        
	        if(choosePath.showOpenDialog(getView())==JFileChooser.APPROVE_OPTION)
	        {
	        	String tempPath = choosePath.getSelectedFile().getPath();
	        	
	        	//Validierung ob Pfad vorhanden (Evtl. Pfad in Zwischenzeit geloescht oder User hat
	        	//zum eingentlichen Pfad (aus Versehen!?) noch etwas dazu getippt).
	        	if(!new File(tempPath).exists())
	        	{
	        		handleException(IRessources.res.getString("0094"), getClass().getName(), 
	        				"actionPerformed");
					return;
	        	}
	        	
	        	//Setzt den Pfad in das Speicherpfad Textfeld
	        	DataSet dataSet = new DataSet();
	        	try 
	        	{
					dataSet.addRow("savePath", tempPath);
					view.setDataSet(dataSet);
				} 
	        	catch (Exception ex) 
	        	{
	        		handleException(ex.getMessage(), getClass().getName(), "actionPerformed", ex);
				}
	        }
	        
	        choosePath.setVisible(false);
		}
	}

	@Override
	public void windowActivated(WindowEvent e) 
	{	
	}

	@Override
	/**
	 * Wird ausgefuehrt wenn der View geschlossen wurde
	 */
	public void windowClosed(WindowEvent evt) 
	{
		mainWindow.getView().setControlOptionsEnable(true);
		mainWindow.doOptionsClose();
	}

	@Override
	public void windowClosing(WindowEvent e) 
	{
	}

	@Override
	public void windowDeactivated(WindowEvent e) 
	{
	}

	@Override
	public void windowDeiconified(WindowEvent e) 
	{
	}

	@Override
	public void windowIconified(WindowEvent e) 
	{
	}

	@Override
	public void windowOpened(WindowEvent e) 
	{
	}
}
