package de.joerg.diary.frontend.general;

import java.awt.Color;
import java.awt.Font;
import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;
import de.joerg.diary.frontend.sdBasicComponents.BackGroundPane;
import de.joerg.diary.frontend.sdBasicComponents.EmptyPanel;
import de.joerg.diary.frontend.sdBasicComponents.View;
import de.joerg.diary.shared.IRessources;
import de.joerg.diary.shared.dataSet.DataSet;
import de.joerg.diary.shared.dataSet.DataSetRowAlreadyExistsException;

/*
 *  SimpleDiary Version 2.0.0
    Copyright (C) 2010  Andreas Joerg (E-Mail: simpleDiary@freenet.de)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
* Beschreibt einen View fuer den "Suche"-Dialog
*
* @author Andreas Joerg
*/
public class SearchV extends View
{
	private static final long serialVersionUID = -5170198981525459368L;
	private SearchVC controller = null;
	private JButton btnOK;
	private JButton btnSearch;
	private JLabel lblSearchCrit;	
	private JLabel lblSearchResults;
	private JLabel lblSearchType;
	private EmptyPanel pnlSearchResults;
	private JRadioButton rbFull;
	private JRadioButton rbKeyword;
	private JRadioButton rbPlace;
	private JScrollPane scrSearchResults;
	private JTextField searchCrit;
	private JTextArea searchResult;
	private DataSet dataSet = null;
	private JCheckBox timeSpanActiv;
    private JTextField timeSpanBegin;
    private JTextField timeSpanEnd;
    private JLabel lblTimeSpanEnd;
    private JLabel lblTimespanBegin;
    private EmptyPanel pnlTimespan;
	
	/**
	 * Konstruktor mit Uebergabe des Controllers.
	 * 
	 * @param controller
	 */
	public SearchV(SearchVC controller)
	{
		setController(controller);
		
		initComponents();
		
		initDataSet();
	}
	
	// ------- GETTER / SETTER ------
	public void setController(SearchVC controller) 
	{
		this.controller = controller;
	}
	public SearchVC getController() 
	{
		return controller;
	}

	public DataSet getDataSet() 
	{
		return dataSet;
	}
	public void setDataSet(DataSet dataSet) 
	{
		this.dataSet = dataSet;
	}
	//-------------------------------

	/**
	 * Initialisieren der ViewComponents.
	 */
	private void initComponents() 
	{
		Font font = new Font("Verdana", Font.BOLD, 12);
		Font fontTwo = new Font("Verdana", Font.BOLD, 10);
		
	   lblSearchType = new JLabel();
       rbKeyword = new JRadioButton();
       rbFull = new JRadioButton();
       rbPlace = new JRadioButton();
       lblSearchCrit = new JLabel();
       searchCrit = new JTextField();
       lblSearchResults = new JLabel();
       
       // ------ ZeitraumbeschraenkungsPanel ------
       timeSpanActiv = new JCheckBox(IRessources.res.getString("0219"));
       timeSpanActiv.setMargin(new java.awt.Insets(0, 0, 0, 0));
       timeSpanActiv.setFont(fontTwo);
       timeSpanActiv.setForeground(Color.WHITE);
       timeSpanActiv.setOpaque(false);
       timeSpanActiv.setActionCommand("0047");
       timeSpanActiv.addActionListener(getController());
       
       timeSpanBegin = new JTextField();
       timeSpanBegin.setEditable(false);
       timeSpanEnd = new JTextField();
       timeSpanEnd.setEditable(false);
       
       lblTimespanBegin = new JLabel(IRessources.res.getString("0220"));
       lblTimespanBegin.setFont(fontTwo);
       lblTimespanBegin.setForeground(Color.WHITE);
       
       lblTimeSpanEnd = new JLabel(IRessources.res.getString("0060"));
       lblTimeSpanEnd.setFont(fontTwo);
       lblTimeSpanEnd.setForeground(Color.WHITE);
       
       pnlTimespan = new EmptyPanel();
       pnlTimespan.setBorder(BorderFactory.createTitledBorder(null,
    		   "Zeitraumbeschr\u00E4nkung", TitledBorder.DEFAULT_JUSTIFICATION, 
    		   TitledBorder.DEFAULT_POSITION, new Font("Tahoma", 0, 11), 
    		   Color.WHITE));
       // ------------------------------------------
       
       searchResult = new JTextArea();
       searchResult.setEditable(false);
       scrSearchResults = new JScrollPane(searchResult);
       
       btnOK = new JButton();
       btnSearch = new JButton();
       
       pnlSearchResults = new EmptyPanel();
       pnlSearchResults.setVisible(false);

       setContentPane(new BackGroundPane(IRessources.BG_IMAGE));
       setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
       setTitle(IRessources.res.getString("0208"));
       setLocation(100, 100);
       
       // ------------------ Labels ------------------
       lblSearchType.setText(IRessources.res.getString("0214"));
       lblSearchType.setFont(font);
       lblSearchType.setForeground(Color.WHITE);
       
       lblSearchCrit.setText(IRessources.res.getString("0215"));
       lblSearchCrit.setFont(font);
       lblSearchCrit.setForeground(Color.WHITE);
       
       lblSearchResults.setText(IRessources.res.getString("0216"));
       lblSearchResults.setFont(font);
       lblSearchResults.setForeground(Color.WHITE);
       //---------------------------------------------
       
       // ------ Radio Buttons ------
       rbKeyword.setSelected(true);
       rbKeyword.setText(IRessources.res.getString("0201"));
       rbKeyword.setFont(fontTwo);
       rbKeyword.setForeground(Color.WHITE);
       rbKeyword.setOpaque(false);
       
       rbFull.setText(IRessources.res.getString("0217"));
       rbFull.setFont(fontTwo);
       rbFull.setForeground(Color.WHITE);
       rbFull.setOpaque(false);
       
       rbPlace.setText(IRessources.res.getString("0218"));
       rbPlace.setFont(fontTwo);
       rbPlace.setForeground(Color.WHITE);
       rbPlace.setOpaque(false);
       
       ButtonGroup btnGrp = new ButtonGroup();
       btnGrp.add(rbKeyword);
       btnGrp.add(rbFull);
       btnGrp.add(rbPlace);
       // ---------------------------
       
       // --------- Buttons ---------
       btnOK.setText(IRessources.res.getString("0010"));
       btnOK.setActionCommand("0044");
       btnOK.addActionListener(controller);

       btnSearch.setText(IRessources.res.getString("0208"));
       btnSearch.setActionCommand("0045");
       btnSearch.addActionListener(controller);
       // ---------------------------
       
       
       //------------------ LAYOUT BY MATISSE ------------------------------------------------------------------
       javax.swing.GroupLayout pnlSearchResultsLayout = new javax.swing.GroupLayout(pnlSearchResults);
       pnlSearchResults.setLayout(pnlSearchResultsLayout);
       pnlSearchResultsLayout.setHorizontalGroup(
           pnlSearchResultsLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
           .addGroup(pnlSearchResultsLayout.createSequentialGroup()
               .addComponent(lblSearchResults)
               .addContainerGap())
           .addComponent(scrSearchResults, javax.swing.GroupLayout.DEFAULT_SIZE, 464, Short.MAX_VALUE)
       );
       pnlSearchResultsLayout.setVerticalGroup(
           pnlSearchResultsLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
           .addGroup(pnlSearchResultsLayout.createSequentialGroup()
               .addComponent(lblSearchResults)
               .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
               .addComponent(scrSearchResults, javax.swing.GroupLayout.DEFAULT_SIZE, 222, Short.MAX_VALUE))
       );
       javax.swing.GroupLayout pnlTimespanLayout = new javax.swing.GroupLayout(pnlTimespan);
       pnlTimespan.setLayout(pnlTimespanLayout);
       pnlTimespanLayout.setHorizontalGroup(
           pnlTimespanLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
           .addGroup(pnlTimespanLayout.createSequentialGroup()
               .addContainerGap()
               .addGroup(pnlTimespanLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                   .addComponent(timeSpanActiv, javax.swing.GroupLayout.PREFERRED_SIZE, 79, javax.swing.GroupLayout.PREFERRED_SIZE)
                   .addGroup(pnlTimespanLayout.createSequentialGroup()
                       .addComponent(lblTimespanBegin)
                       .addGap(18, 18, 18)
                       .addComponent(timeSpanBegin, javax.swing.GroupLayout.PREFERRED_SIZE, 71, javax.swing.GroupLayout.PREFERRED_SIZE)
                       .addGap(18, 18, 18)
                       .addComponent(lblTimeSpanEnd)
                       .addGap(18, 18, 18)
                       .addComponent(timeSpanEnd, javax.swing.GroupLayout.PREFERRED_SIZE, 71, javax.swing.GroupLayout.PREFERRED_SIZE)))
               .addContainerGap(207, Short.MAX_VALUE))
       );
       pnlTimespanLayout.setVerticalGroup(
           pnlTimespanLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
           .addGroup(pnlTimespanLayout.createSequentialGroup()
               .addContainerGap()
               .addComponent(timeSpanActiv, javax.swing.GroupLayout.PREFERRED_SIZE, 23, javax.swing.GroupLayout.PREFERRED_SIZE)
               .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
               .addGroup(pnlTimespanLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                   .addComponent(lblTimespanBegin)
                   .addComponent(timeSpanBegin, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                   .addComponent(lblTimeSpanEnd)
                   .addComponent(timeSpanEnd, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
               .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
       );

       javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
       getContentPane().setLayout(layout);
       layout.setHorizontalGroup(
           layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
           .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
               .addContainerGap()
               .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                   .addComponent(pnlSearchResults, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                   .addGroup(layout.createSequentialGroup()
                       .addComponent(btnSearch, javax.swing.GroupLayout.PREFERRED_SIZE, 85, javax.swing.GroupLayout.PREFERRED_SIZE)
                       .addGap(18, 18, 18)
                       .addComponent(btnOK, javax.swing.GroupLayout.PREFERRED_SIZE, 85, javax.swing.GroupLayout.PREFERRED_SIZE))
                   .addComponent(searchCrit, javax.swing.GroupLayout.DEFAULT_SIZE, 464, Short.MAX_VALUE)
                   .addComponent(pnlTimespan, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                   .addComponent(lblSearchCrit, javax.swing.GroupLayout.Alignment.LEADING)
                   .addGroup(javax.swing.GroupLayout.Alignment.LEADING, layout.createSequentialGroup()
                       .addComponent(lblSearchType)
                       .addGap(18, 18, 18)
                       .addComponent(rbKeyword)
                       .addGap(18, 18, 18)
                       .addComponent(rbFull)
                       .addGap(18, 18, 18)
                       .addComponent(rbPlace)))
               .addContainerGap())
       );
       layout.setVerticalGroup(
           layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
           .addGroup(layout.createSequentialGroup()
               .addContainerGap()
               .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                   .addComponent(lblSearchType)
                   .addComponent(rbKeyword)
                   .addComponent(rbFull)
                   .addComponent(rbPlace))
               .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
               .addComponent(lblSearchCrit)
               .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
               .addComponent(searchCrit, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
               .addGap(12, 12, 12)
               .addComponent(pnlTimespan, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
               .addGap(18, 18, 18)
               .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                   .addComponent(btnOK)
                   .addComponent(btnSearch))
               .addGap(18, 18, 18)
               .addComponent(pnlSearchResults, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
               .addContainerGap())
       );
       //-------------------------------------------------------------------------------------------------------
       
       pack();
   }
	
	/**
	 * Init des DataSets.
	 */
	private void initDataSet()
	{
		dataSet = new DataSet();
		
		try 
		{
			dataSet.addRow("crit", null);
			dataSet.addRow("result", null);
			dataSet.addRow("timeSpanBegin", null);
			dataSet.addRow("timeSpanEnd", null);
		} 
		catch (DataSetRowAlreadyExistsException ex) 
		{
			//Exception kann eigentlich nicht auftreten
			getController().handleException(ex.getMessage(), getClass().getName(), "initDataSet", ex);
		}
	}
	
	/**
	 * Schreiben der Werte aus dem DataSet in die Komponenten.
	 */
	public void updateView()
	{
		searchCrit.setText(dataSet.getString("crit"));
		searchResult.setText(dataSet.getString("result"));
		timeSpanBegin.setText(dataSet.getString("timeSpanBegin"));
		timeSpanEnd.setText(dataSet.getString("timeSpanEnd"));
	}
	
	/**
	 * Schreiben der Werte aus dem Komponenten ins DataSet.
	 */
	public void updateDataSet()
	{
		dataSet.setValueOfRow("crit", searchCrit.getText());
		dataSet.setValueOfRow("result", searchResult.getText());
		dataSet.setValueOfRow("timeSpanBegin", timeSpanBegin.getText());
		dataSet.setValueOfRow("timeSpanEnd", timeSpanEnd.getText());
	}
	
	/**
	 * @return selectedRadioButton 0="Stichwort", 1="Volltext", 2="Ort"
	 */
	public int getSelectedType()
	{
		if(rbKeyword.isSelected())
		{
			return 0;
		}
		else if(rbFull.isSelected())
		{
			return 1;
		}
		else if(rbPlace.isSelected())
		{
			return 2;
		}
		return -1;
	}

	/**
	 * Stellt Zustaende der Komponenten entsprechend der Aktion.
	 * 
	 * @param action
	 */
	public void updateComponents(String action) 
	{
		//Suche wurde abgeschlossen -> Panel mit Suchergebnisen anzeigen
		if(action.equals("search.finished"))
		{
			//Nur sichtbar setzen wenn nicht schon geschehen
			if(pnlSearchResults.isVisible() == false)
			{
				pnlSearchResults.setVisible(true);
				pack();
			}
		}
		//Zeitraumbeschraenkung ist aktiv -> Textfelder enablen
		else if(action.equals("timeSpan.enabled"))
		{
			timeSpanEnd.setEditable(true);
			timeSpanBegin.setEditable(true);
		}
		//Zeitraumbeschraenkung ist inaktiv -> Textfelder disablen
		else if(action.equals("timeSpan.disabled"))
		{
			timeSpanEnd.setEditable(false);
			timeSpanBegin.setEditable(false);
		}
	}
}

