package de.joerg.diary.frontend.general;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;

import javax.swing.JCheckBox;

import de.joerg.diary.backend.fileManagement.DiaryEventFileManager;
import de.joerg.diary.frontend.sdBasicComponents.ViewController;
import de.joerg.diary.shared.IRessources;
import de.joerg.diary.shared.dataSet.DataSet;

/*
 *  SimpleDiary Version 2.0.0
    Copyright (C) 2010  Andreas Joerg (E-Mail: simpleDiary@freenet.de)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Beschreibt einen ViewConroller fuer den "Suche"-Dialog.
 * 
 * @author Andreas Joerg
 */
public class SearchVC extends ViewController implements ActionListener
{
	private SearchV view = null;
	private MainWindowVC parent = null;
	private boolean isTimeSpanActiv = false;

	/**
	 * Konstruktor mit Uebergabe des ParentControllers.
	 * 
	 * @param controller
	 */
	public SearchVC(MainWindowVC controller)
	{
		setParent(controller);
		
		setView(new SearchV(this));
		getView().setVisible(true);
	}
	
	// ------ GETTER / SETTER -------
	public void setView(SearchV view) 
	{
		this.view = view;
	}
	public SearchV getView() 
	{
		return view;
	}

	public MainWindowVC getParent() 
	{
		return parent;
	}
	public void setParent(MainWindowVC parent) 
	{
		this.parent = parent;
	}
	//-------------------------------

	@Override
	public void actionPerformed(ActionEvent evt)
	{
		//OK -> Dialog schliessen
		if(evt.getActionCommand().equals("0044"))
		{
			view.dispose();
		}
		//Suchen
		else if(evt.getActionCommand().equals("0045"))
		{
			doSearch();
		}
		//Zeitraumbeschraenkung
		else if(evt.getActionCommand().equals("0047"))
		{
			//Ist Checkbox angehakt
			isTimeSpanActiv =((JCheckBox)evt.getSource()).isSelected();
			
			//Wenn ja Textfelder enablend, sonst disablen
			if(isTimeSpanActiv)
			{
				getView().updateComponents("timeSpan.enabled");
			}
			else
			{
				getView().updateComponents("timeSpan.disabled");
			}
		}
	}

	/**
	 * Fuehrt eine Suche aus.
	 */
	private void doSearch() 
	{
		try 
		{
			//View DS updaten und holen
			getView().updateDataSet();
			DataSet dsView = getView().getDataSet();
			
			//Holen Suchkriterium
			String crit = dsView.getString("crit");
			
			//Kriterium validieren
			validateSearchCrit(crit);
			
			ArrayList<String> result = null;
			
			//Zeitraumpruefung gewuenscht
			if(isTimeSpanActiv)
			{
				String timeSpanBegin = dsView.getString("timeSpanBegin");
				String timeSpanEnd = dsView.getString("timeSpanEnd");
				
				//Zeitraum pruefen
				validateTimeSpan(timeSpanBegin, timeSpanEnd);
				
				//Suche ausfuehren
				result = new DiaryEventFileManager().doSearch(getView().getSelectedType(),
						crit, parent.getUser(), true, timeSpanBegin, timeSpanEnd);
			}
			else
			{
				//Suche ausfuehren
				result = new DiaryEventFileManager().doSearch(getView().getSelectedType(),
						crit, parent.getUser());
			}
			
			
			//String zur Speicherung der Suchergebnise
			String resultStr = IRessources.EMPTY;
			
			//Es wurde nichts gefunden -> entsprechenden Text in result String schreiben
			if(result == null||result.size()==0)
			{
				resultStr = IRessources.res.getString("0209");
			}
			//Es wurde min. 1 Eintrag gefunden
			else
			{
				//Schleife fuer jeden gefunden Eintrag. Fuegt den Eintrag dem resultString hinzu
				for (String str : result) 
				{
					resultStr += str + "\n";
				}
			}
			
			//resultString ins ViewDS setzen, View + Components updaten
			dsView.setValueOfRow("result", resultStr);
			getView().updateView();
			getView().updateComponents("search.finished");
		} 
		catch (Exception ex) 
		{
			handleException(ex.getMessage(), getClass().getName(), "doSearch", ex);
		}
	}
	
	/**
	 * Ueberprueft, ob es sich um valide Datumsangaben der Zeitspanne handelt.
	 * 
	 * @param timeSpanBegin
	 * @param timeSpanEnd
	 * @throws Exception
	 */
	private void validateTimeSpan(String timeSpanBegin, String timeSpanEnd) throws Exception
	{
		//Anfangsdatum darf nicht leer sein
		if(timeSpanBegin == null||timeSpanBegin.isEmpty())
		{
			throw new Exception(IRessources.res.getString("0221"));
		}
		
		//Enddatum darf nicht leer sein
		if(timeSpanEnd == null||timeSpanEnd.isEmpty())
		{
			throw new Exception(IRessources.res.getString("0222"));
		}
		
		SimpleDateFormat sdf = new SimpleDateFormat(IRessources.DATE_FORMAT);
		Date from = null;
		Date to = null;
		
		//Versuchen Anfangsdatum zu parsen
		try
		{
			from = sdf.parse(timeSpanBegin);
		}
		catch(ParseException ex)
		{
			//Anfangsdatum ist ungueltig
			Exception ex1 = new Exception(IRessources.res.getString("0223"), ex);
			throw ex1;
		}
		
		//Versuchen Enddatum zu parsen
		try
		{
			to = sdf.parse(timeSpanEnd);
		}
		catch(ParseException ex)
		{
			//Enddatum ist ungueltig
			Exception ex1 = new Exception(IRessources.res.getString("0224"), ex);
			throw ex1;
		}
		
		//Anfangsdatum darf nicht nach dem Enddatum sein
		if(from.after(to))
		{
			throw new Exception(IRessources.res.getString("0225"));
		}	
	}

	/**
	 * Prueft ein uebergebenes Suchkriterium auf Gueltigkeit.
	 * 
	 * @param crit
	 * @throws Exception
	 */
	private void validateSearchCrit(String crit) throws Exception
	{
		//Das Suchkriterium darf nicht leer sein.
		if(crit==null||crit.isEmpty())
		{
			throw new Exception(IRessources.res.getString("0210"));
		}
		
		//Das Suchkriterium muss mindestens drei Stellen lang sein.
		if(crit.length()<3)
		{
			throw new Exception(IRessources.res.getString("0211"));
		}
		
		//Type = 0 -> Stichwortsuche
		if(getView().getSelectedType()==0)
		{
			//Das Suchkriterium/Stichwort darf nur ein Wort sein.
			if(crit.contains(" "))
			{
				throw new Exception(IRessources.res.getString("0212"));
			}
		}
		
		//Das Suchkriterium darf nicht l\u00E4nger als 150 Stellen sein.
		if(crit.length()>150)
		{
			throw new Exception(IRessources.res.getString("0213"));
		}
	}
}