package de.joerg.diary.frontend.general;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import javax.swing.JOptionPane;
import de.joerg.diary.frontend.sdBasicComponents.ViewController;
import de.joerg.diary.shared.IRessources;
import de.joerg.diary.shared.dataSet.DataSet;
import de.joerg.diary.shared.models.User;

/*
 *  SimpleDiary Version 2.0.0
    Copyright (C) 2010  Andreas Joerg (E-Mail: simpleDiary@freenet.de)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Beschreibt den ViewController eines EntsperrDialogs (Userauth).
 * 
 * @author Andreas Joerg
 */
public class UnlockDialogVC extends ViewController implements ActionListener, WindowListener 
{
	private MainWindowVC mainCtrl = null;
	private UnlockDialogV view = null;
	
	/**
	 * Konstruktor mit Uebergabe des aufrufenden Controllers.
	 * 
	 * @param mainCtrl
	 */
	public UnlockDialogVC(MainWindowVC mainCtrl)
	{
		setMainCtrl(mainCtrl);
		setView(new UnlockDialogV(this));
	}

	//------------------GETTER / SETTER --------------------
	public void setMainCtrl(MainWindowVC mainCtrl) 
	{
		this.mainCtrl = mainCtrl;
	}
	public MainWindowVC getMainCtrl() 
	{
		return mainCtrl;
	}
	
	public void setView(UnlockDialogV view)
	{
		this.view = view;
	}
	public UnlockDialogV getView() 
	{
		return view;
	}
	//-------------------------------------------------------

	@Override
	public void actionPerformed(ActionEvent evt) 
	{
		//OK
		if(evt.getActionCommand().equals("0025"))
		{
			try 
			{
				DataSet dataSet = view.getDataSet();
				
				//Validierung, ob username eingegeben wurde
				if( ( (String)dataSet.getRowValueByName("userName") ).isEmpty())
				{
					throw new Exception(IRessources.res.getString("0012"));
				}
				//Validierung, ob Passwort eingegeben wurde
				if( ( (String)dataSet.getRowValueByName("userPw") ).isEmpty())
				{
					throw new Exception(IRessources.res.getString("0013"));
				}
				
				//Bauen eines neuen Users aus den Daten im DataSet
				User user = new User();
				user.setUserName((String)dataSet.getRowValueByName("userName"));
				user.setUserPW( String.valueOf( ( (String)dataSet.getRowValueByName("userPw") ).hashCode() ) );
				
				//Wenn der temporaere User s.o. dem akutell angemeldeten User der Anwendung entspricht
				// -> Anwendung entsperren.
				if(mainCtrl.getUser().equals(user))
				{
					view.dispose();
					JOptionPane.showMessageDialog(view, IRessources.res.getString("0117"),
							IRessources.res.getString("0003"),1);
					mainCtrl.doUnlock();
				}
				else
				{
					throw new Exception(IRessources.res.getString("0118"));
				}
			} 
			catch (Exception ex)
			{
				handleException(ex.getMessage(), getClass().getName(), "actionPerformed", ex);
			}
		}	
	}

	@Override
	public void windowActivated(WindowEvent arg0) 
	{
	}
	@Override
	public void windowClosed(WindowEvent arg0) 
	{
	}

	@Override
	public void windowClosing(WindowEvent arg0) 
	{
		//Setzt den Button "Entsperren" wieder aktiv, da der Dialog geschlossen wird und man die 
		//Moeglichkeit haben muss, ihn wieder oeffnen zu koennen.
		mainCtrl.getView().setUnlockButtonEnabled(true);
	}

	@Override
	public void windowDeactivated(WindowEvent arg0) 
	{
	}
	@Override
	public void windowDeiconified(WindowEvent arg0)
	{
	}
	@Override
	public void windowIconified(WindowEvent arg0) 
	{
	}
	@Override
	public void windowOpened(WindowEvent arg0)
	{
	}
}