package de.joerg.diary.frontend.sdCalendar;

/*
 *  SimpleDiary Version 2.0.0
    Copyright (C) 2010  Andreas Joerg (E-Mail: simpleDiary@freenet.de)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridLayout;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.GregorianCalendar;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.EtchedBorder;

import de.joerg.diary.shared.IRessources;
 
/**
 * Beschreibt ein Panel, auf dem ein Kalender liegt.
 * 
 * @author Andreas Joerg
 */
public class SDCalendarPanel extends JPanel
{
	private static final long serialVersionUID = -8350237618328645891L;
	private SDLabel labels[][] = new SDLabel[7][7];
	private JLabel lblMonthYear = null;
    private JPanel daysPnl = null;
    private SDCalendar controller = null;
    
	public SDCalendarPanel(SDCalendar controller)
	{
		setController(controller);
    	init();
	}

    //-------------GETTER / SETTER----------------------
    public void setController(SDCalendar controller) 
    {
		this.controller = controller;
	}
	public SDCalendar getController() 
	{
		return controller;
	}
	
	public JPanel getDaysPnl() 
	{
		return daysPnl;
	}
	public void setDaysPnl(JPanel daysPnl)
	{
		this.daysPnl = daysPnl;
	}
	//--------------------------------------------------
	
	/**
	 * Initialisiert das Panel des Kalenders.
	 */
	private void init()
    {        
		this.setLayout(new BorderLayout());
        
    	//----------------------------------------------------------------------------  
    	//	Beschreibt das Panel im oberen Teil mit den Labels "<<" "<" ">" ">>" um
    	//	ein Monat/Jahr vor/zurueck zu schalten. Desweiteren wird der Monat und das
    	//	Jahr angezeigt.
    	//----------------------------------------------------------------------------  
        JPanel topPnl = new JPanel();
        topPnl.setBorder(new SDUnderlinedEtchedBorder());
        
        lblMonthYear = new JLabel();
        
        JLabel lblYearBack = new JLabel("<<");
        lblYearBack.setToolTipText(IRessources.resCal.getString("001"));
        lblYearBack.addMouseListener(controller);

        JLabel lblMonthBack = new JLabel("<");
        lblMonthBack.setToolTipText(IRessources.resCal.getString("002"));
        lblMonthBack.addMouseListener(controller);
        
        JLabel lblMonthAfter = new JLabel(">");
        lblMonthAfter.setToolTipText(IRessources.resCal.getString("003"));
        lblMonthAfter.addMouseListener(controller);
        
        JLabel lblYearAfter = new JLabel(">>");
        lblYearAfter.setToolTipText(IRessources.resCal.getString("004"));
        lblYearAfter.addMouseListener(controller);
        
        topPnl.add(lblYearBack);
        topPnl.add(new JLabel(" "));
        topPnl.add(lblMonthBack);
        topPnl.add(new JLabel(IRessources.EMPTY));
        topPnl.add(lblMonthYear);
        topPnl.add(new JLabel(IRessources.EMPTY));
        topPnl.add(lblMonthAfter);
        topPnl.add(new JLabel(" "));
        topPnl.add(lblYearAfter);
        
        this.add(topPnl, BorderLayout.NORTH);
        //----------------------------------------------------------------------------        
        
        
        //------------------------------------------------------------------------------    
        //	Beschreibt das Panel, dass fuer die eigentliche Anzeige des Kalenders
        //	verantwortlich ist. Fuegt 7x7 Labels hinzu (maximale Hoehe x Breite
        //	darzustellender Datume). Die Labels in der ersten Reihe werden mit den
        //	entsprechenden Wochentagsbezeichnern initialisiert. Die restliche ohne Text.
        //------------------------------------------------------------------------------        
        daysPnl = new JPanel();
        daysPnl.setLayout(new GridLayout(7, 7));
        
        labels[0][0] = new SDLabel("MO");
        labels[0][1] = new SDLabel("DI");
        labels[0][2] = new SDLabel("MI");
        labels[0][3] = new SDLabel("DO");
        labels[0][4] = new SDLabel("FR");
        labels[0][5] = new SDLabel("SA");
        labels[0][6] = new SDLabel("SO");
        
        for(int x=0;x<7;x++)
        {
                for(int y=0;y<7;y++)
                {
                       	if(x!=0)
                       	{
                            labels[x][y] = new SDLabel();
                        }
                       	else
                       	{
                       		labels[x][y].setBorder(new EtchedBorder());
                       	}
                             
                        labels[x][y].setHorizontalAlignment(JLabel.CENTER);
                        daysPnl.add(labels[x][y]);
                }
        }
        //------------------------------------------------------------------------------        
        
        this.add(daysPnl, BorderLayout.CENTER);
        
        this.setOpaque(false);    
        getDaysPnl().setOpaque(false);
    }
    
	/**
	 * Setzt den Inhalt der entsprechenden Labels fuer einen uebergebenen Monat und Jahr.
	 * zB. Monat Januar 2010 -> Label "1" -> erster Freitag, bis "31"
	 * 
	 * @param month
	 * @param year
	 * @throws ParseException
	 */
    public void doSetControlsForMonth(Integer month, Integer year) throws ParseException
    {
    	final String[] MONTHS = new String[]
    	{
    		"Januar", "Februar", "M\u00E4rz", "April", "Mai", "Juni", "Juli", "August", "September", "Oktober",
    		"November", "Dezember"
    	};
    	
    	//setzt entsprechend des uebergebenen Monats, den Inhalt des Labels lblMonthYear
    	lblMonthYear.setText(MONTHS[month-1]);
    	
    	
    	//Fuegt die Jahresangabe zum Label lblMonthYear hinzu
    	lblMonthYear.setText(lblMonthYear.getText()+", "+String.valueOf(year));
    	
    	//GregorianCalendar erzeugen mit Datums init - erster Tag des uebergebenen Monats
    	SimpleDateFormat sdf = new SimpleDateFormat(IRessources.DATE_FORMAT);
        Date date = sdf.parse("01."+month+"."+year);
        GregorianCalendar cal = new GregorianCalendar();
        cal.setTime(date);
        
        //Speicher Anzahl Tage in Monat
        int maxDays = cal.getActualMaximum(GregorianCalendar.DAY_OF_MONTH);
        int start = -1;
        int x = 1;
        int count = 1;
        
        //Wenn der erste im Monat ein Sonntag ist -> Setzen von start auf 6, da Sonntag im
        //GregorianCalendar eigentlich der erste Tag ist.
        if(cal.get(GregorianCalendar.DAY_OF_WEEK)==GregorianCalendar.SUNDAY)
        {
        		start = 6;
        }
        //Sonst setzen von start auf DAY_OF_WEEK-2 ( Montag zB ist 2, -2 = 0!)
        else
        {
        	start = cal.get(GregorianCalendar.DAY_OF_WEEK)-2;
        }
        
        //Schleife fuer alle sieben Reihen, solange count nicht groesser als die Anzahl der Tage des
        //Monats ist.
        while(x<7&&count<=maxDays)
        {
        	//Schleife fuer alle sieben Spalten der jeweiligen Reihe. Beginnend mit dem ermittelten
        	//Startpunkt.
        	while(start<7&&count<=maxDays)
        	{
        		//Setzt den Text der Variable count (zaehlt die einzelnen Tage des Monats hoch)
        		labels[x][start].setText(String.valueOf(count));
        		labels[x][start].setBorder(new EtchedBorder());
        		labels[x][start].setOpaque(true);       
        		
        		start++;
        		count++;
        	}
        	//Setzen von start auf 0, da start nur einmalig in der ersten Reihe (aufgrund des ersten Tages)
        	//im Monat einen besonderen Wert hat und nicht von Anfang an der Spalten geht (auser erster Tag
        	//ist ein Montag).
        	start = 0;
        	x++;
        }
    }
    
    /**
     * Setzt den Text der Labels in der ersten, vorletzten und letzten Reihe auf "".
     */
    public void doCleanUp()
    {
    	for(int x=1;x<7;x++ )
    	{
    		if(x==1||x>=5)
    		{
    			for(int y=0;y<7;y++)
            	{
            		labels[x][y].setText(IRessources.EMPTY);
            		labels[x][y].setBorder(null);
            		labels[x][y].setOpaque(false);
            	}
    		}
    	}
    }
    
    /**
     * Aktualisiert die Eintraege auf dem Panel.
     * 
     * @param entrysForMonth
     * @param date
     */
    public void doUpdate(ArrayList<String> entrysForMonth, String date)
    {
    	//Schleife fuer die Reihen des Kalenders
    	for(int x=1;x<7;x++)
		{
    		//Schleife fuer die Spalten
			for(int y=0;y<7;y++)
			{
				//Wenn aktuelles Label einen Wert hat und Eintraege vorhanden sind
				if(!labels[x][y].getText().equals(IRessources.EMPTY)&&entrysForMonth.size()!=0)
				{
					for (String entry : entrysForMonth) 
					{
						String compare = null;
						
						//Datume von 1.xx.xxxx bis 9.xx.xxxx -> 0 wird vorne angehaeng da sonst
						//beim 5. zB das Label fuer die Tage 15+25 mit geaendert wird.
						if(Integer.valueOf(labels[x][y].getText())<10)
						{
							compare = "0"+labels[x][y].getText()+date;
						}
						else
						{
							compare = labels[x][y].getText()+date;
						}
						
						//Wenn zu aktuellem Datum ein Eintrag vorhanden ist.
						if( entry.contains(compare))
						{
							//Wenn an diesem Tag mehrere Eintraege vorhanden sind
							if(entry.charAt(0)=='!')
							{
								labels[x][y].setBackground(Color.orange);
								labels[x][y].setMulti(true);	
							}
							//Sonst -> nur ein Eintrag
							else
							{
								labels[x][y].setBackground(Color.green);
								labels[x][y].setMulti(false);
							}
							
							//Fuegt den CalendarController als MouseListener hinzu, falls nicht schon
							//geschehen. Es kann sein, dass
							if(labels[x][y].getMouseListeners().length<1)
							{
								//AddListener um MouseClicks abzufangen
								labels[x][y].addMouseListener(getController());
							}
							
							//Entfernt den Eintrag da er nun bereits gekennzeichnet wurde und sonst weiter
							//nach ihm gesucht werden wuerde.
							entrysForMonth.remove(0);
							
							//beendet die foreach Schleife, da sonst auf jeden weiteren Eintrag geprueft
							//werden wuerde aber der entsprechende ja bereits gefunden wurde.
							break;
						}
						//Kein Eintrag zu diesem Datum
						else
						{
							labels[x][y].removeMouseListener(getController());
							labels[x][y].setBackground(labels[0][1].getBackground());
						}
					}
					labels[x][y].setOpaque(true);
				}
				//Entweder label hat keinen Wert ("") oder es sind keine Eintraege vorhanden.
				else
				{
					labels[x][y].removeMouseListener(getController());
					//Setzt Hintergrundfarbe von spalte [0,1] -> Standardfarbe
					labels[x][y].setBackground(labels[0][1].getBackground());
					
					//Label durchsichtig setzen, da es keinen Inhalt hat
					if(labels[x][y].getText().equals(IRessources.EMPTY))
					{
						labels[x][y].setOpaque(false);
					}
					else
					{
						labels[x][y].setOpaque(true);
					}
				}
			}
		}
    	repaint();
    }
}