package de.joerg.diary.frontend.sdCalendar;

import java.awt.BorderLayout;
import java.awt.Dimension;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.border.EtchedBorder;
import de.joerg.diary.frontend.sdBasicComponents.BackGroundPane;
import de.joerg.diary.frontend.sdBasicComponents.EmptyPanel;
import de.joerg.diary.frontend.sdBasicComponents.View;
import de.joerg.diary.shared.IRessources;
import de.joerg.diary.shared.dataSet.DataSet;

/*
 *  SimpleDiary Version 2.0.0
    Copyright (C) 2010  Andreas Joerg (E-Mail: simpleDiary@freenet.de)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Beschreibt den View zur Auswahl eines Eintrags aus mehreren am selben Tag.
 * 
 * @author Andreas Joerg
 */
public class SDCalendarSelectSubEntryV extends View
{
	private static final long serialVersionUID = -5962699674313433592L;
	private SDCalendarSelectSubEntryVC controller = null;
	private final static String[] ROW_NAMES = {IRessources.resCal.getString("005")};
	private JTable table = null;
	private JScrollPane scrollPane = null;
	private JButton btnSelect = null;
	
	/**
	 * Konstruktor mit Uebergabe des aufrufenden Controllers
	 * 
	 * @param controller
	 */
	public SDCalendarSelectSubEntryV(SDCalendarSelectSubEntryVC controller)
	{
		setController(controller);
		init();
	}

	//------------- GETTER / SETTER-------------------------------------
	public void setController(SDCalendarSelectSubEntryVC controller) 
	{
		this.controller = controller;
	}
	public SDCalendarSelectSubEntryVC getController()
	{
		return controller;
	}
	//-------------------------------------------------------------------
	
	/**
	 * Initialisiert den View
	 */
	private void init()
	{
		this.setPreferredSize(new Dimension(300,300));
		this.setTitle(IRessources.resCal.getString("006"));
		this.setContentPane(new BackGroundPane(IRessources.BG_IMAGE));
		this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		this.setLocation(100, 100);
		this.setLayout(new BorderLayout());
		this.addWindowListener(controller);
		
		btnSelect = new JButton(IRessources.resCal.getString("007"));
		btnSelect.setEnabled(false);
		btnSelect.setActionCommand("0028");
		btnSelect.addActionListener(getController());
		
		EmptyPanel bottomPnl = new EmptyPanel();
		bottomPnl.add(btnSelect);
		
		this.add(bottomPnl, BorderLayout.SOUTH);
		
		this.pack();
		this.setVisible(true);
	}
	
	/**
	 * Setzt das DataSet des Views
	 * 
	 * @param dataSet
	 */
	public void setDataSet(DataSet dataSet)
	{
		int rowsCount = dataSet.getRowsCount();
		String[][] tableData = new String[rowsCount][1];
		
		//Schreibt die Eintraege des DataSets in das Array tableData
		for(int x=0;x<rowsCount;x++)
		{
			tableData[x][0] = (String) dataSet.getRowValueByIndex(x);
		}
		
		//Wenn scrollPane nicht null ist -> ScrollPane ist nur beim ersten Aufruf null
		if(scrollPane!=null)
		{
			this.remove(scrollPane);
			scrollPane.remove(table);
		}
		
		//Init der table, schreibgeschuetzt setzen.
		table = new JTable(tableData,ROW_NAMES)
		{
			private static final long serialVersionUID = 1L;

			@Override
			public boolean isCellEditable(int row, int column) 
			{
				return false;
			}
		};
		//Tabellenspalten duerfen nicht verschoben werden.
		table.getTableHeader().setReorderingAllowed(false);
		
		//Init ScrollPane
		scrollPane = new JScrollPane(table);
		scrollPane.getViewport().setOpaque(false);
		scrollPane.setOpaque(false);
		scrollPane.setBorder(new EtchedBorder());
		
		//Controller als ListSelection Listener adden
		table.getSelectionModel().setSelectionMode(ListSelectionModel. SINGLE_SELECTION); 
		table.getSelectionModel().addListSelectionListener(getController());
		
		this.add(scrollPane, BorderLayout.CENTER);
	}
	
	/**
	 * Setz Button "Auswaehlen" aktiv bzw. inaktiv
	 * 
	 * @param enabled
	 */
	public void setButtonSelectEnabled(boolean enabled)
	{
		btnSelect.setEnabled(true);
	}
}