package de.joerg.diary.frontend.sdCalendar;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import de.joerg.diary.frontend.general.MainWindowVC;
import de.joerg.diary.frontend.sdBasicComponents.ViewController;
import de.joerg.diary.shared.dataSet.DataSet;
import de.joerg.diary.shared.dataSet.DataSetRowAlreadyExistsException;

/*
 *  SimpleDiary Version 2.0.0
    Copyright (C) 2010  Andreas Joerg (E-Mail: simpleDiary@freenet.de)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Beschreibt den ViewController eines Views zur Auswahl eines Eintrags aus mehreren am selben Tag.
 * 
 * @author Andreas Joerg
 */
public class SDCalendarSelectSubEntryVC extends ViewController implements ListSelectionListener, 
ActionListener, WindowListener
{
	private SDCalendarSelectSubEntryV view = null;
	private int selectedEntry = -1;
	private MainWindowVC parent = null;
	private String date = null;
	
	/**
	 * Konstruktor mit Uebergabe date, entrysCount und parent.
	 * date = Datum im Format dd.MM.yyyy
	 * entrysCount = Anzahl Eintraege
	 * parent = Aufrufender ViewController
	 * 
	 * @param date
	 * @param entrysCount
	 * @param parent
	 */
	public SDCalendarSelectSubEntryVC(String date, int entrysCount, MainWindowVC parent)
	{
		setParent(parent);
		
		//View init
		setView(new SDCalendarSelectSubEntryV(this));
		
		//setzt im Hauptfenster Flag, dass ein Auswahlfenster offen ist
		parent.setCalendarSelectionDialogOpenend(true);
		
		//Setzt das Datum des Eintrags
		setDate(date);
		
		DataSet dataSet = new DataSet();
		
		//Erstellt fuer jeden Eintrag ein Field im DataSet
		for(int x=1;x<=entrysCount;x++)
		{
			try 
			{
				dataSet.addRow("E_"+x, date+" - "+x);
			} 
			catch (DataSetRowAlreadyExistsException ex)
			{
				handleException(ex.getMessage(), getClass().toString(), "constructor", ex);
			}
		}
		
		view.setDataSet(dataSet);
	}

	//---------------- GETTER/SETTER ------------------------
	public void setView(SDCalendarSelectSubEntryV view) 
	{
		this.view = view;
	}
	public SDCalendarSelectSubEntryV getView() 
	{
		return view;
	}

	public void setParent(MainWindowVC parent)
	{
		this.parent = parent;
	}
	public MainWindowVC getParent() 
	{
		return parent;
	}
	
	public void setDate(String date) 
	{
		this.date = date;
	}
	public String getDate() 
	{
		return date;
	}
	//--------------------------------------------------------

	@Override
	/**
	 * Reagiert auf Selection Aenderungen der Auswahlliste des Views
	 */
	public void valueChanged(ListSelectionEvent evt) 
	{
		selectedEntry = evt.getLastIndex();
		
		//Der Button "Auswaehlen ist beim init des Views inaktiv, da es sonst zu einer
		//NullPointerException kommt, wenn kein Eintrag selected ist.
		view.setButtonSelectEnabled(true);
	}

	@Override
	public void actionPerformed(ActionEvent evt) 
	{
		//Button "Auswaehlen"
		if(evt.getActionCommand().equals("0028"))
		{
			view.dispose();
			parent.doOpenSubEventFromCalendar(selectedEntry, date);
		}
	}

	@Override
	public void windowActivated(WindowEvent arg0) 
	{
	}

	@Override
	public void windowClosed(WindowEvent arg0) 
	{
		//Macht es im Hauptfenster wieder moeglich einen Auswahldialog zu oeffnen.
		parent.setCalendarSelectionDialogOpenend(false);
	}

	@Override
	public void windowClosing(WindowEvent arg0) 
	{
	}

	@Override
	public void windowDeactivated(WindowEvent arg0) 
	{
	}

	@Override
	public void windowDeiconified(WindowEvent arg0) 
	{
	}

	@Override
	public void windowIconified(WindowEvent arg0) 
	{
	}

	@Override
	public void windowOpened(WindowEvent arg0) 
	{
	}
}
