package de.joerg.diary.frontend.sdUndoManager;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import javax.swing.JComponent;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import de.joerg.diary.backend.fileManagement.DiaryEventFileManager;
import de.joerg.diary.frontend.general.MainWindowVC;
import de.joerg.diary.shared.IRessources;
import de.joerg.diary.shared.SDSharedServices;
import de.joerg.diary.shared.dataSet.DataSet;
import de.joerg.diary.shared.models.DiaryEvent;

/*
 *  SimpleDiary Version 2.0.0
    Copyright (C) 2010  Andreas Joerg (E-Mail: simpleDiary@freenet.de)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Singleton Klasse die Aktionen verwaltet, die rueckgaengig gemacht werden koennen 
 * und diese ggf. wieder zurueck setzt.
 * Speichert maximal 15 Aktionen.
 * 
 * @author Andreas Joerg
 */
public class SDUndoManager implements DocumentListener
{
	//Singleton instance
	private static SDUndoManager instance = null;
	
	//interne Liste fuer Aktionen die rueckgaengig gemacht werden koennen
	private ArrayList<Object> actions = new ArrayList<Object>();
	
	//ViewController, dessen Views Komponenten im UndoManager registriert sind
	private MainWindowVC controller = null;
	
	//interne Liste der registrierten (Text)Komponenten
	private ArrayList<SDUndoAbleComponent> components = new ArrayList<SDUndoAbleComponent>();
	
	//Flag, dass benoetigt wird, um festzustellen ob eine Aenderung eines Textelements von der Methode,
	//welche eine Aktion rueckgaengig macht ausgeloest wurde
	private boolean isChangeHandledByMe = false;
	
	//Flag, dass benoetigt wird, um festzustellen ob eine Aenderung eines Textelemnts von der Methode,
	//welche Loeschaktionen einer TextKomponente registriert ausgeloest wurde.
	private boolean isChangeHandledByRemove = false;
	
	//Flag, dass benoetigt wird, falls trotz Aenderung einer TextKomponente keine Aktion erstellt werden
	//soll. (zB bei Update des Views = neu setzen Inhalt TextFelder mit altem Inhalt)
	private boolean ignoreChanges = false;
	
	/**
	 * Singleton constructor
	 */
	private SDUndoManager()
	{
		
	}
	
	//-------Getter/Setter------------------
	/**
	 * Liefert die Liste der Aktionen, die rueckgaengig gemacht werden koennen.
	 */
	public ArrayList<Object> getActions()
	{
		return actions;
	}
	
	/**
	 * Setzt den ViewController.
	 * 
	 * @param controller
	 */
	public void setController(MainWindowVC controller)
	{
		this.controller = controller;
	}
	
	/**
	 * Setzt ob fuer eine Aenderung einer registrierten TextKomponente eine rueckgaengig machende Aktion
	 * gespeichert werden soll oder nicht.
	 * 
	 * @param ignoreChanges
	 */
	public void setIgnoreChanges(boolean ignoreChanges)
	{
		this.ignoreChanges = ignoreChanges;
	}
	//--------------------------------------
	
	/**
	 * Liefert Singleton Instanz der Klasse
	 */
	public static SDUndoManager getInstance()
	{
		if(instance==null)
		{
			instance = new SDUndoManager();
		}
		return instance;
	}
	
	/**
	 * Fuegt eine Aktion hinzu, die rueckgaengig gemacht werden kann.
	 * 
	 * @param action
	 */
	public void addAction(Object action)
	{
		//Es koennen nicht mehr als 15 Aktionen gespeichert werden.
		if(actions.size()==15)
		{
			actions.remove(0);
		}
		actions.add(action);
		controller.getView().setUndoActive(true);
	}
	
	/**
	 * Macht die letzte gespeicherte Aktion rueckgaengig.
	 * 
	 * @throws Exception
	 */
	public void makeLastActionUndo() throws Exception
	{
		//letzte Aktion
		Object lastAction = actions.get(actions.size()-1);
		
		DiaryEventFileManager evtFileMgr = new DiaryEventFileManager();
		
		//Wenn zuletzt ein Eintrag geaendert wurde
		if(lastAction instanceof UndoActionForChange)
		{
			UndoActionForChange last = (UndoActionForChange) lastAction;
			
			//Aendert das DiaryEvent und setzt das Model des Controllers entsprechend neu.
			evtFileMgr.changeDiaryEvent(last.getNewEvt(), last.getOldEvt(), 
					controller.getUser(), controller.getSavePath());
			
			//Model neu setzen
			controller.setModel(evtFileMgr.loadDiaryEventFromFile(controller.getSavePath(), 
					SDSharedServices.getFileName(controller.getUser().getUserID(), 
							last.getOldEvt().getDateOfEvent()),
					controller.getUser()));
			
			//Zuruecksetzen des DataSets des Views
			DataSet dataSet = new DataSet();
			
			//TODO nix clear s. Mantis98
				//TODO was is eigentlich wenn Datumsaenderung in undo und schon neuer Eintrag b
			
			dataSet.addRow("date", IRessources.EMPTY);
			dataSet.addRow("place", IRessources.EMPTY);
			dataSet.addRow("timeStart", IRessources.EMPTY);
			dataSet.addRow("timeEnd", IRessources.EMPTY);
			dataSet.addRow("text", IRessources.EMPTY);
	
			controller.getView().setDataSet(dataSet, false);
			controller.getView().setControlsAtApplicationStart();
			
			//Bei Datumsaenderung, Kalendar Auswahldialog handling (MANTIS 69 Notiz 184)
			if(last.getOldEvt().getDateOfEvent().equals(last.getNewEvt().getDateOfEvent()) == false)
			{
				//Wenn Kalendardialog angezeigt wird.
				if(controller.isCalendarSelectionDialogOpenend())
				{
					SimpleDateFormat sdf = new SimpleDateFormat(IRessources.DATE_FORMAT);
					String date = sdf.format(last.getNewEvt().getDateOfEvent());
					
					//Wenn der Auswahldialog zum neuen Eintrag geoffnet ist -> beenden
					if(controller.getCalSelectSubEntryCtrl().getDate().equals(date))
					{
						controller.doCloseCalenderSelectSubEntryDialog();
					}
				}
			}
			
			//View muss abgedatet werden, da es sonst passieren kann zB bei Datumsaenderungen eines Untereintrags
			//dass die Eintraege nicht aktuell sind.
			controller.updateView();
		}
		//Wenn letzte Aktion eine Aenderung einer TextKomponente war
		else if(lastAction instanceof UndoActionForTextChange)
		{
			UndoActionForTextChange last = (UndoActionForTextChange) lastAction;
			//Setzt Flag, dass Aenderung durch Rueckgaengig-Aktion verursacht wurde und somit keine
			//weiteren Aktionen erstellt werden.
			isChangeHandledByMe = true;

			//Setzt den Inhalt der entsprechenden TextKomponente zurueck
			for (SDUndoAbleComponent comp : components)
			{				
				if(last.getComponent().equals(comp.getComponentName()))
				{
					if(comp.getComponent() instanceof JTextField)
					{
						((JTextField)comp.getComponent()).setText(last.getOldText());
					}
					else if(comp.getComponent() instanceof JTextArea)
					{
						( (JTextArea) comp.getComponent() ).setText(last.getOldText());
					}
					
					//Entfernt die rueckgaengig gemacht Aktion
					actions.remove(actions.size()-1);
					
					//Zuruecksetzen der beiden Flags, da nun keine Aenderungen ueber die Listener
					//mehr durchgefuehrt werden koennen und es sonst zu Exceptions kommen kann
					//(s. Mantis 69 Notiz 109)
					isChangeHandledByMe = false;
					isChangeHandledByRemove = false;
					
					//Wenn keine rueckgaengig machbare Aktionen mehr vorhanden sind -> Setze Schaltflaeche
					//"Rueckgaengig" inaktiv.
					if(actions.size()==0)
					{
						controller.getView().setUndoActive(false);
					}
					
					return;
				}
			}
		}
		//Wenn letzte Aktion eine Loeschung war
		else if(lastAction instanceof UndoActionForDelete)
		{
			UndoActionForDelete last = (UndoActionForDelete) lastAction;
			
			DiaryEvent delEvent = last.getDeletedEvt();
			
			//Wiederherstellen des geloeschten Eintrages
			evtFileMgr.saveDiaryEvent(delEvent, controller.getUser(),controller.getSavePath());
			
			//Wenn geloeschter Eintrag Untereintraege hatte -> Wieder KZ als MultiEintrag setzen
			if(delEvent.getSubEvents()!=null&&delEvent.getSubEvents().size()!=0)
			{
				//Datei als MultiEintrag kz
				evtFileMgr.markFileAsParent(controller.getSavePath(), new File(
						SDSharedServices.getFileName(controller.getUser().getUserID(),
								delEvent.getDateOfEvent())));
			}
			
			controller.updateView();
		}
		//Wenn letzte Aenderung eine Speicherung war
		else if(lastAction instanceof UndoActionForSingleEntrySave||
				lastAction instanceof UndoActionForMultiEntrySave)
		{
			//Untereintrag wurde gespeichert
			if(lastAction instanceof UndoActionForMultiEntrySave)
			{
				UndoActionForMultiEntrySave last = (UndoActionForMultiEntrySave)lastAction;
				
				//Holen des gespeicherten Eintrags
				DiaryEvent evtToDel = last.getAddedEntry();
				
				//Loeschen des Eintrags
				new DiaryEventFileManager().deleteDiaryEvent(
						SDSharedServices.getFileName(evtToDel.getUser().getUserID(), evtToDel.getDateOfEvent()),
						evtToDel, controller.getUser(), controller.getSavePath());
				
				//Wenn Kalendarauswahl Dialog offen ist
				if(controller.isCalendarSelectionDialogOpenend())
				{
					String date1 = controller.getCalSelectSubEntryCtrl().getDate();
					String date2 = new SimpleDateFormat(IRessources.DATE_FORMAT).format(
							evtToDel.getDateOfEvent());
					
					if(date1.equals(date2))
					{
						controller.doCloseCalenderSelectSubEntryDialog();
					}
				}
			}
			//Einzelner Eintrag wurde gespeichert
			else
			{
				//Loeschen des gespeicherten Files
				new DiaryEventFileManager().deleteDiaryEventFileFromFS(
						((UndoActionForSingleEntrySave) lastAction).getFile());
			}
			
			//Neusetzen des DataSets des Views (leer)
			DataSet dataSet = new DataSet();
			
			dataSet.addRow("date", IRessources.EMPTY);//TODO nix leeren s. Mantis 98
			dataSet.addRow("place", IRessources.EMPTY);
			dataSet.addRow("timeStart", IRessources.EMPTY);
			dataSet.addRow("timeEnd", IRessources.EMPTY);
			dataSet.addRow("text", IRessources.EMPTY);
			
			controller.getView().setDataSet(dataSet, false);
			
			controller.setModel(null);
			
			controller.updateView();
			controller.getView().setControlsAtApplicationStart();
		}
		//Letze Aenderung war Bild adden
		else if(lastAction instanceof UndoActionForAddPicture)
		{
			controller.doDeletePicture(((UndoActionForAddPicture)lastAction).getImgIndex(), true);
		}
		//Letzte Aenderung war Bild loeschen
		else if(lastAction instanceof UndoActionForDeletePicture)
		{
			controller.addPictureToModelAndView(((UndoActionForDeletePicture)lastAction).getImg());
		}
		//Letzte Aenderung war Bildbeschreibung aendern
		else if(lastAction instanceof UndoActionForChangeImgDesc)
		{
			UndoActionForChangeImgDesc last = (UndoActionForChangeImgDesc)lastAction;
			
			//ParentEintrag
			if(last.getEvtNr() == -1)
			{
				controller.getModel().getPictures().get(last.getImgNr()).setPictureDesc(last.getOldText());
			}
			//Untereintrag
			else
			{
				controller.getModel().getSubEvent(last.getEvtNr()).getPictures().get(last.getImgNr()).
					setPictureDesc(last.getOldText());
			}
			
			//Wenn Dialog "Bildbeschreibung" offen ist -> updaten
			if(controller.getImgDescVC()!=null)
			{
				controller.getImgDescVC().updateView();
			}
		}
		
		//Entfernt die rueckgaengig gemacht Aktion
		actions.remove(actions.size()-1);
		
		//Zuruecksetzen der beiden Flags, da nun keine Aenderungen ueber die Listener
		//mehr durchgefuehrt werden koennen und es sonst zu Exceptions kommen kann
		//(s. Mantis 69 Notiz 109)
		isChangeHandledByMe = false;
		isChangeHandledByRemove = false;
		
		//Wenn keine rueckgaengig machbare Aktionen mehr vorhanden sind -> Setze Schaltflaeche
		//"Rueckgaengig" inaktiv.
		if(actions.size()==0)
		{
			controller.getView().setUndoActive(false);
		}
	}

	/**
	 * Entfernt alle gespeicherten Aktionen fuer Aenderungen an TextKomponenten und an den Bildern des Eintrags.
	 * 
	 * Verwendung: - zBEin anderer Eintrag wurde ausgewaehlt
	 */
	public void removeUndoActionsForEntryChanges() 
	{
		for (int x=0;x<actions.size();x++) 
		{
			if(actions.get(x) instanceof UndoActionForTextChange||
					actions.get(x) instanceof UndoActionForAddPicture||
					actions.get(x) instanceof UndoActionForDeletePicture||
					actions.get(x) instanceof UndoActionForChangeImgDesc)
			{
				actions.remove(actions.get(x));
				x-=1;
			}
		}
		if(actions.size()==0)
		{
			controller.getView().setUndoActive(false);
		}
		
		isChangeHandledByRemove = false;
	}
	
	/**
	 * Registriert eine TextKomponente, um Eingaben etc. auf dieser rueckgaengig machen zu koennen.
	 * 
	 * @param componentName
	 * @param component
	 */
	public void addComponent(String componentName, JComponent component)
	{
		components.add(new SDUndoAbleComponent(componentName, component));
	}
	
	/**
	 * Entfernt eine registrierte Komponente anhand des uebergebenen Bezeichners.
	 * 
	 * @param componentName
	 * @throws SDUndoAbleComponentNotFoundException
	 */
	public void removeComponent(String componentName) throws SDUndoAbleComponentNotFoundException
	{
		for (SDUndoAbleComponent comp : components)
		{
			if(comp.getComponentName().equals(componentName))
			{
				components.remove(comp);
				return;
			}
		}
		throw new SDUndoAbleComponentNotFoundException(componentName);
	}
	
	/**
	 * @return true wenn es zu dem Eintrag undoable Changes gibt (Text- oder Bildaenderungen)
	 */
	public boolean hasChangesOnEntry()
	{
		return hasTextChanges()||hasImgChanges();
	}
	
	/**
	 * @return true wenn undoable BildChanges vorhanden sind, wenn nicht false
	 */
	private boolean hasImgChanges()
	{
		for (Object action : actions)
		{
			if(action instanceof UndoActionForAddPicture||
					action instanceof UndoActionForDeletePicture||
					action instanceof UndoActionForChangeImgDesc)
			{
				return true;
			}
		}
		
		return false;
	}
	
	/**
	 * @return true wenn undoable TextChanges vorhanden sind, wenn nicht false
	 */
	private boolean hasTextChanges()
	{
		for (Object action : actions)
		{
			if(action instanceof UndoActionForTextChange)
			{
				return true;
			}
		}
		
		return false;
	}

	//---------------DOCUMENT LISTENER---------------------------
	@Override
	public void changedUpdate(DocumentEvent evt) 
	{
	}

	@Override
	/**
	 * EventHandler fuer Eingaben in einer TextKomponente.
	 */
	public void insertUpdate(DocumentEvent evt) 
	{
		//Schleife fuer alle registrierten Komponenten
		for (SDUndoAbleComponent comp : components)
		{
			if(comp.getComponent() instanceof JTextField)
			{
				//Wenn das QuellenDocument dem Document der aktuellen Komponente entspricht
				if(((JTextField)comp.getComponent()).getDocument().equals(evt.getDocument()))
				{
					String name = comp.getComponentName();
					
					if(name.equals("tfDate")||name.equals("tfPlace")||name.equals("tfTimeStart")||
							name.equals("tfTimeEnd")||name.equals("text"))
					{
						//Wenn Aenderung nicht durch rueckgaengig zu machende Aktion ausgeloest wurde
						//und Aenderung nicht zu ignorieren ist
						// -> Hinzufuegen einer entsprechenden neuen Aktion
						if(!isChangeHandledByRemove&&!ignoreChanges)
						{
							addAction(new UndoActionForTextChange(comp.getComponentText(), 
									comp.getComponentName()));
						}
						
						//Synchronisieren des tatsaechlichen Textes der Komponente mit der "virtuellen"
						//gespeicherten Komponente.
						comp.setComponentText( ( (JTextField)comp.getComponent() ).getText() );
						isChangeHandledByRemove = false;
						return;
					}
				}
			}
			else if(comp.getComponent() instanceof JTextArea)
			{
				if(((JTextArea)comp.getComponent()).getDocument().equals(evt.getDocument()))
				{
					String name = comp.getComponentName();
					
					if(name.equals("text"))
					{
						if(!isChangeHandledByRemove&&!ignoreChanges)
						{
							addAction(new UndoActionForTextChange(comp.getComponentText(), 
									comp.getComponentName()));
						}
						
						comp.setComponentText( ( (JTextArea)comp.getComponent() ).getText() );
						isChangeHandledByRemove = false;
						return;
					}
				}
			}
		}
	}

	@Override
	/**
	 * EventHandler fuer entfernen von Text einer TextKomponente.
	 */
	public void removeUpdate(DocumentEvent evt)
	{
		for (SDUndoAbleComponent comp : components)
		{
			if(comp.getComponent() instanceof JTextField)
			{
				if(((JTextField)comp.getComponent()).getDocument().equals(evt.getDocument()))
				{
					String name = comp.getComponentName();
					
					if(name.equals("tfDate")||name.equals("tfPlace")||name.equals("tfTimeStart")||
							name.equals("tfTimeEnd")||name.equals("text"))
					{
						if(!isChangeHandledByMe&&!ignoreChanges)
						{
							addAction(new UndoActionForTextChange(comp.getComponentText(), 
								comp.getComponentName()));
						}
						
						comp.setComponentText( ( (JTextField)comp.getComponent() ).getText() );
						
						isChangeHandledByMe = false;
						//Setzt Flag, dass Aenderung durch entfernen eines Zeichens aus dem Text der
						//Komponente ausgeloest wurde.
						isChangeHandledByRemove = true;
						return;
					}
				}
			}
			else if(comp.getComponent() instanceof JTextArea)
			{
				if(((JTextArea)comp.getComponent()).getDocument().equals(evt.getDocument()))
				{
					String name = comp.getComponentName();
					
					if(name.equals("text"))
					{
						if(!isChangeHandledByMe&&!ignoreChanges)
						{
							addAction(new UndoActionForTextChange(comp.getComponentText(), 
								comp.getComponentName()));
						}
						
						comp.setComponentText( ( (JTextArea)comp.getComponent() ).getText() );
						
						isChangeHandledByMe = false;
						isChangeHandledByRemove = true;
						return;
					}
				}
			}
		}
		isChangeHandledByMe = false;
	}
	//-----------------------------------------------------------
}