package de.joerg.diary.frontend.tools;

import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.PrintGraphics;
import java.awt.PrintJob;
import java.io.LineNumberReader;
import java.io.StringReader;
import java.text.SimpleDateFormat;
import java.util.Properties;
import javax.swing.JFrame;
import javax.swing.JTextArea;

import de.joerg.diary.shared.IRessources;
import de.joerg.diary.shared.models.DiaryEvent;

/*
 *  SimpleDiary Version 2.0.0
    Copyright (C) 2010  Andreas Joerg (E-Mail: simpleDiary@freenet.de)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Klasse um einen String oder einen Eintrag zu drucken.
 * 
 * @author Andreas Joerg
 */
public class SDPrinter 
{
	private String textToPrint = null;
	private DiaryEvent dEvt = null; 
									 //123456789012345
	private static final String TAB = "               ";
	
	/**
	 * This method initializes 
	 */
	public SDPrinter(String textToPrint) throws Exception 
	{
		//Text darf nicht null sein
		if(textToPrint==null)
		{
			throw new NullPointerException(IRessources.resPrint.getString("001"));
		}
		
		this.textToPrint = textToPrint;
	}
	
	public SDPrinter(DiaryEvent diaryEvent) throws Exception
	{
		//DiaryEvent darf nicht null sein
		if(diaryEvent==null)
		{
			throw new NullPointerException(IRessources.resPrint.getString("002"));
		}
		
		this.dEvt = diaryEvent;
	}
	
	public String getTextToPrint() 
	{
		return textToPrint;
	}
	
	public DiaryEvent getDiaryEventToPrint()
	{
		return dEvt;
	}

	/**
	 * Druckt den Text im String textToPrint
	 */
	public void doPrint() throws Exception
	{		
		//Wenn kein String zu drucken gesetzt wurde, muss ein Eintrag gesetzt worden sein. Aus dessen
		//Werten wird in der Methode setTextToPrintForDiaryEvent der zu druckende String gebildet
		if(textToPrint==null)
		{
			setTextToPrintForDiaryEvent();
		}
		
		//Pseudo JTextArea
		JTextArea text = new JTextArea();
		text.setText(textToPrint);
		
		//Neuen PrintJob erzeugen aus Toolkit der Pseudo JTextArea mit Uebergabe eines Pseudo Fensters,
		//dem JobTitle und einem neuen Properties Object.
		PrintJob pjob = text.getToolkit().getPrintJob(new JFrame(), IRessources.resPrint.getString("003"),
				new Properties());

		//Wenn ein Drucker ausgewaehlt wurde.
		if (pjob != null) 
		{
			Graphics pg = pjob.getGraphics();

			//Wenn vom PrintJob eine Graphic zurueck geliefert wurde.
			if (pg != null) 
			{
				//Stoesst den Druck an
				print(pjob, pg, text.getText());
				
				//Schliesst die Graphic
				pg.dispose();
			}
			
			//beendet den Druckauftrag
			pjob.end();
		}
	}
	
	/**
	 * Setzt den Text der gedruckt werden soll entsprechend den Angaben im Eintrag.
	 * 
	 * @throws Exception
	 */
	private void setTextToPrintForDiaryEvent() throws Exception
	{
		final String NEW_LINE = "\n";
		
		//Datum darf nicht null sein
		if(dEvt.getDateOfEvent()==null)
		{
			throw new NullPointerException(IRessources.resPrint.getString("004"));
		}
		
		//Bei allen weiteren Feldern wird einfach ein leerer Text als String gesetzt, falls diese null sind,
		//da diese nicht zwingend erforderlich sind.
		if(dEvt.getPlace()==null)
		{
			dEvt.setPlace(IRessources.EMPTY);
		}
		if(dEvt.getTimeStart()==null)
		{
			dEvt.setTimeStart(IRessources.EMPTY);
		}
		if(dEvt.getTimeEnd()==null)
		{
			dEvt.setTimeEnd(IRessources.EMPTY);
		}
		if(dEvt.getDescription()==null)
		{
			dEvt.setDescription(IRessources.EMPTY);
		}
		
		//Format des Drucks. (Datum, Ort, von, bis, Eintrag-Text
		textToPrint = 
			IRessources.resPrint.getString("005")+
			new SimpleDateFormat(IRessources.DATE_FORMAT).format(dEvt.getDateOfEvent())+NEW_LINE+
			IRessources.resPrint.getString("006")+dEvt.getPlace()+NEW_LINE+
			IRessources.resPrint.getString("007")+dEvt.getTimeStart()+NEW_LINE+
			IRessources.resPrint.getString("008")+dEvt.getTimeEnd()+
			IRessources.resPrint.getString("009")+dEvt.getDescription();
	}
	
	/**
	 * Eigentlich Funktionalitaet des Drucks
	 * 
	 * @param pjob
	 * @param g
	 * @param s
	 */
	private void print(PrintJob pjob, Graphics g, String text) throws Exception
	{
		//Wenn g nicht das Interfice PrintGraphics implementiert -> Exception da nur eine PrintGraphic gedruckt
		//werden kann
		if (!(g instanceof PrintGraphics)) 
		{
			throw new IllegalArgumentException(IRessources.resPrint.getString("011"));
		}
		
		//Tabs durch je 15 Leerzeichen ersetzen, 15 weil Leerzeichen in der Schriftart nur ca. halb so breit
		//sind wie ein Zeichen. Die Darstellung ist dadurch uU etwas verzerrt und Tabs sind trennbar.
		// !----- Kann evtl. noch weiter ausgebaut werden ist allerdings nicht so kritisch -----!
		text = text.replaceAll("\t", TAB);
		
		LineNumberReader lnr = new LineNumberReader(new StringReader(text));
		
		//Variable zur Speicherung der jeweils naechsten Zeile
		String nextLine;
		
		//Variablen fuer max. Hoehe und Breite des Druckbereichs
		int pageHeight = pjob.getPageDimension().height;
		int pageWidth = pjob.getPageDimension().width;
		
		//Setzt die Schriftart und die Groesse
		Font font = new Font("Helvetica", Font.PLAIN, 9);
		g.setFont(font);
		
		//Speichert die Groessenangeben der Schriftart
		FontMetrics fm = g.getFontMetrics(font);
		
		//Variable zur Speicherung von Hoehe einer Zeile der Schriftart
		int fontHeight = fm.getHeight();
		
		//Variable zur Speicherung des Abstand zwischen der Basislinie der Schriftart und dem Boden der Zeichen
		int fontDescent = fm.getDescent();
		
		//Variable zur Speicherung der aktuellen Y Position des "Druckkopfs"
		int curHeight = 40;

		do 
		{
			//Speichern der neuen zu druckenden Zeile
			nextLine = lnr.readLine();

			//Prueft ob noch etwas zu drucken ist
			if (nextLine != null)
			{
				//Wenn die aktuelle Groesse + die Groesse der Schriftart groeser ist als die Hoehe der Seite
				//-30 -> Neue Seite, da es nicht mehr auf die aktuelle passen wuerde.
				if ((curHeight + fontHeight) > (pageHeight - 30)) 
				{			
					//Schliessen der Graphic und anschliessend laden der naechsten Seite in die Graphic
					g.dispose();
					g = pjob.getGraphics();

					//Setzt die Schriftart erneut fuer die Graphic der naechsten Seite
					if (g != null) 
					{
						g.setFont(font);
					}
					
					//Setzt die aktuelle Hoehe wieder auf den Startwert (40)
					curHeight = 40;
				}
				
				curHeight += fontHeight;
				
				//Wenn noch etwas zu drucken ist
				if (g != null) 
				{					
					//Wenn die aktuelle Zeile so lang ist, dass sie nicht auf die Seite passen wuerde
					while (fm.stringWidth(nextLine) > (pageWidth - 100)) 
					{						
						//temporaere Speicherun der naechsten Zeile
						String templine = nextLine;

						//Wenn die temp Zeile so lang ist, dass sie nicht auf die Seite passen wuerde
						while (fm.stringWidth(templine) > (pageWidth - 100)) 
						{
							//Wenn ein Leerzeichen gefunden wurde wird der Text bis zum Leerzeichen als
							//String zu drucken gesetzt
							if (templine.lastIndexOf(' ') != -1) 
							{
								templine = templine.substring(0, templine.lastIndexOf(' '));
							}
							//Wenn kein Leerzeichen (mehr) gefunden wurde und der Text trotzdem zu lang ist,
							//wird er so lange um ein Zeichen gekuertz bis er kurz genug ist.
							else 
							{
								templine = templine.substring(0, templine.length() - 1);
							}
						}
						//Druckt den temp String
						g.drawString(templine, 40, curHeight - fontDescent);
						
						//setzt als naechste Zeile den Rest der zu langen Zeile, der zuvor "abgeschnitten"
						//wurde
						nextLine = nextLine.substring(templine.length()+1);
						
						curHeight += fontHeight;
					}
					
					//Druckt eine Zeile
					g.drawString(nextLine, 40, curHeight - fontDescent);
				} 
			}
		} 
		while (nextLine != null);
	}
}  