package de.joerg.diary.shared;

import java.io.BufferedWriter;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintStream;
import java.text.SimpleDateFormat;
import java.util.Date;

/*
 *  SimpleDiary Version 2.0.0
    Copyright (C) 2010  Andreas Joerg (E-Mail: simpleDiary@freenet.de)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Klasse zum Loggen von Exceptions oder Strings auf das Filesystem.
 * 
 * @author Andreas Joerg
 */
public class DiaryLogger extends Thread
{
	private Exception ex = null;
	private String msg = null;
	private final String LOGFILE = System.getProperty("user.dir")+"\\"+"DiaryErrorLog.log";
	
	/**
	 * Exception Konstruktor
	 * 
	 * @param ex
	 */
	public DiaryLogger(Exception ex)
	{
		setEx(ex);
	}
	
	/**
	 * String Konstruktor
	 * 
	 * @param msg
	 */
	public DiaryLogger(String msg)
	{
		setMsg(msg);
	}
	
	//-----------Getter/Setter-----------------------------------------
	public Exception getEx() 
	{
		return ex;
	}
	public void setEx(Exception ex)
	{
		//Die Exception wird nur gesetzt, wenn kein String gespeichert ist, da der Logger
		//nur entweder eine Exception oder einen String loggen kann.
		if(msg==null)
		{
			this.ex = ex;
		}
	}

	public String getMsg() 
	{
		return msg;
	}
	public void setMsg(String msg) 
	{
		//Der String wird nur gesetzt, wenn keine Exception gespeichert ist, da der Logger
		//nur entweder eine Exception oder einen String loggen kann.
		if(ex==null)
		{
			this.msg = msg;
		}
	}
	//-----------------------------------------------------------------
	
	@Override
	public void run() 
	{
		if(ex!=null)
		{
			writeException(0);
		}
		else if(msg!=null)
		{
			writeMessage(0);
		}
	}
	
	/**
	 * Schreibt den Stacktrace der Exception in das ErrorLogFile.
	 * Bei IO Exception -> Retry (max. 5 mal)
	 * 
	 * @param trys
	 */
	private void writeException(int trys)
	{
		try 
		{
			SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm");
			ByteArrayOutputStream out = new ByteArrayOutputStream();
			PrintStream printStream = new PrintStream(out);
			ex.printStackTrace(printStream);
			
			FileWriter writer = new FileWriter(new File(LOGFILE),true);
			BufferedWriter buffWriter = new BufferedWriter(writer);

			buffWriter.write("["+sdf.format(new Date())+"]");
			buffWriter.newLine();
			if(ex.getMessage()!=null)
			{
				buffWriter.write(ex.getMessage());
				buffWriter.newLine();
			}
			buffWriter.write(out.toString());
			
			buffWriter.close();
		} 
		catch (IOException ex) 
		{
			if(trys<5)
			{
				writeException(trys+1);
			}
		}
		
	}
	
	/**
	 * Schreibt den String in der Membervariable msg in das ErrorLogFile.
	 * Bei IO Exception -> Retry (max. 5 mal)
	 * 
	 * @param trys
	 */
	private void writeMessage(int trys)
	{
		try 
		{
			SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm");
			FileWriter writer = new FileWriter(new File(LOGFILE),true);
			BufferedWriter buffWriter = new BufferedWriter(writer);
			
			buffWriter.write("["+sdf.format(new Date())+"]");
			buffWriter.newLine();
			buffWriter.write(getMsg());
			buffWriter.newLine();
			
			buffWriter.close();
			writer.close();
		} 
		catch (IOException ex) 
		{
			if(trys<5)
			{
				writeMessage(trys+1);
			}
		}
	}
}
