package de.joerg.diary.shared.dataSet;

import java.util.ArrayList;

import de.joerg.diary.shared.IRessources;

/*
 *  SimpleDiary Version 2.0.0
    Copyright (C) 2010  Andreas Joerg (E-Mail: simpleDiary@freenet.de)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Beschreibt eine Liste von Werten mit einem jeweiligen Bezeichner.
 * 
 * @author Andreas Joerg
 */
public class DataSet 
{	
	//interne Liste fuer die einzelnen Werte
	private ArrayList<DataSetRow> rows = null;
	
	/**
	 * Default constructor
	 */
	public DataSet()
	{
		rows = new ArrayList<DataSetRow>();
	}

	//----Getter/Setter------------------------------
	public void setRows(ArrayList<DataSetRow> rows)
	{
		this.rows = rows;
	}
	public ArrayList<DataSetRow> getRows() 
	{
		return rows;
	}
	//-----------------------------------------------
	
	/**
	 * Liefert einen Wert anhand des uebergebenen Bezeichners.
	 */
	public Object getRowValueByName(String attributeName) throws IllegalArgumentException
	{
		for(int x = 0;x<rows.size();x++)
		{
			if(rows.get(x).getAttributeName().equals(attributeName))
			{
				return rows.get(x).getAttribute();
			}
		}
		//nicht vorhanden
		throw new IllegalArgumentException(IRessources.resDataSet.getString("001")+attributeName+
				IRessources.resDataSet.getString("002"));
	}
	
	/**
	 * Gibt true zurueck, wenn das DataSet ein Attribut mit dem entsprechenden Namen hat. Wenn nicht false.
	 * 
	 * @param attributeName
	 * @return true/false
	 */
	public boolean contains(String attributeName)
	{
		for (DataSetRow row : rows)
		{
			if(row.getAttributeName().equals(attributeName))
			{
				return true;
			}
		}
		
		return false;
	}
	
	/**
	 * Liefert einen String Wert anhand des uebergebenen Bezeichners.
	 * 
	 * @param attributeName
	 * @return string
	 */
	public String getString(String attributeName)
	{
		return (String) getRowValueByName(attributeName);
	}
	
	/**
	 * Liefert ein DataSet im DataSet anhand des uebergebenen DataSet Namens.
	 * 
	 * @param attributeName
	 * @return dataSet
	 */
	public DataSet getDataSet(String attributeName)
	{
		return (DataSet) getRowValueByName(attributeName);
	}
	
	/**
	 * Liefert einen Integer im DataSet anhand des uebergebenen AttributNamens.
	 * 
	 * @param attributeName
	 * @return Integer
	 */
	public Integer getInteger(String attributeName)
	{
		return (Integer) getRowValueByName(attributeName);
	}
	
	/**
	 * Liefert einen Boolean im DataSet anhand des uebergebenen AttributNamens.
	 * 
	 * @param attributeName
	 * @return Boolean
	 */
	public Boolean getBoolean(String attributeName)
	{
		return (Boolean) getRowValueByName(attributeName);
	}
	
	/**
	 * Fuegt einen Wert mit zugehoerigen Bezeichner hinzu.
	 * 
	 * @param attributeName
	 * @param attribute
	 * @throws DataSetRowAlreadyExistsException
	 */
	public void addRow(String attributeName, Object attribute) throws DataSetRowAlreadyExistsException
	{
		for (DataSetRow row : rows) 
		{
			if(row.getAttributeName().equals(attributeName))
			{
				throw new DataSetRowAlreadyExistsException(attributeName);
			}
		}
		rows.add(new DataSetRow(attributeName, attribute));
	}
	
	/**
	 * Fuegt dem DataSet ein Feld hinzu mit dem uebergebenem Bezeichner und setzt den Wert auf null.
	 * 
	 * @param attributeName
	 * @throws DataSetRowAlreadyExistsException
	 */
	public void addRow(String attributeName) throws DataSetRowAlreadyExistsException
	{
		addRow(attributeName, null);
	}
	
	/**
	 * Setzt das Attribut fuer den Wert mit dem uebergebenem Bezeichner.
	 * 
	 * @param attributeName
	 * @param attribute
	 * @throws IllegalArgumentException
	 */
	public void setValueOfRow(String attributeName, Object attribute) throws IllegalArgumentException
	{
		for (DataSetRow row : rows) 
		{
			if(row.getAttributeName().equals(attributeName))
			{
				row.setAttribute(attribute);
				return;
			}
		}
		throw new IllegalArgumentException(IRessources.resDataSet.getString("001")+attributeName+
				IRessources.resDataSet.getString("002"));
	}
	
	/**
	 * Loescht einen Wert anhand des uebergebenen Bezeichners.
	 * 
	 * @param attributeName
	 * @throws IllegalArgumentException
	 */
	public void deleteRowByName(String attributeName) throws IllegalArgumentException
	{
		for (DataSetRow row : rows) 
		{
			if(row.getAttributeName().equals(attributeName))
			{
				rows.remove(row);
				return;
			}
		}
		throw new IllegalArgumentException(IRessources.resDataSet.getString("001")+attributeName+
				IRessources.resDataSet.getString("002"));
	}

	/**
	 * 
	 * @return Anzahl der gespeicherten Werte.
	 */
	public int getRowsCount() 
	{
		return rows.size();
	}
	
	/**
	 * 
	 * @param index
	 * @return Wert an Stelle x in der DataSetRow-Liste.
	 * @throws IllegalArgumentException
	 */
	public Object getRowValueByIndex(int index) throws IllegalArgumentException
	{
		if(index<0||index>=rows.size())
		{
			throw new IllegalArgumentException(IRessources.resDataSet.getString("003")+index+
					IRessources.resDataSet.getString("003"));
		}
		
		return rows.get(index).getAttribute();
	}
}
