package de.joerg.diary.shared.models;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;

import de.joerg.diary.shared.IRessources;

/*
 *  SimpleDiary Version 2.0.0
    Copyright (C) 2010  Andreas Joerg (E-Mail: simpleDiary@freenet.de)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Klasse fuer einen Eintrag im Diary
 * 
 * @author Andreas Joerg
 */
public class DiaryEvent
{
	private User user = null;
	private Date dateOfEvent = null;
	private String timeStart = null;
	private String timeEnd = null;
	private String place = null;
	private String description = null;
	private ArrayList<DiaryEvent> subEvents = null;
	private ArrayList<SDPicture> pictures = null;
	private String[] keywords = new String[3];
	
	/**
	 * Default Konstruktor
	 */
	public DiaryEvent()
	{
		
	}
	
	/**
	 * erweiterter Konstruktor mit Pflichtangaben
	 * 
	 * @param user
	 * @param dateOfEvent
	 */
	public DiaryEvent(User user, Date dateOfEvent)
	{
		setUser(user);
		setDateOfEvent(dateOfEvent);
	}
	
	/**
	 * erweiterter Konstruktor mit Pflichtangaben + Zusatzangaben
	 * 
	 * @param user
	 * @param dateOfEvent
	 * @param timeStart
	 * @param timeEnd
	 * @param place
	 * @param description
	 */
	public DiaryEvent(User user, Date dateOfEvent, String timeStart, String timeEnd, String place, String description)
	{
		setUser(user);
		setDateOfEvent(dateOfEvent);
		setTimeStart(timeStart);
		setTimeEnd(timeEnd);
		setPlace(place);
		setDescription(description);
	}
	
	//----------------------------------------------------------------------
	//   Getter/Setter
	//----------------------------------------------------------------------
	public String getDescription()
	{
		return description;
	}
	public void setDescription(String description)
	{
		this.description = description;
	}
	
	public User getUser() 
	{
		return user;
	}
	public void setUser(User user) 
	{
		this.user = user;
	}
	
	public Date getDateOfEvent()
	{
		return dateOfEvent;
	}
	public void setDateOfEvent(Date dateOfEvent) 
	{
		this.dateOfEvent = dateOfEvent;
	}

	public String getTimeStart() {
		
		return timeStart;
	}
	public void setTimeStart(String timeStart)
	{
		this.timeStart = timeStart;
	}

	public String getTimeEnd() 
	{
		return timeEnd;
	}
	public void setTimeEnd(String timeEnd) 
	{
		this.timeEnd = timeEnd;
	}

	public String getPlace()
	{
		return place;
	}
	public void setPlace(String place) 
	{
		this.place = place;
	}
	
	public ArrayList<DiaryEvent> getSubEvents()
	{
		return subEvents;
	}
	public void setSubEvents(ArrayList<DiaryEvent> subEvents)
	{
		this.subEvents = subEvents;
	}
	
	public void setPictures(ArrayList<SDPicture> pictures) 
	{
		this.pictures = pictures;
	}
	public ArrayList<SDPicture> getPictures() 
	{
		return pictures;
	}
	
	public void setKeywords(String[] keywords) 
	{
		this.keywords = keywords;
	}
	public String[] getKeywords()
	{
		return keywords;
	}
	//--------------------------------------------------------------------

	/**
	 * Fuegt ein SubEvent zum aktuellen Event hinzu
	 * 
	 * @param subEvent
	 */
	public void addSubEvent(DiaryEvent subEvent)
	{
		if(subEvents == null)
		{
			subEvents = new ArrayList<DiaryEvent>();
		}
		subEvents.add(subEvent);
	}
	
	/**
	 * Loescht ein Subevent (falls vorhanden) aus dem aktuellen Event
	 * 
	 * @param subEvent
	 */
	public void deleteSubEvent(DiaryEvent subEvent)
	{
		subEvents.remove(subEvent);
	}
	
	/**
	 * Gibt ein DiaryEvent an Stelle (index) zurueck
	 * 
	 * @param index
	 * @return getSubEvents().get(index)
	 */
	public DiaryEvent getSubEvent(int index)
	{
		return getSubEvents().get(index);
	}
	
	/**
	 * Fuegt ein Bild zum Eintrag hinzu
	 * 
	 * @param pictureData
	 */
	public void addPicture(SDPicture pictureData)
	{
		if(pictures==null)
		{
			pictures = new ArrayList<SDPicture>();
		}
		pictures.add(pictureData);
	}
	
	/**
	 * Loescht das Bild an der Stelle die uebergeben wurde.
	 * 
	 * @param index
	 */
	public void removePicture(int index)
	{
		pictures.remove(index);
	}
	
	/**
	 * Entfernt alle Untereintraege.
	 */
	public void doRemoveAllSubEvents()
	{
		if(subEvents!=null)
		{
			subEvents.clear();
		}
	}
	
	/**
	 * Prueft zwei Eintraege auf Gleichheit. Allerdings nur auf inhaltliche Gleichheit.
	 * Untereintraege zB werden nicht beachtet.
	 * 
	 * #########################################################################################################
	 * ACHTUNG: NICHT VERWECHSELN MIT isEqual!!!!!!!
	 * 			Diese Methode ist dazu gedacht um Dubletten zu erkennen.
	 * 			Dabei spielen zB auch Sachen eine Rolle wie, wenn einer der beiden Eintraege keine Beschreibung
	 * 			hat gelten Sie als ungleich, da es nicht eindeutig ist!!!
	 * #########################################################################################################
	 * 
	 * @param evt
	 * @return true/false
	 */
	public boolean isSame(DiaryEvent evt)
	{
		//hab keine Stichwoerter
		if(keywords == null)
		{
			//anderer hat aber welche
			if(evt.getKeywords()!=null)
			{
				return false;
			}
		}
		//hab Stichwoerter
		else
		{	
			//anderer aber nicht
			if(evt.getKeywords()==null)
			{
				return false;
			}

			//unterschiedlich viele Stichwoerter
			if(keywords.length!=evt.getKeywords().length)
			{
				return false;
			}
			
			//Pruefung ob eines der Stichwoerter unterschiedlich ist.
			for(int x=0;x<keywords.length;x++)
			{
				//aktuelles Stichwort ist null
				if(keywords[x] == null)
				{
					//beim andern aber ned
					if(evt.getKeywords()[x] != null)
					{
						return false;
					}
				}
				else
				{
					if(keywords[x].equals(evt.getKeywords()[x]) == false)
					{
						return false;
					}
				}
			}
		}

		//hab keine Bilder
		if(getPictures()==null || getPictures().size()<1)
		{
			//Andere hat aber welche
			if(evt.getPictures()!=null && evt.getPictures().size()>0)
			{
				return false;
			}
		}
		//hab Bilder
		else
		{
			//Andere hat aber keine
			if(evt.getPictures()==null || evt.getPictures().size()<1)
			{
				return false;
			}
			
			//Beide haben Bilder
			//gleich viele
			if(getPictures().size()==evt.getPictures().size())
			{
				for(int x=0;x<getPictures().size();x++)
				{
					//Bild ist ein anderes.
					if(!getPictures().get(x).equals(evt.getPictures().get(x)))
					{
						return false;
					}
				}
			}
			//Anzahl Bilder ist unterschiedlich
			else
			{
				return false;
			}
		}
		
		//Wenn Ort leer ist und mein Ort nicht -> ungleich
		if(evt.getPlace()==null||evt.getPlace().isEmpty())
		{
			if(place != null && place.isEmpty()==false )
			{
				return false;
			}
		}
		//Wenn Ort gesetzt ist und mein Ort nicht der gleiche ist -> ungleich
		else
		{
			//Wenn
			if(evt.getPlace().equals(place) == false)
			{
				return false;
			}
		}
		
		//Wenn die Beschreibung null ist oder leer
		//-> nicht gleich, da es nicht eindeutig ist ohne Beschreibung
		//Bsp.: Zwei Eintraege werden erstellt selbes Datum, selbe Uhrzeiten. Beschreibung ist leer.
		//		Es kann sein, dass der User zu diesem Zeitraum absichtlich zwei Eintraege fuer 2 unterschied-
		//		liche Ereignisse erstellen will und diese nur vorerst leer laesst um sie spaeter nachzutragen.
		if(description==null||description.isEmpty()||evt.getDescription()==null||evt.getDescription().isEmpty())
		{
			return false;
		}
		else
		{
			//Wenn die Beschreibungen gleich sind
			if(description.equals(evt.getDescription())&&!description.isEmpty())
			{
				//Wenn beide Eintraege keine Uhrzeiten definiert haben
				//-> Gleich
				if((timeStart == null||timeStart.isEmpty())
						&&(evt.getTimeStart()==null||evt.getTimeStart().isEmpty())
						&&(timeEnd==null||timeEnd.isEmpty())
						&&(evt.getTimeEnd()==null||evt.getTimeEnd().isEmpty()))
				{
					return true;
				}
				
				//Wenn eine Anfangszeit gesetzt wurde
				if( timeStart!=null && timeStart.isEmpty()==false)
				{
					//Wenn eine Endzeit gesetzt wurde
					if( timeEnd!=null && timeEnd.isEmpty()==false )
					{
						//Wenn Anfangs- und Endzeit gleich sind -> Gleich
						if(timeStart.equals( evt.getTimeStart() )&&
								timeEnd.equals(evt.getTimeEnd()))
						{
							return true;
						}
					}
					//Es wurde keine Endzeit gesetzt
					else
					{
						//Es wurde bei beiden Eintraegen keine Endzeit gesetzt und die Startzeit ist gleich
						//-> Gleich
						if((evt.getTimeEnd()==null||evt.getTimeEnd().isEmpty())&&
								timeStart.equals(evt.getTimeStart()))
						{
							return true;
						}
					}
				}
				//Es wurde keine Anfangszeit gesetzt
				else
				{
					//Es wurde bei beiden Eintraegen keine Anfangszeit gesetzt
					//+Es wurde eine Endzeit gesetzt
					//+Es wurden die gleichen Endzeiten gesetzt -> Gleich
					if((evt.getTimeStart()==null||evt.getTimeStart().isEmpty())
							&&(timeEnd!=null&&!timeEnd.isEmpty())
							&&(timeEnd.equals(evt.getTimeEnd())))
					{
						return true;
					}
				}
			}
		}
		return false;
	}

	/**
	 * Prueft zwei Eintraege auf inhaltliche Gleichheit.
	 * Untereintraege werden nicht geprueft (Fuer Pruefung mit Untereintraegen Methode equals benutzen).
	 * 
	 * @param evt
	 * @return true/false
	 */
	public boolean isEqual(DiaryEvent evt)
	{
		//Wenn Referenzen gleich sind
		if(this == evt)
		{
			return true;
		}

		if(evt == null)
		{
			return false;
		}
		
		//Ueberpruefung ob eines der beiden Objecte null ist
		if(getDateOfEvent()==null||evt.getDateOfEvent()==null)
		{
			//Wenn nicht beide Objecte null sind -> return false
			if(!(getDateOfEvent()==null&&evt.getDateOfEvent()==null))
			{
				return false;
			}
		}
		else
		{
			SimpleDateFormat sdf = new SimpleDateFormat(IRessources.DATE_FORMAT);
			if(! sdf.format(evt.getDateOfEvent()).equals(sdf.format(getDateOfEvent())))
			{
				return false;
			}
		}
		
		//Ueberpruefung ob eines der beiden Objecte null ist
		if(user.getUserPW()==null||evt.getUser().getUserPW()==null)
		{
			//Wenn nicht beide Objecte null sind -> return false
			if(!(user.getUserPW()==null&&evt.getUser().getUserPW()==null))
			{
				return false;
			}
		}
		else
		{
			if(!evt.getUser().getUserPW().equals(user.getUserPW()))
			{
				return false;
			}
		}
		
		//Ueberpruefung ob eines der beiden Objecte null ist
		if(getPlace()==null||evt.getPlace()==null)
		{
			//Wenn nicht beide Objecte null sind -> return false
			if(!(getPlace()==null&&evt.getPlace()==null))
			{
				return false;
			}
		}
		else
		{
			if(!evt.getPlace().equals(getPlace()))
			{
				return false;
			}
		}
		
		//Ueberpruefung ob eines der beiden Objecte null ist
		if(getTimeEnd()==null||evt.getTimeEnd()==null)
		{
			//Wenn nicht beide Objecte null sind -> return false
			if(!(getTimeEnd()==null&&evt.getTimeEnd()==null))
			{
				return false;
			}
		}
		else
		{
			if(!evt.getTimeEnd().equals(getTimeEnd()))
			{
				return false;
			}
		}
		
		//Ueberpruefung ob eines der beiden Objecte null ist
		if(getTimeStart()==null||evt.getTimeStart()==null)
		{
			//Wenn nicht beide Objecte null sind -> return false
			if(!(getTimeStart()==null&&evt.getTimeStart()==null))
			{
				return false;
			}
		}
		else
		{
			if(!evt.getTimeStart().equals(getTimeStart()))
			{
				return false;
			}
		}
		
		//Ueberpruefung ob eines der beiden Objecte null ist
		if(getDescription()==null||evt.getDescription()==null)
		{
			//Wenn nicht beide Objecte null sind -> return false
			if(!(getDescription()==null&&evt.getDescription()==null))
			{
				return false;
			}
		}
		else
		{
			if(!(evt.getDescription().equals(getDescription())))
			{
				return false;
			}
		}
		
		//Keine Stichwoerter da
		if(keywords == null)
		{
			//Anderer hat aber Stichwoerter
			if(evt.getKeywords()!=null)
			{
				return false;
			}
		}
		//Stichwoerter da
		else
		{
			//Andere hat aber keine
			if(evt.getKeywords()==null)
			{
				return false;
			}
			
			//Unterschiedlich viele Stichwoerter
			if(keywords.length!=evt.getKeywords().length)
			{
				return false;
			}
			
			for(int x=0;x<keywords.length;x++)
			{
				//aktuells Stichwort ist null
				if(keywords[x] == null)
				{
					//beim andern aber ned
					if(evt.getKeywords()[x] != null)
					{
						return false;
					}
				}
				else
				{
					//Unterschiedliches Stichwort
					if(keywords[x].equals(evt.getKeywords()[x]) == false)
					{
						return false;
					}
				}
			}
		}
		
		//keine Bilder vorhanden
		if(pictures==null)
		{
			//anderer hat aber Bilder
			if(evt.getPictures()!=null)
			{
				return false;
			}
		}
		//Bilder vorhanden
		else
		{
			//anderer hat aber keine Bilder
			if(evt.getPictures()==null)
			{
				return false;
			}
			//Anzahl der Bilder ist unterschiedlich
			if(pictures.size()!=evt.getPictures().size())
			{
				return false;
			}
			
			for (int x = 0; x<pictures.size(); x++)
			{
				//Das aktuelle Bild ist ein anderes
				if(!(pictures.get(x).equals(evt.getPictures().get(x))))
				{
					return false;
				}
			}
		}
		
		return true;
	}
	
	@Override
	/**
	 * Ueberschriebene Methode equals() zur Pruefung zweier DiaryEvents auf Gleichheit
	 * 
	 * @param obj
	 */
	public boolean equals(Object obj)
	{
		//Wenn Referenzen gleich sind
		if(this == obj)
		{
			return true;
		}

		if(obj == null)
		{
			return false;
		}
		
		if(!obj.getClass().equals(this.getClass()))
		{
			return false;
		}
		
		DiaryEvent evt = (DiaryEvent)obj;
		
		//Ueberpruefung ob eines der beiden Objecte null ist
		if(getSubEvents() == null||evt.getSubEvents()==null)
		{
			//Wenn nicht beide Objecte null sind -> return false
			if(!(getSubEvents() == null&&evt.getSubEvents()==null))
			{
				return false;
			}
		}
		
		//Ueberpruefung ob eines der beiden Objecte null ist
		if(getDateOfEvent()==null||evt.getDateOfEvent()==null)
		{
			//Wenn nicht beide Objecte null sind -> return false
			if(!(getDateOfEvent()==null&&evt.getDateOfEvent()==null))
			{
				return false;
			}
		}
		else
		{
			SimpleDateFormat sdf = new SimpleDateFormat(IRessources.DATE_FORMAT);
			if(! sdf.format(evt.getDateOfEvent()).equals(sdf.format(getDateOfEvent())))
			{
				return false;
			}
		}
		
		//Ueberpruefung ob eines der beiden Objecte null ist
		if(user.getUserPW()==null||evt.getUser().getUserPW()==null)
		{
			//Wenn nicht beide Objecte null sind -> return false
			if(!(user.getUserPW()==null&&evt.getUser().getUserPW()==null))
			{
				return false;
			}
		}
		else
		{
			if(!evt.getUser().getUserPW().equals(user.getUserPW()))
			{
				return false;
			}
		}
		
		//Ueberpruefung ob eines der beiden Objecte null ist
		if(getPlace()==null||evt.getPlace()==null)
		{
			//Wenn nicht beide Objecte null sind -> return false
			if(!(getPlace()==null&&evt.getPlace()==null))
			{
				return false;
			}
		}
		else
		{
			if(!evt.getPlace().equals(getPlace()))
			{
				return false;
			}
		}
		
		//Ueberpruefung ob eines der beiden Objecte null ist
		if(getTimeEnd()==null||evt.getTimeEnd()==null)
		{
			//Wenn nicht beide Objecte null sind -> return false
			if(!(getTimeEnd()==null&&evt.getTimeEnd()==null))
			{
				return false;
			}
		}
		else
		{
			if(!evt.getTimeEnd().equals(getTimeEnd()))
			{
				return false;
			}
		}
		
		//Ueberpruefung ob eines der beiden Objecte null ist
		if(getTimeStart()==null||evt.getTimeStart()==null)
		{
			//Wenn nicht beide Objecte null sind -> return false
			if(!(getTimeStart()==null&&evt.getTimeStart()==null))
			{
				return false;
			}
		}
		else
		{
			if(!evt.getTimeStart().equals(getTimeStart()))
			{
				return false;
			}
		}
		
		//Ueberpruefung ob eines der beiden Objecte null ist
		if(getDescription()==null||evt.getDescription()==null)
		{
			//Wenn nicht beide Objecte null sind -> return false
			if(!(getDescription()==null&&evt.getDescription()==null))
			{
				return false;
			}
		}
		else
		{
			if(!(evt.getDescription().equals(getDescription())))
			{
				return false;
			}
		}
		
		//Keine Stichwoerter da
		if(keywords == null)
		{
			//Anderer hat aber Stichwoerter
			if(evt.getKeywords()!=null)
			{
				return false;
			}
		}
		//Stichwoerter da
		else
		{
			//Andere hat aber keine
			if(evt.getKeywords()==null)
			{
				return false;
			}
			
			//Unterschiedlich viele Stichwoerter
			if(keywords.length!=evt.getKeywords().length)
			{
				return false;
			}
			
			for(int x=0;x<keywords.length;x++)
			{
				//aktuells Stichwort ist null
				if(keywords[x] == null)
				{
					//beim andern aber ned
					if(evt.getKeywords()[x] != null)
					{
						return false;
					}
				}
				else
				{
					//Unterschiedliches Stichwort
					if(keywords[x].equals(evt.getKeywords()[x]) == false)
					{
						return false;
					}
				}
			}
		}
		
		//keine Bilder vorhanden
		if(pictures==null)
		{
			//anderer hat aber Bilder
			if(evt.getPictures()!=null)
			{
				return false;
			}
		}
		//Bilder vorhanden
		else
		{
			//anderer hat aber keine Bilder
			if(evt.getPictures()==null)
			{
				return false;
			}
			//Anzahl der Bilder ist unterschiedlich
			if(pictures.size()!=evt.getPictures().size())
			{
				return false;
			}
			
			for (int x = 0; x<pictures.size(); x++)
			{
				//Das aktuelle Bild ist ein anderes
				if(!(pictures.get(x).equals(evt.getPictures().get(x))))
				{
					return false;
				}
			}
		}
		
		//Ueberpruefung ob beide DiaryEvents keine SubEvents (=null) haben, wenn ja -> true
		if(getSubEvents()==null && evt.getSubEvents()==null)
		{
			return true;
		}
		
		//Wenn die Anzahl der SubEvents der DiaryEvents nicht gleich ist -> false
		if(getSubEvents().size()!=evt.getSubEvents().size())
		{
			return false;
		}
		//Vergleiche jedes SubEvent der beiden DiaryEvents. Bei Ungleichheit -> false
		for(int x = 0; x<getSubEvents().size();x++)
		{
			if(!getSubEvent(x).equals(evt.getSubEvent(x)))
			{
				return false;
			}
		}
		return true;
	}

	/**
	 * @return exakte Kopie des Objekts
	 */
	public DiaryEvent getCopy()
	{
		DiaryEvent newEvt = new DiaryEvent();
		User newUser = new User();
		
		//User kopieren falls vorhanden
		if(user!=null)
		{
			newUser.setSavePath(user.getSavePath());
			newUser.setUserID(user.getUserID());
			newUser.setUserName(user.getUserName());
			newUser.setUserPW(user.getUserPW());
		}

		//Datum, Beschreibung, Ort, Zeiten kopieren
		newEvt.setDateOfEvent(dateOfEvent);
		newEvt.setDescription(description);
		newEvt.setPlace(place);
		newEvt.setTimeEnd(timeEnd);
		newEvt.setTimeStart(timeStart);
		
		//Bilder kopieren, falls vorhanden
		if(pictures!=null)
		{
			ArrayList<SDPicture> newPictures = new ArrayList<SDPicture>();
			
			for (SDPicture img : pictures)
			{
				SDPicture imgNew = new SDPicture();
				imgNew.setPictureData(img.getPictureData());
				imgNew.setPictureDesc(img.getPictureDesc());
				newPictures.add(imgNew);
			}
			
			newEvt.setPictures(newPictures);
		}
		
		//Stichwoerter kopieren, falls vorhanden
		if(keywords != null)
		{
			String[] newKeywords = new String[keywords.length];
			
			for (int x=0;x<keywords.length;x++)
			{
				newKeywords[x] = keywords[x];
			}
			
			newEvt.setKeywords(newKeywords);
		}
		
		//Untereintraege kopieren falls vorhanden
		if(subEvents!=null)
		{
			ArrayList<DiaryEvent> newEntrys = new ArrayList<DiaryEvent>();
			
			for (DiaryEvent diaryEvent : subEvents)
			{
				//rekursiver Aufruf fuer Untereintrag
				newEntrys.add(diaryEvent.getCopy());
			}
			
			newEvt.setSubEvents(newEntrys);
		}
		
		newEvt.setUser(newUser);
		
		return newEvt;
	}
}
