package de.joerg.diary.shared.models;

import de.joerg.diary.shared.IRessources;

/*
 *  SimpleDiary Version 2.0.0
    Copyright (C) 2010  Andreas Joerg (E-Mail: simpleDiary@freenet.de)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Klasse fuer einen UserAccount
 * 
 * @author Andreas Joerg
 */
public class User 
{
	private int userID = -1;
	private String userName = null;
	private String userPW = null;
	private String savePath = System.getProperty("user.dir")+IRessources.SAVE_PATH;
	
	/**
	 * Default Constructor
	 */
	public User()
	{
		
	}
	
	/**
	 * Erweiterter Konstruktor
	 * 
	 * @param userName
	 * @param userPW
	 */
	public User(String userName, String userPW)
	{
		setUserName(userName);
		setUserPW(userPW);
	}
	
	/**
	 * erweiterter Konstruktor mit Uebergabe der ID
	 * 
	 * @param userID
	 * @param userName
	 * @param userPW
	 */
	public User(int userID, String userName, String userPW)
	{
		setUserID(userID);
		setUserName(userName);
		setUserPW(userPW);
	}
	
	/**
	 * copy constructor
	 * 
	 * @param user
	 */
	public User(User user)
	{
		user.setUserID(userID);
		user.setSavePath(savePath);
		user.setUserName(userName);
		user.setUserPW(userPW);
	}

	//----------------------------------------------------------------------------------
	//                 Getter/Setter
	//----------------------------------------------------------------------------------
	public void setUserPW(String userPW)
	{
		this.userPW = userPW;
	}
	public String getUserPW() 
	{
		return userPW;
	}
	
	public void setUserName(String userName) 
	{
		this.userName = userName;
	}
	public String getUserName() 
	{
		return userName;
	}
	
	public void setUserID(int userID)
	{
		this.userID = userID;
	}
	public int getUserID() 
	{
		return userID;
	}
	
	public void setSavePath(String savePath) 
	{
		this.savePath = savePath;
	}
	public String getSavePath() 
	{
		return savePath;
	}
	//------------------------------------------------------------------------------------

	@Override
	/**
	 * Ueberschriebene Methode equals() zur Ueberpruefung zweier User auf Gleichheit.
	 * ACHTUNG!: Die ID und SavePath werden nicht auf Gleichheit geprueft!
	 * Geprueft wird nur UserName und Passwort!
	 * 
	 * @param obj
	 * @return true/false
	 */
	public boolean equals(Object obj) 
	{
		if(obj==null)
		{
			return false;
		}
		if(!obj.getClass().equals(this.getClass()))
		{
			return false;
		}
		
		if(getUserName()==null||((User) obj).getUserName()==null)
		{
			if(!(getUserName()==null&&((User) obj).getUserName()==null))
			{
				return false;
			}
		}
		else
		{
			if(!getUserName().equals(((User) obj).getUserName()))
			{
				return false;
			}
		}
		
		if(getUserPW()==null||((User)obj).getUserPW()==null)
		{
			if(!(getUserPW()==null&&((User)obj).getUserPW()==null))
			{
				return false;
			}
		}
		else
		{
			if(!getUserPW().equals(((User)obj).getUserPW()))
			{
				return false;
			}
		}
		return true;
	}
	
	
}
